#include "stdafx.h"
#include "common.h"


namespace bec {

  std::string fmt(const char *fmt, ...)
  {
    va_list args;
    char *tmp;
    std::string ret;

    va_start(args, fmt);
    tmp= g_strdup_vprintf(fmt, args);
    va_end(args);

    ret= tmp;
    g_free(tmp);

    return ret;
  }

  std::string fmttime(time_t t, const char *fmt)
  {
    char date[100];
  #ifdef _WIN32
    errno_t err;
  #else
    int err;
  #endif
    struct tm newtime;


    if (t == 0)
      time(&t);

  #ifdef _WIN32
    err= localtime_s(&newtime, &t);
  #else
    localtime_r(&t, &newtime);
    err= 0;
  #endif

    if (!err)
      strftime(date, sizeof(date), fmt, &newtime);
    else
      date[0]= 0;

    return date;
  }

  
  std::string escape_sql_string(const std::string &text)
  {
    std::string escaped;
    std::string::size_type p, s, len= text.length();

    // escapes ' " \n \r

    p= 0;
    s= 0;
    while (p < len)
    {
      switch (text[p])
      {
      case '\'':
      case '"':
        if (p > s)
          escaped.append(text.substr(s, p-s));
        escaped.append("\\");
        escaped.append(text.substr(p, 1));
        s= p+1;
        break;
      case '\n':
        if (p > s)
          escaped.append(text.substr(s, p-s));
        escaped.append("\\n");
        s= p+1;
        break;
      case '\r':
        if (p > s)
          escaped.append(text.substr(s, p-s));
        escaped.append("\\r");
        s= p+1;
        break;
      }
      p++;
    }
    if (p > s)
      escaped.append(text.substr(s));

    return escaped;
  }

  
  std::string replace_string(const std::string &s,
                                    const std::string &from,
                                    const std::string &to)
  {
    std::string::size_type p;
    std::string ss, res;

    ss= s;
    p= ss.find(from);
    while (p != std::string::npos)
    {
      res+= ss.substr(0, p) + to;
      ss= ss.substr(p+from.size());
      p= ss.find(from);
    }
    res+= ss;

    return res;
  }


  std::string append_extension_if_needed(const std::string &path,
                                         const std::string &ext)
  {
    if (!has_suffix(path, ext))
      return path+ext;
    return path;
  }


  std::string make_path(const std::string &prefix, const std::string &file)
  {
    if (prefix.empty())
      return file;

    if (prefix[prefix.size()-1] == '/' || prefix[prefix.size()-1] == '\\')
      return prefix+file;
    return prefix+G_DIR_SEPARATOR+file;
  }

  /**
   * Removes all unnecessary path separators as well as "./" combinations.
   * If there is a parent-dir entry (../) then this as well as the directly prefacing
   * dir entry is removed.
   */
  std::string normalize_path(const std::string path)
  {
    // First convert all separators to the one that is used on the platform (no mix)
    // and ease so at the same time further processing here.
    std::string result;
    std::string separator(1, G_DIR_SEPARATOR);

    result= replace_string(path, "\\", separator);
    result= replace_string(result, "/", separator);

    std::string double_separator = separator + separator;
    while (result.find(double_separator) != std::string::npos)
      result= replace_string(result, double_separator, separator);

    // Sanity check. Return *after* we have converted the slashs. This is part of the normalization.
    if (result.size() < 2)
      return result;

    std::vector<std::string> parts= split_string(result, separator);

    // Construct result backwards while examining the path parts.
    result= "";
    int pending_count= 0;
    for (int i= parts.size() - 1; i >= 0; i--)
    {
      if (parts[i].compare(".") == 0)
        // References to the current directory can be removed without further change.
        continue;

      if (parts[i].compare("..") == 0)
      {
        // An entry that points back to the parent dir.
        // Ignore this and keep track for later removal of the parent dir.
        pending_count++;
      }
      else
        if (pending_count > 0)
        {
          // If this is a normal dir entry and we have pending parent-dir redirections
          // then go one step up by removing (ignoring) this entry.
          pending_count--;
        }
        else
          result = separator + parts[i] + result;
    }

    // Don't return the leading separator.
    return result.substr(1);
  }

  std::vector<std::string> split_string(const std::string &s, const std::string &sep, int count)
  {
    std::vector<std::string> parts;
    std::string ss= s;

    std::string::size_type p;

    if (s.empty())
      return parts;

    if (count == 0)
      count= -1;

    p= ss.find(sep);
    while (!ss.empty() && p != std::string::npos && (count<0 || count>0))
    {
      parts.push_back(ss.substr(0, p));
      ss= ss.substr(p+sep.size());

      --count;
      p= ss.find(sep);
    }
    parts.push_back(ss);

    return parts;
  }



};
