#ifndef _BE_COMMON_H_
#define _BE_COMMON_H_

#include <cstring>
#include <vector>
#include <glib.h>

#ifndef _WIN32
#include <stdexcept>
#include <stdarg.h>
#endif

#include <string.h>

#include "grt_backend_public_interface.h"

#define _(s) s

namespace bec {

  std::string MYSQLGRTBACKEND_PUBLIC_FUNC fmt(const char *fmt, ...);

#define DATETIME_FMT "%Y-%m-%d %H:%M"
  std::string MYSQLGRTBACKEND_PUBLIC_FUNC fmttime(time_t t= 0, const char *fmt= "%b %d, %Y");

  std::string MYSQLGRTBACKEND_PUBLIC_FUNC escape_sql_string(const std::string &text);
  
  std::string MYSQLGRTBACKEND_PUBLIC_FUNC replace_string(const std::string &s,
                            const std::string &from,
                            const std::string &to);
  
  std::string MYSQLGRTBACKEND_PUBLIC_FUNC append_extension_if_needed(const std::string &path,
                                                              const std::string &ext);

  std::string MYSQLGRTBACKEND_PUBLIC_FUNC normalize_path(const std::string path);

  inline bool has_prefix(const std::string &str, const std::string &pref)
  {
    if (strncmp(str.c_str(), pref.c_str(), pref.length())==0)
      return true;
    return false;
  }


  inline bool has_suffix(const std::string &str, const std::string &suf)
  {
    if (suf.length() < str.length() && strncmp(str.c_str()+str.length()-suf.length(), suf.c_str(), suf.length())==0)
      return true;
    return false;
  }


  std::string MYSQLGRTBACKEND_PUBLIC_FUNC make_path(const std::string &prefix, const std::string &file);

  inline std::string pathlist_append(const std::string &l, const std::string &s)
  {
    if (l.empty())
      return s;
    return l+G_SEARCHPATH_SEPARATOR+s;
  }

  inline std::string pathlist_prepend(const std::string &l, const std::string &s)
  {
    if (l.empty())
      return s;
    return s+G_SEARCHPATH_SEPARATOR+l;
  }

  std::vector<std::string> MYSQLGRTBACKEND_PUBLIC_FUNC split_string(const std::string &s, const std::string &sep, int count=0);

  //! \brief RAII style GMutex locker/unlocker  
  class GMutexLock
  {
      GMutex    *mutex;
    public:
      GMutexLock(GMutex* mtx) : mutex(mtx)  {   g_mutex_lock(mtx);     }
      ~GMutexLock()                         {   g_mutex_unlock(mutex); }
  };

  //! \brief RAII style GStaticMutex locker/unlocker  
  class GStaticMutexLock
  {
      GStaticMutex    &mutex;
    public:
      GStaticMutexLock(GStaticMutex& mtx) : mutex(mtx)  {   g_static_mutex_lock(&mutex);   }
      ~GStaticMutexLock()                               {   g_static_mutex_unlock(&mutex); }
  };
  
};

#endif /* _BE_COMMON_H_ */
