#ifndef _GRT_SHELL_H_
#define _GRT_SHELL_H_


#include <grtpp.h>
#include <grtpp_shell.h>

#include "grt_dispatcher.h"
#include "grt_backend_public_interface.h"

#define ShellBE_VERSION 4

namespace bec {
class GRTManager;

class MYSQLGRTBACKEND_PUBLIC_FUNC ShellBE
{
public:
  ShellBE(GRTManager *grtm, GRTDispatcher *dispatcher);
  ~ShellBE();

  bool setup();

  void enable_completion();
  void set_save_directory(const std::string &path);
  void start();

  void process_line_async(const std::string &line);
  
  void run_script(const std::string &path);
  void run_script_grt(grt::GRT *grt, const std::string &path);

  bool previous_history_line(const std::string &current_line, std::string &line);
  bool next_history_line(std::string &line);
  void reset_history_position();

  void write_line(const std::string &line);
  void write(const std::string &text);
  void writef(const char *fmt, ...);

  void set_output_handler(const sigc::slot<void,const std::string&> &slot);
  void set_ready_handler(const sigc::slot<void,const std::string&> &slot);

  void flush_shell_output();

  void set_saves_history(bool flag);

  GList *complete_word(const std::string &prefix, std::string &nprefix);

  grt::ValueRef get_shell_variable(const std::string &varname);

  void clear_history();
  void save_history_line(const std::string &line);

  std::string get_snippet_data();
  void set_snippet_data(const std::string &data);

  void store_history();
  void restore_history();

  void handle_msg(const grt::Message &msgs);
  
protected:
  GRTManager *_grtm;
  grt::GRT *_grt;
  grt::Shell *_shell;
  GRTDispatcher *_dispatcher;

  GCompletion *_completer;
  GList *_completer_items;
  std::string _savedata_dir;

  std::string _current_statement;

  std::list<std::string> _history; // most recent first
  std::list<std::string>::iterator _history_ptr;

  sigc::slot<void,const std::string&> _ready_slot;

  sigc::slot<void, const std::string&> _output_slot;

  GStaticMutex _text_queue_mutex;
  std::list<std::string> _text_queue;

  bool _save_history;

  void update_completer();
  
private:
  void shell_finished_cb(grt::ShellCommand result, const std::string &prompt, const std::string &line);
};


};

#endif /* _GRT_SHELL_H_ */
