#ifndef _UI_FORM_H_
#define _UI_FORM_H_

#include "grt_backend_public_interface.h"
#include <sigc++/sigc++.h>
#include <vector>
#include <string>


namespace bec {

class Clipboard;


 
enum MenuItemType 
{
  MenuAction,
  MenuSeparator,
  MenuCascade,
  MenuCheck,
  MenuRadio,
  MenuUnavailable
};

struct MYSQLGRTBACKEND_PUBLIC_FUNC MenuItem
{
  std::string oid;
  std::string caption;
  std::string shortcut;
  std::string name;
  MenuItemType type;

  bool enabled;
  bool checked;

  MenuItem() : type(MenuAction), enabled(true), checked(false) {}
};
typedef std::vector<MenuItem> MenuItemList;


/** Base class for application forms.
 * 
 * This is the base class for application windows or panel backends that can 
 * receive focus and contain a workarea in the application (eg: canvas, overview panel,
 * query pages etc). It provides some virtual methods for common editing operations. 
 * 
 * @ingroup begrt
 */ 
class MYSQLGRTBACKEND_PUBLIC_FUNC UIForm : public sigc::trackable
{

public:
  UIForm();
  virtual ~UIForm();// {}

  void set_owner_data(void *data);// { _owner_data= data; }
  void *get_owner_data();// { return _owner_data; }

  void set_frontend_data(void *data);
  void *get_frontend_data();

  virtual bool is_main_form();// { return false; }

  // target description for cuy/copy/delete menu items and for paste, after a copy is made
  virtual std::string get_edit_target_name();// { return ""; }

  virtual bool can_cut(/*Clipboard *clip*/);// { return can_copy() && can_delete(); }
  virtual bool can_copy(/*Clipboard *clip*/);// { return false; }
  virtual bool can_paste(/*Clipboard *clip*/);// { return false; }
  virtual bool can_delete();// { return false; }

  virtual void cut(/*Clipboard *clip*/);// {}
  virtual void copy(/*Clipboard *clip*/);// {}
  virtual void paste(/*Clipboard *clip*/);// {}
  virtual void delete_selection();// {}

  //unused? void set_refresh_ui_callback(const sigc::slot<void> &cb);

protected:
  //sigc::slot<void> _refresh_ui;

  void *_owner_data;
  void *_frontend_data;
};


};

#endif /* _UI_FORM_H_ */
