#include "stdafx.h"

#ifndef _WIN32
#include <pcre.h>
#include <stdio.h>
#endif

//XXX should remove this mysql specific code from here --alfredo
#include <mysql.h> // needed for enum_field_types

#include "db_object_helpers.h"
#include <grtpp_undo_manager.h>
#include <grtpp_util.h>
#include "grt/common.h"
#include "grt/parse_utils.h"
#include "grts/structs.workbench.physical.h"

#ifndef _WIN32
#include <string.h>
#endif

using namespace bec;
using namespace grt;


template<class T>
class auto_array_ptr
{
  T *_ptr;

public:
  auto_array_ptr(T* ptr) : _ptr(ptr) {}
  ~auto_array_ptr() { delete[] _ptr; }

  operator T* () { return _ptr; }
};


db_mgmt_RdbmsRef get_rdbms_for_db_object(const ::grt::ValueRef &object)
{
  GrtObjectRef parent= GrtObjectRef::cast_from(object);
  while (parent.is_valid() && !parent.is_instance("workbench.physical.Model"))
    parent= parent->owner();

  // do it the hard way to avoid havig to link to objimpl for model
  //return workbench_physical_ModelRef::cast_from(parent)->rdbms();

  if (parent.is_valid())
    return db_mgmt_RdbmsRef::cast_from(parent.get_member("rdbms"));
  return db_mgmt_RdbmsRef();
}

db_SimpleDatatypeRef CatalogHelper::get_datatype(grt::ListRef<db_SimpleDatatype> types, const std::string &name)
{
  for (size_t c= types.count(), i= 0; i < c; i++)
  {
    if (g_strcasecmp(types[i]->name().c_str(), name.c_str())==0)
      return types[i];
    //for (size_t d= types[i]->synonyms().count(), j= 0; j < d; j++)
    //{
    //  if (g_strcasecmp(types[i]->synonyms().get(j).c_str(), name.c_str())==0)
    //    return types[i];
    //}
  }

  return db_SimpleDatatypeRef();
}


//------------------------------------------------------------------------------------


db_TableRef TableHelper::create_associative_table(const db_SchemaRef &schema, 
                                               const db_TableRef &table1, const db_TableRef &table2, 
                                               bool mandatory1, bool mandatory2,
                                               const grt::DictRef &global_options,
                                               const grt::DictRef &options)
{
  db_TableRef atable;
  std::string name;

  AutoUndo undo(schema->get_grt());

  name= options.get_string("AuxTableTemplate", global_options.get_string("AuxTableTemplate", "%stable%_%dtable%"));

  name= replace_string(name, "%stable%", table1->name().c_str());
  name= replace_string(name, "%dtable%", table2->name().c_str());

  atable= schema.get_grt()->create_object<db_Table>(table1.get_metaclass()->name());
  atable->owner(schema);
  atable->name(get_name_suggestion_for_list_object(schema->tables(), name, false));
  atable->oldName(atable->name());

  create_foreign_key_to_table(atable, table1, true, mandatory1, true, true, global_options, options);
  create_foreign_key_to_table(atable, table2, true, mandatory2, true, true, global_options, options);
  
  schema->tables().insert(atable);

  undo.end("Create Associative Table");

  return atable;
}



static std::string format_ident_with_table(const std::string &fmt, const db_TableRef &table)
{
  return replace_string(fmt, "%table%", table->name().c_str());
}

static std::string format_ident_with_stable_dtable(const std::string &fmt, const db_TableRef &stable, const db_TableRef &dtable)
{
  return replace_string(replace_string(fmt, "%stable%", stable->name().c_str()), "%dtable%", dtable->name().c_str());
}

static std::string format_ident_with_column(const std::string &fmt, const db_ColumnRef &column)
{
  return replace_string(replace_string(fmt, "%table%", db_TableRef::cast_from(column->owner())->name().c_str()),
                        "%column%", column->name().c_str());
}


static db_IndexRef create_index_for_fk(grt::GRT *grt, const db_ForeignKeyRef &fk)
{
  std::string index_name(fk->name().c_str());
  //index_name.append("_index");

  // add the corresponding index
  db_IndexRef index= grt->create_object<db_Index>(db_TableRef::cast_from(fk->owner()).get_metaclass()->get_member_type("indices").content.object_class.c_str());
  
  index->owner(fk->owner());
  index->name(index_name);
  index->oldName(fk->oldName());
  index->indexType("FOREIGN");

  for(size_t i= 0, sz= fk->columns().count(); i < sz; i++)
  {
    db_ColumnRef col(fk->columns().get(i));

    db_IndexColumnRef index_col(grt->create_object<db_IndexColumn>(index.get_metaclass()->get_member_type("columns").content.object_class));
    index_col->owner(index);
//    "name", col->name().valueptr(),
    index_col->descend(grt::IntegerRef(0));
    //"columnLength", grt::IntegerRef(col->length().valueptr()).valueptr(),
    index_col->columnLength(grt::IntegerRef(0));
    index_col->referencedColumn(col);

    index->columns().insert(index_col);
  }

  return index;
}


void TableHelper::update_foreign_keys_from_column_notnull(const db_TableRef &table, const db_ColumnRef &column)
{
  AutoUndo undo(table->get_grt());

  // go through all foreign keys and update the ones that have this column
  grt::ListRef<db_ForeignKey> fklist(table->foreignKeys());

  for (size_t c= fklist.count(), i= 0; i < c; i++)
  {
    db_ForeignKeyRef fk(fklist[i]);
    size_t notnull= 0;
    bool flag= false;

    for (size_t d= fk->columns().count(), j= 0; j < d; j++)
    {
      db_ColumnRef col(fk->columns()[j]);

      if (*col->isNotNull())
        notnull++;

      if (col == column)
        flag= true;
    }
    if (flag)
    {
      if (notnull == fk->columns().count())
      {
        fk->mandatory(1);
      }
      else if (notnull == 0)
      {
        fk->mandatory(0);
      }
    }
  }

  undo.end("Update FK Mandatory Flag");
}


std::string TableHelper::generate_foreign_key_name()
{
  return std::string("fk_") + grt::get_guid();
}


db_ForeignKeyRef TableHelper::create_empty_foreign_key(grt::GRT *grt, const db_TableRef &table, const std::string &name)
{
  db_ForeignKeyRef fk;

  // create a new FK
  fk= grt->create_object<db_ForeignKey>(table.get_metaclass()->get_member_type("foreignKeys").content.object_class);
  fk->owner(table);
  fk->name(name.empty()?generate_foreign_key_name():name);

  grt::AutoUndo undo(grt);
  table->foreignKeys().insert(fk);

  db_IndexRef index(create_index_for_fk(grt, fk));

  table->indices().insert(index);

  undo.end(_("Create Foreign Key"));

  return fk;
}


void TableHelper::update_foreign_key_index(grt::GRT *grt, const db_ForeignKeyRef &fk)
{
  //! todo: add undo group

  db_IndexRef index;
  db_TableRef table(db_TableRef::cast_from(fk->owner()));
  ListRef<db_Index> indices= table->indices();
  for (size_t n= 0, count= indices.count(); n < count; ++n)
  {
    db_IndexRef ind(indices.get(n));
    if ((0 == (*ind->indexType()).compare("FOREIGN")) && (ind->name() == fk->name()))
    {
      index= ind;
      break;
    }
  }

  if (!index.is_valid())
    return;

  // recreate index columns
  while (index->columns().count() > 0)
    index->columns().remove(0);

  ListRef<db_Column> fk_columns(fk->columns());
  for (size_t n= 0, count= fk_columns.count(); n < count; ++n)
  {
    db_ColumnRef column(fk_columns.get(n));
    db_IndexColumnRef index_column(grt->create_object<db_IndexColumn>(index.get_metaclass()->get_member_type("columns").content.object_class));
    index_column->owner(index);
      //"name", column->name().valueptr(),
    index_column->referencedColumn(column);
    index->columns().insert(index_column);
  }
}


/** 
 ****************************************************************************
 * @brief Creates a foreign key in table, refering to ref_table
 *
 * 
 * Options:
 * FKNameTemplate, FKColumnNameTemplate, db->foreignKey:deleteRule, updateRule
 * 
 * @param table the table to have the FK added to
 * @param ref_table the table that the FK refers to
 * @param mandatory if the relationship is mandatory in table
 * @param ref_table if the relationship is mandatory in ref_table
 * @param many cardinality of the table in the relationship
 * @param identifying whether the rel is identifying (FK is also made PK)
 * @param options the options dictionary (see above)
 *
 * @return the created foreign key
 ****************************************************************************
 */
db_ForeignKeyRef TableHelper::create_foreign_key_to_table(const db_TableRef &table, 
                                                       const db_TableRef &ref_table,
                                                       bool mandatory, bool ref_mandatory,
                                                       bool many, bool identifying,
                                                       const grt::DictRef &global_options,
                                                       const grt::DictRef &options)
{
  db_ForeignKeyRef new_fk;
  db_IndexRef pk= ref_table->primaryKey();
  grt::GRT *grt= table.get_grt();
  std::string name_format;
  std::string column_name_format;

  // check if there is a PK
  if (!pk.is_valid() || pk->columns().count() == 0)
    return new_fk;

  grt::AutoUndo undo(grt, !table->is_global());

  name_format= options.get_string("FKNameTemplate", global_options.get_string("FKNameTemplate","FK%table%"));
  column_name_format= options.get_string("FKColumnNameTemplate", global_options.get_string("FKColumnNameTemplate", "FK%table%%column%"));


  name_format= format_ident_with_stable_dtable(name_format, table, ref_table);
  name_format= format_ident_with_column(name_format, pk->columns().get(0)->referencedColumn());

  column_name_format= format_ident_with_stable_dtable(column_name_format, table, ref_table);

  // create a new FK
  new_fk= grt->create_object<db_ForeignKey>(table.get_metaclass()->get_member_type("foreignKeys").content.object_class);
  new_fk->name(get_name_suggestion_for_list_object(table->foreignKeys(), name_format, false));
  new_fk->oldName(new_fk->name());

  new_fk->deleteRule(options.get_string("db->foreignKey:deleteRule", global_options.get_string("db->foreignKey:deleteRule", "NO ACTION")));
  new_fk->updateRule(options.get_string("db->foreignKey:updateRule", global_options.get_string("db->foreignKey:updateRule", "NO ACTION")));

  new_fk->mandatory(mandatory?1:0);
  new_fk->referencedMandatory(ref_mandatory?1:0);
  new_fk->many(many?1:0);

  new_fk->referencedTable(ref_table);

  for (size_t c= pk->columns().count(), i= 0; i < c; i++)
  {
    db_IndexColumnRef pk_column= pk->columns().get(i);
    db_ColumnRef column= pk_column->referencedColumn();
    db_ColumnRef new_fk_column;

    // create the column that will be the FK in the other table
    new_fk_column= grt->create_object<db_Column>(column.class_name());
    new_fk_column->owner(table);
    new_fk_column->name(get_name_suggestion_for_list_object(table->columns(), format_ident_with_column(column_name_format, column), false));
    new_fk_column->oldName(new_fk_column->name());

    ColumnHelper::copy_column(column, new_fk_column);

    table->columns().insert(new_fk_column);

    // add the new column to the FK
    new_fk->columns().insert(new_fk_column);
    new_fk->referencedColumns().insert(column);

    if (identifying)
      table->addPrimaryKeyColumn(new_fk_column);
  }

  new_fk->owner(table); // set the owner last

  // add the FK to the source table
  table->foreignKeys().insert(new_fk);
  undo.set_description_for_last_action("Add Foreign Key to Table");

  table->indices().insert(create_index_for_fk(grt, new_fk));
  undo.set_description_for_last_action("Add Index for FK to Table");

  undo.end(_("Add Foreign Key"));
  
  return new_fk;
}



db_ForeignKeyRef TableHelper::create_foreign_key_to_table(const db_TableRef &table, const std::vector<db_ColumnRef> &columns,
                                                     const db_TableRef &ref_table, const std::vector<db_ColumnRef> &refcolumns,
                                                     bool mandatory, bool ref_mandatory,
                                                     bool many,
                                                     const grt::DictRef &global_options,
                                                     const grt::DictRef &options)
{
  db_ForeignKeyRef new_fk;
  grt::GRT *grt= table.get_grt();
  std::string name_format;

  grt::AutoUndo undo(grt, table->is_global());
  
  name_format= options.get_string("FKNameTemplate", global_options.get_string("FKNameTemplate","FK%table%"));

  name_format= format_ident_with_stable_dtable(name_format, table, ref_table);
  name_format= format_ident_with_column(name_format, refcolumns[0]);

  // create a new FK
  new_fk= grt->create_object<db_ForeignKey>(table.get_metaclass()->get_member_type("foreignKeys").content.object_class);
  new_fk->name(get_name_suggestion_for_list_object(table->foreignKeys(), name_format, false));
  new_fk->oldName(new_fk->name());

  new_fk->deleteRule(options.get_string("db->foreignKey:deleteRule", global_options.get_string("db->foreignKey:deleteRule", "NO ACTION")));
  new_fk->updateRule(options.get_string("db->foreignKey:updateRule", global_options.get_string("db->foreignKey:updateRule", "NO ACTION")));

  new_fk->referencedTable(ref_table);
  
  new_fk->mandatory(mandatory?1:0);
  new_fk->referencedMandatory(ref_mandatory?1:0);
  new_fk->many(many?1:0);

  for (size_t c= refcolumns.size(), i= 0; i < c; i++)
  {
    db_ColumnRef column(refcolumns[i]);
    db_ColumnRef fk_column(columns[i]);

    // add the new column to the FK
    new_fk->columns().insert(fk_column);
    new_fk->referencedColumns().insert(column);
  }

  new_fk->owner(table);

  // add the FK to the source table
  table->foreignKeys().insert(new_fk);
  table->indices().insert(create_index_for_fk(grt, new_fk));

  undo.end(_("Add Foreign Key"));
  
  return new_fk;
}

bool TableHelper::rename_foreign_key(const db_TableRef &table, db_ForeignKeyRef &fk, const std::string &new_name)
{
  std::string old_name;

  // check if the name is already taken
  if (find_named_object_in_list(table->foreignKeys(), new_name).is_valid())
    return false;

  old_name= fk->name();

  grt::AutoUndo undo(table->get_grt());

  fk->name(new_name);

  // find index for the FK
  for (size_t c= table->indices().count(), i= 0; i < c; i++)
  {
    db_IndexRef index(table->indices().get(i));

    if (*index->indexType() == "FOREIGN" && *index->name() == old_name)
    {
      // rename index
      index->name(new_name);
      break;
    }
  }

  undo.end(_("Rename Foreign Key"));

  return true;
}



bool TableHelper::is_identifying_foreign_key(const db_TableRef &table, const db_ForeignKeyRef &fk)
{
  // check if the fk is part of the PK
  if (table->primaryKey().is_valid())
  {
    for (size_t c= fk->columns().count(), i= 0; i < c; i++)
    {
      if (!table->isPrimaryKeyColumn(fk->columns().get(i)))
        return false;
    }
    return true;
  }
  return false;
}






void ColumnHelper::copy_column(const db_ColumnRef &from, db_ColumnRef &to)
{
  to->userType(from->userType());
  to->precision(from->precision());
  to->scale(from->scale());
  to->length(from->length());
  to->characterSetName(from->characterSetName());
  to->collationName(from->collationName());
  while (to->flags().count() > 0)
    to->flags().remove(0);
  for (size_t c= from->flags().count(), i= 0; i < c; i++)
    to->flags().insert(from->flags().get(i));
  to->simpleType(from->simpleType());
  to->structuredType(from->structuredType());
  to->datatypeExplicitParams(from->datatypeExplicitParams());
}


ColumnTypeCompareResult ColumnHelper::compare_column_types(const db_ColumnRef &from, const db_ColumnRef &to)
{
  std::string sfrom= format_column_type(from, 0, true);
  std::string sto= format_column_type(to, 0, true);

  if (sfrom != sto)
    return COLUMNS_TYPES_DIFFER;

  if (to->characterSetName() != from->characterSetName() )
    return COLUMNS_CHARSETS_DIFFER;
    
  if ( to->collationName() != from->collationName() )
    return COLUMNS_COLLATIONS_DIFFER;

  /*
  if (to->userType() != from->userType() ||
    to->precision() != from->precision() ||
    to->scale() != from->scale() ||
    to->length() != from->length() ||
    to->characterSetName() != from->characterSetName() ||
    to->collationName() != from->collationName() ||
    to->simpleType() != from->simpleType() ||
    to.structuredType() != from.structuredType() ||
    to->datatypeExplicitParams() != from->datatypeExplicitParams())
    return false;
    */

  if (to->flags().count() != from->flags().count())
    return COLUMNS_FLAGS_DIFFER;

  for (size_t c= from->flags().count(), i= 0; i < c; i++)
    if (to->flags().get_index(from->flags().get(i)) == BaseListRef::npos)
      return COLUMNS_FLAGS_DIFFER;

  //XXX compare db specific attribs

  return COLUMNS_TYPES_EQUAL;
}

std::string ColumnHelper::format_column_type(const db_ColumnRef &column, size_t shorten, bool resolve_userdefined)
{
  db_SimpleDatatypeRef simpleType(column->simpleType());
  db_StructuredDatatypeRef structuredType(column->structuredType());
  std::string caption;
  char buffer[100];

  if (column->userType().is_valid())
  {
    if (resolve_userdefined)
      caption= column->userType()->sqlDefinition();
    else
      caption= column->userType()->name();
  }
  else if (simpleType.is_valid())
  {
    caption= simpleType->name();

    if (simpleType->numericPrecision() != EMPTY_TYPE_PRECISION)
    {
      if (column->scale() != EMPTY_COLUMN_SCALE)
        g_snprintf(buffer, sizeof(buffer), "(%i,%i)", (int)column->precision(), (int)column->scale());
      else if (column->precision() != EMPTY_COLUMN_PRECISION)
        g_snprintf(buffer, sizeof(buffer), "(%i)", (int)column->precision());
      else
        buffer[0]= 0;
      caption= caption + buffer;
    }
    else
    {
      if (*simpleType->characterMaximumLength() != EMPTY_TYPE_MAXIMUM_LENGTH 
        || *simpleType->characterOctetLength() != EMPTY_TYPE_OCTET_LENGTH)
      {
        if (column->length() != EMPTY_COLUMN_LENGTH)
        {
          g_snprintf(buffer, sizeof(buffer), "(%i)", (int)column->length());
          caption= caption + buffer;
        }
      }
      else if (column->datatypeExplicitParams().is_valid() && *column->datatypeExplicitParams() != "")
        caption= caption + *column->datatypeExplicitParams();
    }
  }
  else if (structuredType.is_valid())
  {
  }
  else
  {
    char buffer[100];
    // if no simple or structured datatype is set,
    // simply take the parameters
    if (column->length() != EMPTY_COLUMN_LENGTH)
    {
      g_snprintf(buffer, sizeof(buffer), "(%i)", (int)column->length());
      caption= caption + buffer;
    }
    else if (column->precision() != EMPTY_COLUMN_PRECISION)
    {
      if (column->scale() != EMPTY_COLUMN_SCALE)
        g_snprintf(buffer, sizeof(buffer), "(%i,%i)", (int)column->precision(), (int)column->scale());
      else
        g_snprintf(buffer, sizeof(buffer), "(%i)", (int)column->precision());
      caption= caption + buffer;
    }
    else if (column->datatypeExplicitParams().is_valid() && *column->datatypeExplicitParams() != "")
    {
      caption= caption + *column->datatypeExplicitParams();      
    }
  }

  if (shorten > 0 && caption.length() > shorten)
    caption= caption.substr(0, shorten)+"...";

  return caption;
}


static int pcre_compile_exec(const char *pattern, const char *str, int *patres, int patresnum)
{
  const char *errptr;
  int erroffs;
  int c;
  pcre *patre= pcre_compile(pattern, 0, &errptr, &erroffs, NULL);
  if (!patre)
    throw std::logic_error("error compiling regex "+std::string(errptr));

  c= pcre_exec(patre, NULL, str, strlen(str), 0, 0, patres, patresnum);
  pcre_free(patre);

  return c;
}

/** 
 ****************************************************************************
 * @brief Sets the datatype defined by a string to the given column
 *
 * Assigns a datatype defined by a string like NUMERIC(7, 2) to the
 * given column.
 *
 * @param rdbms
 * @param userTypes
 * @param column
 * @param datatypeString
 * 
 * @return true on success or false on parse error or invalid type/invalid params
 ****************************************************************************
 */
bool ColumnHelper::parse_column_type(const db_mgmt_RdbmsRef &rdbms, const grt::ListRef<db_UserDatatype> &user_types,
                                    const std::string &typestr, db_ColumnRef column)
{
  auto_array_ptr<char> buffer(new char[typestr.size()+1]);
  int patres[21];
  int c;

  db_UserDatatypeRef userType;
  db_SimpleDatatypeRef simpleType;
  int precision= EMPTY_COLUMN_PRECISION;
  int scale= EMPTY_COLUMN_SCALE;
  int length= EMPTY_COLUMN_LENGTH;
  std::string datatypeExplicitParams;


  if (user_types.is_valid())
  {
    // 1st check if this is a user defined type
    for (size_t c= user_types.count(), i= 0; i < c; i++)
    {
      db_UserDatatypeRef utype(user_types[i]);

      if (g_strcasecmp(utype->name().c_str(), typestr.c_str()) == 0)
      {
        userType= utype;
        break;
      }
    }
  }

  if (!userType.is_valid())
  {
    /* split the datatype name from the parameters
     * (\w+)\s*(\(.*\))?
     *
     * 1.. TYPE
     * 2.. params (optional)
     */

    c= pcre_compile_exec("^(\\w+)\\s*(\\(.*\\))?\\s*$", typestr.c_str(), patres, sizeof(patres)/sizeof(int));

    if (c > 0 &&
      pcre_copy_substring(typestr.c_str(), patres, c, 1, buffer, static_cast<int>(typestr.size()+1)) > 0)
    {
      // find the simple datatype with this type_name
      simpleType= CatalogHelper::get_datatype(rdbms->simpleDatatypes(), std::string(buffer));

      if (simpleType.is_valid())
      {
        std::string params;
        std::string param1;
        std::string param2;

        // split the parameter into its components

        if (pcre_copy_substring(typestr.c_str(), patres, c, 2, buffer, static_cast<int>(typestr.size()+1)) > 0)
          params= buffer;

        switch (*simpleType->parameterFormatType())
        {
        case 0: // no params
          if (!params.empty())
            return false;
          break;

        case 1: // (n)
          c= pcre_compile_exec("^\\((\\d+)\\)$", params.c_str(), patres, sizeof(patres)/sizeof(int));
          if (c < 0)
            return false;
          if (pcre_copy_substring(params.c_str(), patres, c, 1, buffer, static_cast<int>(params.size()+1)) > 0)
          {
            param1= buffer;
          }
          else
            return false;
          break;

        case 2: // [(n)]
          c= pcre_compile_exec("^(\\((\\d+)\\))?$", params.c_str(), patres, sizeof(patres)/sizeof(int));
          if (c < 0)
            return false;
          if (pcre_copy_substring(params.c_str(), patres, c, 2, buffer, static_cast<int>(params.size()+1)) > 0)
          {
            param1= buffer;
          }
          break;

        case 3: // (m, n)
          c= pcre_compile_exec("^\\((\\d+),\\s*(\\d+)\\)$", params.c_str(), patres, sizeof(patres)/sizeof(int));
          if (c < 0)
            return false;
          if (pcre_copy_substring(params.c_str(), patres, c, 1, buffer, static_cast<int>(params.size()+1)) > 0)
          {
            param1= buffer;

            if (pcre_copy_substring(params.c_str(), patres, c, 2, buffer, static_cast<int>(params.size()+1)) > 0)
              param2= buffer;
            else
              return false;
          }
          else
            return false;
          break;

        case 4: // (m[,n])
          c= pcre_compile_exec("^\\((\\d+)(,\\s*(\\d+))?\\)$", params.c_str(), patres, sizeof(patres)/sizeof(int));
          if (c < 0)
            return false;
          if (pcre_copy_substring(params.c_str(), patres, c, 1, buffer, static_cast<int>(params.size()+1)) > 0)
          {
            param1= buffer;
            if (pcre_copy_substring(params.c_str(), patres, c, 3, buffer, static_cast<int>(params.size()+1)) > 0)
              param2= buffer;
          }
          else
            return false;
          break;

        case 5: // [(m,n)]
          c= pcre_compile_exec("^(\\((\\d+)(,\\s*(\\d+))\\))?$", params.c_str(), patres, sizeof(patres)/sizeof(int));
          if (c < 0)
            return false;
          if (pcre_copy_substring(params.c_str(), patres, c, 2, buffer, static_cast<int>(params.size()+1)) > 0)
          {
            param1= buffer;
            if (pcre_copy_substring(params.c_str(), patres, c, 4, buffer, static_cast<int>(params.size()+1)) > 0)
              param2= buffer;
            else
              return false;
          }
          break;

        case 6: // [(m[,n])]
          c= pcre_compile_exec("^(\\((\\d+)(,\\s*(\\d+))?\\))?$", params.c_str(), patres, sizeof(patres)/sizeof(int));
          if (c < 0)
            return false;
          if (pcre_copy_substring(params.c_str(), patres, c, 2, buffer, static_cast<int>(params.size()+1)) > 0)
          {
            param1= buffer;
            if (pcre_copy_substring(params.c_str(), patres, c, 4, buffer, static_cast<int>(params.size()+1)) > 0)
              param2= buffer;
          }
          break;

        case 10: // ('a','b','c')
          if (params.empty())
            return false;
          c= pcre_compile_exec("^\\(([^)]*)\\)?$", params.c_str(), patres, sizeof(patres)/sizeof(int));
          if (c < 0)
            return false;
          // For now we just keep the parameter list as a whole. If necessary this can be split later for other uses.
          param1= buffer;
          // validate format
          if (pcre_copy_substring(params.c_str(), patres, c, 1, buffer, static_cast<int>(params.size()+1)) > 0)
          {
            std::list<std::string> tokens;
            if (!tokenize_string_list(std::string(buffer), '\'', true, tokens))
              return false;
          }
          else
            return false;
          break;
        }

        // get precision
        if (*simpleType->numericPrecision() != EMPTY_TYPE_PRECISION)
        {
          if (!param1.empty())
            precision= atoi(param1.c_str());

          // get scale
          if (*simpleType->numericScale() != EMPTY_TYPE_SCALE)
          {
            if (!param2.empty())
              scale= atoi(param2.c_str());
          }
        }
        // get length
        else if (simpleType->characterMaximumLength() != EMPTY_TYPE_MAXIMUM_LENGTH 
              || simpleType->characterOctetLength()   != EMPTY_TYPE_OCTET_LENGTH)
        {
          if (!param1.empty())
            length= atoi(param1.c_str());
        }
        else if (*simpleType->parameterFormatType() == 10)
          datatypeExplicitParams= param1;
      }
      else
        return false; // unknown type
    }
    else
      return false; // bad format
  }

  AutoUndo undo(rdbms->get_grt());

  if (userType.is_valid())
  {
    column->simpleType(db_SimpleDatatypeRef());
    column->userType(userType);
  }
  else
  {
    column->userType(db_UserDatatypeRef());
    column->simpleType(simpleType);
    column->precision(precision);
    column->scale(scale);
    column->length(length);
    column->datatypeExplicitParams(datatypeExplicitParams);
  }
  undo.end(_("Change Column Type"));

  return true;
}

void ColumnHelper::set_default_value(db_ColumnRef column, const std::string &value)
{
  // TODO: this also should affect the isNotNull flag, but sometimes e.g. for 
  // auto-inc or timestamp columns we shouldn't touch it, need to investigate 
  // the full list of exceptions, but not sure if all these fields are still present in 5.1
  column->defaultValueIsNull(g_strcasecmp(value.c_str(), "NULL") ? 0 : 1);
  column->defaultValue(value.c_str());
}

//------------------------------------------------------------------------------------

static enum_field_types get_mysql_type_from_string(const char *type_name)
{
  if(g_strcasecmp(type_name, "INT") == 0)
    return MYSQL_TYPE_LONG;
  if(g_strcasecmp(type_name, "TINYINT") == 0)
    return MYSQL_TYPE_TINY;
  if(g_strcasecmp(type_name, "SMALLINT") == 0)
    return MYSQL_TYPE_SHORT;
  if(g_strcasecmp(type_name, "BIGINT") == 0)
    return MYSQL_TYPE_LONGLONG;
  if(g_strcasecmp(type_name, "FLOAT") == 0)
    return MYSQL_TYPE_FLOAT;
  if(g_strcasecmp(type_name, "DOUBLE") == 0)
    return MYSQL_TYPE_DOUBLE;
  if(g_strcasecmp(type_name, "TIME") == 0)
    return MYSQL_TYPE_TIME;
  if(g_strcasecmp(type_name, "DATE") == 0)
    return MYSQL_TYPE_DATE;
  if(g_strcasecmp(type_name, "DATETIME") == 0)
    return MYSQL_TYPE_DATETIME;
  if(g_strcasecmp(type_name, "TIMESTAMP") == 0)
    return MYSQL_TYPE_TIMESTAMP;
  if(g_strcasecmp(type_name, "TEXT") == 0)
    return MYSQL_TYPE_STRING;
  if(g_strcasecmp(type_name, "CHAR") == 0)
    return MYSQL_TYPE_STRING;
  if(g_strcasecmp(type_name, "VARCHAR") == 0)
    return MYSQL_TYPE_STRING;
  if(g_strcasecmp(type_name, "BLOB") == 0)
    return MYSQL_TYPE_BLOB;
  if(g_strcasecmp(type_name, "BINARY") == 0)
    return MYSQL_TYPE_BLOB;
  if(g_strcasecmp(type_name, "VARBINARY") == 0)
    return MYSQL_TYPE_BLOB;

  return MYSQL_TYPE_NULL;
}

void CatalogHelper::apply_defaults(db_mysql_CatalogRef cat)
{
  cat->defaultCharacterSetName("latin1");
  cat->defaultCollationName("latin1_swedish_ci");

  // for numeric types only
  std::map<std::string, int> def_precision_map;
  def_precision_map["INT"]= 11;
  def_precision_map["TINYINT"]= 4;
  def_precision_map["SMALLINT"]= 6;
  def_precision_map["MEDIUMINT"]= 9;
  def_precision_map["BIGINT"]= 20;
  def_precision_map["FLOAT"]= 11; // not sure about last 2
  def_precision_map["DOUBLE"]= 11;
	
  for(size_t i= 0, schemata_count= cat->schemata().count(); i < schemata_count; i++)
  {
    db_mysql_SchemaRef schema= cat->schemata().get(i);

    if(strlen(schema->defaultCharacterSetName().c_str()) == 0)
      schema->defaultCharacterSetName(cat->defaultCharacterSetName());

    if(strlen(schema->defaultCollationName().c_str()) == 0)
      schema->defaultCollationName(get_cs_def_collation(schema->defaultCharacterSetName().c_str()));

    for(size_t j= 0, tables_count= schema->tables().count(); j < tables_count; j++)
    {
      db_mysql_TableRef table= schema->tables().get(j);

      if(strlen(table->defaultCharacterSetName().c_str()) == 0)
        table->defaultCharacterSetName(schema->defaultCharacterSetName());

      if(strlen(table->defaultCollationName().c_str()) == 0)
        table->defaultCollationName(get_cs_def_collation(table->defaultCharacterSetName().c_str()));
      
      if(strlen(table->tableEngine().c_str()) == 0)
        table->tableEngine("InnoDB");

      for(size_t k= 0, col_count= table->columns().count(); k < col_count; k++)
      {
        db_mysql_ColumnRef column= table->columns().get(k);
        bool default_default_value= !column->isNotNull() && (strlen(column->defaultValue().c_str()) == 0);
        
        if (!column->simpleType().is_valid())
          continue;

        //enum_field_types ft= get_mysql_type_from_string(column->simpleType()->name().c_str());

        std::map<std::string, int>::const_iterator prec_map_it= 
          def_precision_map.find(column->simpleType()->name().c_str());
        if(prec_map_it != def_precision_map.end())
        {
          column->length(grt::IntegerRef(-1));
          if(column->precision() == bec::EMPTY_COLUMN_PRECISION)
            column->precision(grt::IntegerRef(prec_map_it->second));
          if (default_default_value)
            bec::ColumnHelper::set_default_value(column, "NULL");
        }
        else if((strcmp(column->simpleType()->name().c_str(), "VARCHAR") == 0)
          || (strcmp(column->simpleType()->name().c_str(), "CHAR") == 0)
          || (strcmp(column->simpleType()->name().c_str(), "DECIMAL") == 0)
          || (strcmp(column->simpleType()->name().c_str(), "BOOLEAN") == 0)
          || (strcmp(column->simpleType()->name().c_str(), "BINARY") == 0)
          || (strcmp(column->simpleType()->name().c_str(), "TINYBLOB") == 0)
          || (strcmp(column->simpleType()->name().c_str(), "MEDIUMBLOB") == 0)
          || (strcmp(column->simpleType()->name().c_str(), "BLOB") == 0)
          || (strcmp(column->simpleType()->name().c_str(), "LONGBLOB") == 0)
          || (strcmp(column->simpleType()->name().c_str(), "TINYTEXT") == 0)
          || (strcmp(column->simpleType()->name().c_str(), "TEXT") == 0)
          || (strcmp(column->simpleType()->name().c_str(), "MEDIUMTEXT") == 0)
          || (strcmp(column->simpleType()->name().c_str(), "LONGTEXT") == 0)
          || (strcmp(column->simpleType()->name().c_str(), "ENUM") == 0)
          || (strcmp(column->simpleType()->name().c_str(), "SET") == 0))
        {
          if (default_default_value)
            bec::ColumnHelper::set_default_value(column, "NULL");
        }
        else if((strcmp(column->simpleType()->name().c_str(), "DATETIME") == 0)
          || (strcmp(column->simpleType()->name().c_str(), "DATE") == 0)
          || (strcmp(column->simpleType()->name().c_str(), "TIME") == 0)
          || (strcmp(column->simpleType()->name().c_str(), "TIMESTAMP") == 0)
          || (strcmp(column->simpleType()->name().c_str(), "YEAR") == 0)
          || (strcmp(column->simpleType()->name().c_str(), "BIT") == 0))
        {
          column->length(grt::IntegerRef(-1));
          if (default_default_value)
            bec::ColumnHelper::set_default_value(column, "NULL");
        }
      }
    }
  }
}
