#ifndef _SQL_FACADE_H_
#define _SQL_FACADE_H_


#include <memory>
#include "sql_parser_public_interface.h"
#include "sql_parser.h"
#include "invalid_sql_parser.h"
#include "grts/structs.db.mgmt.h"


/** Serves as a single point of access to all functionality related to SQL processing.
 * Provides helper methods to perform most of the tasks.
 * Singleton.
 * 
 * @ingroup sqlparser
 */
class SQL_PARSER_PUBLIC_FUNC SqlFacade
{
public:
  static SqlFacade * instance_for_db_obj(db_DatabaseObjectRef db_obj);
  static SqlFacade * instance_for_rdbms(db_mgmt_RdbmsRef rdbms);
  static SqlFacade * instance_for_rdbms_name(grt::GRT *grt, const std::string &name);

  virtual int splitSqlScript(const std::string &sql_script, std::list<std::string> &statements)= 0;

  virtual std::auto_ptr<Sql_parser> sqlParser()= 0;
  virtual int parseSqlScriptString(grt::Ref<db_Catalog> catalog, const std::string sql_script_string)= 0;
  virtual int parseSqlScriptStringEx(grt::Ref<db_Catalog> catalog, const std::string sql_script_string, const grt::DictRef options)= 0;
  virtual int parseSqlScriptFile(grt::Ref<db_Catalog> catalog, const std::string sql_script_file)= 0;
  virtual int parseSqlScriptFileEx(grt::Ref<db_Catalog> catalog, const std::string sql_script_file, const grt::DictRef options)= 0;

  virtual std::auto_ptr<Invalid_sql_parser> invalidSqlParser()= 0;
  virtual int parseInserts(grt::Ref<db_Table> table, const std::string sql)= 0;
  virtual int parseTriggers(grt::Ref<db_Table> table, const std::string sql)= 0;
  virtual int parseRoutine(grt::Ref<db_Routine> routine, const std::string sql)= 0;
  virtual int parseRoutines(grt::Ref<db_RoutineGroup> routineGroup, const std::string sql)= 0;
  virtual int parseView(grt::Ref<db_View> view, const std::string sql)= 0;

  virtual int checkTriggerSyntax(const std::string sql_script_string)= 0;
  virtual int checkViewSyntax(const std::string sql_script_string)= 0;
  virtual int checkRoutineSyntax(const std::string sql_script_string)= 0;

  virtual std::string normalizeSqlStatement(const std::string sql_script, const std::string schema_name)= 0;

  virtual int renameSchemaReferences(grt::Ref<db_Catalog> catalog, const std::string old_schema_name, const std::string new_schema_name)= 0;
};


#endif /* _SQL_FACADE_H_ */
