#include "checkbox_list_control.h"
#include <algorithm>


StringCheckBoxList::StringCheckBoxList()
  : mforms::ScrollPanel(false), _box(false)
{
  _box.set_padding(8);
  add(&_box);
}


void StringCheckBoxList::set_strings(const std::vector<std::string> &strings)
{
  for (std::vector<mforms::CheckBox*>::const_iterator iter= _items.begin();
       iter != _items.end(); ++iter)
  {
    _box.remove(*iter);
    delete *iter;
  }
  _items.clear();
  
  for (std::vector<std::string>::const_iterator iter= strings.begin();
       iter != strings.end(); ++iter)
  {
    mforms::CheckBox *cb= new mforms::CheckBox();
    cb->set_text(*iter);
    cb->set_name(*iter);
    
    cb->signal_toggled().connect(sigc::mem_fun(this, &StringCheckBoxList::toggled));

    _box.add(cb, false, false);
    _items.push_back(cb);
  }
}


void StringCheckBoxList::set_strings(const grt::StringListRef &strings)
{
  for (std::vector<mforms::CheckBox*>::const_iterator iter= _items.begin();
       iter != _items.end(); ++iter)
  {
    _box.remove(*iter);
    delete *iter;
  }
  _items.clear();

  for (grt::StringListRef::const_iterator iter= strings.begin();
       iter != strings.end(); ++iter)
  {
    mforms::CheckBox *cb= new mforms::CheckBox();
    cb->set_text(*iter);
    cb->set_name(*iter);
    
    cb->signal_toggled().connect(sigc::mem_fun(this, &StringCheckBoxList::toggled));

    _box.add(cb, false, false);
    _items.push_back(cb);
  }
}


static void delete_(mforms::CheckBox *cb)
{
  delete cb;
}


StringCheckBoxList::~StringCheckBoxList()
{
  std::for_each(_items.begin(), _items.end(), sigc::ptr_fun(delete_));
}


bool StringCheckBoxList::has_selection()
{
  for (std::vector<mforms::CheckBox*>::const_iterator iter= _items.begin();
       iter != _items.end(); ++iter)
  {    
    if ((*iter)->get_active())
      return true;
  }
  return false;
}


void StringCheckBoxList::set_selected(const std::string &name, bool flag)
{
  for (std::vector<mforms::CheckBox*>::const_iterator iter= _items.begin();
       iter != _items.end(); ++iter)
  {    
    if ((*iter)->get_name() == name)
    {
      (*iter)->set_active(flag);
    }
  }
}


std::vector<std::string> StringCheckBoxList::get_selection()
{
  std::vector<std::string> list;
  
  for (std::vector<mforms::CheckBox*>::const_iterator iter= _items.begin();
       iter != _items.end(); ++iter)
  {
    if ((*iter)->get_active())
      list.push_back((*iter)->get_name());
  }
  return list;
}


void StringCheckBoxList::toggled()
{
  _signal_changed.emit();
}
