
#include <mforms/mforms.h>
#include <algorithm>

#include <grt/grt_manager.h>

#include "grt_wizard_form.h"

using namespace grtui;
using namespace mforms;

#define WIZARD_WIDTH 860
#define WIZARD_HEIGHT 600



struct WizardForm::StepItem
{
  WizardPage *page;
  mforms::Label *label;
  
  StepItem(WizardPage *apage)
  {
    page= apage;
    label= new mforms::Label();
    label->set_text(apage->get_title());
    label->set_style(BoldStyle);
  }
  
  void reset()
  {
    label->set_color("#999999");
  }
  
  void set_executing()
  {
    label->set_color("#ffffff");
  }
  
  void set_executed()
  {
    label->set_color("#cccccc");
  }
};


WizardForm::WizardForm(bec::GRTManager *mgr)
: _grtm(mgr),
  _header_back(false), _hheader(true), _header(false), _contents(false),
  _side(false), _button_box(true)
{
#ifdef _DEBUG
  set_name("WizardForm");
  _toplevel.set_name("WizardForm:toplevel");
  _header_back.set_name("WizardForm:header_back");
  _hheader.set_name("WizardForm:hheader");
  _header.set_name("WizardForm:header");
  _contents.set_name("WizardForm:contents");
  _button_box.set_name("WizardForm:button_box");
  _sakila.set_name("WizardForm:sakila");
#endif

  // TODO: optimize by disabling all layouting while constructing the form.
  set_size(WIZARD_WIDTH, WIZARD_HEIGHT);

  set_content(&_toplevel);
  _toplevel.set_row_count(3);
  _toplevel.set_column_count(2);

  _active_page= 0;

  _top_title.set_style(BoldStyle);
  _top_subtitle.set_color("#555555");

  _header.set_padding(12);
  _header.set_spacing(4);
  _header.add(&_top_title, false, true);
  _header.add(&_top_subtitle, false, true);

  _header_back.set_padding(0);
  _header_back.set_back_color("#ffffff");
  _header_back.set_size(-1, 60);
  _header_back.add(&_hheader);
  
  _hheader.add(&_header, true, true);
  _hheader.add(&_sakila, false, false);
  _sakila.set_image(bec::IconManager::get_instance()->get_icon_path("sakila.png"));

  _cancel.set_text(_("_Cancel"));
  _cancel.set_size(100, -1);
  _cancel.signal_clicked().connect(sigc::mem_fun(this, &WizardForm::cancel));

  _next.set_text(_("_Next >"));
  _next.set_size(100, -1);
  _next.signal_clicked().connect(sigc::mem_fun(this, &WizardForm::go_to_next));

  _back.set_text(_("< _Back"));
  _back.set_size(100, -1);
  _back.signal_clicked().connect(sigc::mem_fun(this, &WizardForm::go_to_back));

  _advanced.set_text(_("_Advanced >>"));
  _advanced.signal_clicked().connect(sigc::mem_fun(this, &WizardForm::toggle_advanced));

  _button_box.set_padding(12);
  _button_box.set_spacing(8);

  _button_box.add_end(&_next, false, false);
  _button_box.add_end(&_back, false, false);
  _button_box.add_end(&_cancel, false, false);
  _advanced.show(false);
  _button_box.add(&_advanced, false, false);

  _toplevel.add(&_header_back, 1, 2, 0, 1, mforms::HExpandFlag|mforms::HFillFlag|mforms::VFillFlag);
//  _toplevel.add(&_side, 0, 1, 0, 2, mforms::HFillFlag|mforms::VExpandFlag|mforms::VFillFlag);
  _toplevel.add(&_contents, 1, 2, 1, 2, mforms::HExpandFlag|mforms::HFillFlag|mforms::VExpandFlag|mforms::VFillFlag);
  _toplevel.add(&_button_box, 0, 2, 2, 3, mforms::HExpandFlag|mforms::HFillFlag|mforms::VFillFlag);

  _contents.set_padding(12);
  
  _side.set_name("wizard_summary");
  _side.set_padding(0);
  _side.set_size(220, -1);
  _side.set_back_color("#036c94");

  _side_table.set_padding(14);
  _side_table.set_row_spacing(16);
  _side_table.set_column_spacing(2);
  _side.add(&_side_table);
  
  _values= grt::DictRef(mgr->get_grt());
}


WizardForm::~WizardForm() 
{
  for (std::vector<WizardPage*>::iterator iter= _pages.begin();
       iter != _pages.end(); ++iter)
    delete *iter;
}


int WizardForm::default_width() const
{
  return WIZARD_WIDTH;
}


void WizardForm::add_page(WizardPage *page)
{
  _pages.push_back(page);
}


WizardPage *WizardForm::get_page_with_id(const std::string &id)
{
  for (std::vector<WizardPage*>::const_iterator iter= _pages.begin();
       iter != _pages.end(); ++iter)
  {
    if ((*iter)->get_id() == id)
      return *iter;
  }
  return 0;
}


void WizardForm::reset()
{
  if (_active_page)
  {
    _contents.remove(_active_page);
    _turned_pages.clear();
    
    _active_page= 0;
  }
}


grt::DictRef WizardForm::run_modal(grt::DictRef &values)
{
  rebuild_step_list();
  
  reset();

  switch_to_page(_pages.front(), true);

  mforms::Form::run_modal(0, 0);

  return values;
}


void WizardForm::rebuild_step_list()
{  
  for (std::vector<StepItem*>::iterator step= _step_items.begin();
       step != _step_items.end(); ++step)
  {
    _side_table.remove((*step)->label);
    delete *step;
  }
  _step_items.clear();

  _side_table.set_row_count(_pages.size()+1);
  _side_table.set_column_count(2);

  int row= 0;
  for (std::vector<WizardPage*>::const_iterator p= _pages.begin(); p != _pages.end(); ++p)
  {
    StepItem *step= new StepItem(*p);
    step->reset();
    _side_table.add(step->label, 1, 2, row, row+1, HFillFlag);
    _step_items.push_back(step);

    ++row;
  }
}


void WizardForm::refresh_step_list()
{
  bool passed= false;
  for (std::vector<StepItem*>::iterator step= _step_items.begin();
       step != _step_items.end(); ++step)
  {
    if ((*step)->page == _active_page)
    {
      (*step)->set_executing();
      passed= true;
    }
    else if (!passed)
      (*step)->set_executed();
    else
      (*step)->reset();
  }
}


void WizardForm::switch_to_page(WizardPage *page, bool advancing)
{
  if (_active_page)
  {
    if (page != _active_page)
    {
      _active_page->show(false);
      _active_page->leave(_values);
    }
  }

  if (_active_page != page)
  {
    if (advancing)
      page->load();

    if (_active_page)
      _contents.remove(_active_page);

    _contents.add(page);
    _contents.set_size(-1, -1);

    _active_page= page;

    _active_page->show(true);

    _active_page->enter(_values);

    update_buttons();
    
    if (advancing)
      _active_page->execute();
  }

  // Set new section text and let it resize to minimal size (which then gets layed out to our rules).
  _top_title.set_text(_active_page->get_title());
  _top_title.set_size(-1, -1);

  _top_subtitle.set_text(_active_page->get_subtitle());
  _top_subtitle.set_size(-1, -1);
  if (_active_page->get_advanced_panel())
    _advanced.show(true);
  else
    _advanced.show(false);

  update_buttons();
  
  refresh_step_list();
}


int WizardForm::get_active_page_number()
{
  return (int)(std::find(_pages.begin(), _pages.end(), _active_page) - _pages.begin());
}


void WizardForm::set_problem(const std::string &problem)
{
  _problem= problem;
  
  update_buttons();
}


void WizardForm::clear_problem()
{
  _problem.clear();
  update_buttons();
}


void WizardForm::update_buttons()
{
  if (_active_page)
  {
    std::string caption= _active_page->next_button_caption();
    if (caption.empty())
    {
      if (_active_page->next_closes_wizard())
        caption= _("_Close");
      else
        caption= _("_Next >");
    }
    _next.set_text(caption);

    _next.set_enabled(_problem.empty() && _active_page->allow_next());
    _back.set_enabled(!_turned_pages.empty() && _active_page->allow_back());
    _cancel.set_enabled(_active_page->allow_cancel());

    if (_active_page->advanced_shown())
      _advanced.set_text(" _Advanced << ");
    else
      _advanced.set_text(" _Advanced >> ");
  }
}


WizardPage *WizardForm::get_next_page(WizardPage *current)
{
  for (std::vector<WizardPage*>::const_iterator iter= _pages.begin();
       iter != _pages.end(); ++iter)
  {
    if (*iter == current)
    {
      ++iter;
      if (iter != _pages.end())
        return *iter;
      return 0;
    }
  }
  return 0;
}


void WizardForm::finish()
{
  if (_active_page)
    _active_page->leave(_values);

  close();
}


void WizardForm::go_to_next()
{
  if (_active_page)
  {    
    if (_active_page->override_next_action() &&
        _active_page->execute_overriden_next_action())
      return;

    _next.set_enabled(false);
    _back.set_enabled(false);
    _cancel.set_enabled(false);

    _turned_pages.push_back(_active_page);
    
    if (_active_page->next_closes_wizard())
    {
      finish();
      return;
    }

    WizardPage *next_page= get_next_page(_active_page);
    if (!next_page)
    {
      g_warning("Trying to turn past the last page of the wizard");
      finish();
      return;
    }
    switch_to_page(next_page, true);
  }
}


void WizardForm::go_to_back()
{
  if (!_turned_pages.empty())
  {
    WizardPage *back_page= _turned_pages.back();
    _turned_pages.pop_back();

    switch_to_page(back_page, false);
  }
}


void WizardForm::cancel()
{
  close();
}


void WizardForm::toggle_advanced()
{
  if (_active_page)
  {
    _active_page->show_advanced(!_active_page->advanced_shown());
    update_buttons();
  }
}


//--------------------------------------------------------------------------------

WizardPage::WizardPage(WizardForm *form, const std::string &id)
: Box(false), _form(form), _id(id)
{
  _advanced_shown= false;
  set_padding(0);
  set_spacing(12);
}


void WizardPage::set_title(const std::string &title)
{
  _title= title;
}


void WizardPage::set_subtitle(const std::string &title)
{
  _subtitle= title;
}



void WizardPage::show_advanced(bool flag)
{
  _advanced_shown= flag;
  get_advanced_panel()->show(flag);
}

bool WizardPage::advanced_shown() const
{
  return _advanced_shown;
}


bool WizardPage::override_next_action()
{
  return false;
}


bool WizardPage::execute_overriden_next_action()
{
  return false;
}


void WizardPage::enable_file_browsing(mforms::TextEntry *entry, mforms::Button *button, mforms::FileChooserType type,
                                      const std::string &extensions)
{
  button->signal_clicked().
    connect(sigc::bind(sigc::mem_fun(this, &WizardPage::browse_file_callback),
                       entry, type, extensions));
}


void WizardPage::browse_file_callback(mforms::TextEntry *entry, mforms::FileChooserType type,
                                      const std::string &extensions)
{
  FileChooser fsel(type);
  
  if (!extensions.empty())
    fsel.set_extensions(extensions, "");

  if (fsel.run_modal())
    entry->set_value(fsel.get_path());
}



void WizardPage::validate()
{
  if (_form->get_active_page() == this)
  {
    do_validate();
  }
  _form->update_buttons();
}



void WizardPage::enter(grt::DictRef values)
{
  _signal_enter.emit(values);
}


void WizardPage::leave(grt::DictRef values)
{
  _signal_leave.emit(values);
}

