
#include "grtdb_object_filter.h"
#include "grt/grt_string_list_model.h"
#include "text_input_dialog.h"

using namespace grtui;


static void refill_list(mforms::ListBox &list, bec::GrtStringListModel *model)
{
  list.clear();
  for (size_t c= model->count(), i= 0; i < c; i++)
  {
    std::string item;
    model->get_field(i, 0, item);
    list.add_item(item);
  }
}



DBObjectFilterFrame::DBObjectFilterFrame(bec::GRTManager *grtm)
  : mforms::Panel(true), _filter_be(grtm), _box(false),
    _object_list(true), _mask_list(true)
{
  set_padding(8);
  
  add(&_box);

  // summary view
  _box.add(&_summary_table, false, true);

  _summary_table.set_row_count(2);
  _summary_table.set_column_count(4);
  
  _summary_table.set_row_spacing(4);
  _summary_table.set_column_spacing(12);

  _icon.set_size(48, 48);
  
  _summary_table.add(&_icon, 0, 1, 0, 2, 0);

  _summary_label1.set_text(_("Objects of Type:"));
  _summary_label1.set_text_align(mforms::MiddleRight);
  
  _summary_label2.set_text(_("Selection Count:"));
  _summary_label2.set_text_align(mforms::MiddleRight);

  _summary_bold_label1.set_text_align(mforms::MiddleLeft);
  _summary_bold_label1.set_style(mforms::BoldStyle);

  _summary_bold_label2.set_text_align(mforms::MiddleLeft);
  _summary_bold_label2.set_style(mforms::BoldStyle);

  _summary_table.add(&_summary_label1, 1, 2, 0, 1, 0);
  _summary_table.add(&_summary_bold_label1, 2, 3, 0, 1, mforms::HExpandFlag|mforms::HFillFlag);

  _summary_table.add(&_summary_label2, 1, 2, 1, 2, 0);
  _summary_table.add(&_summary_bold_label2, 2, 3, 1, 2, mforms::HExpandFlag|mforms::HFillFlag);

  _toggle_label.set_text(_("Click [Detailed Selection] to add objects to the ignore list."));
  _show_button.set_text(_("Detailed Selection >>"));
  _show_button.set_size(150, -1);
  _show_button.signal_clicked().connect(sigc::mem_fun(this, &DBObjectFilterFrame::show_detailed));

  _summary_table.add(&_toggle_label, 3, 4, 0, 1, mforms::HExpandFlag|mforms::HFillFlag);
  _summary_table.add(&_show_button, 3, 4, 1, 2, mforms::HExpandFlag);
  
  
  // detailed view
  _box.add(&_detailed_table, false, true);
  _detailed_table.show(false);

  _detailed_table.set_row_count(9);
  _detailed_table.set_column_count(4);
  
  _detailed_table.set_row_spacing(8);
  _detailed_table.set_column_spacing(12);

  //not implemented yet
  //_search_label.set_text(_("Search:"));
  //_detailed_table.add(&_search_label, 0, 1, 0, 1, 0);

  //_detailed_table.add(&_search_text, 1, 2, 0, 1);

  /*
  _filter_label.set_text(_("Filter:"));
  _detailed_table.add(&_filter_label, 2, 3, 0, 1, 0);


  _filter_combo.set_tooltip(_("Select a preset filter."));
  _detailed_table.add(&_filter_combo, 3, 4, 0, 1);
  */

  _object_label.set_text(_("Objects to Process"));
  _detailed_table.add(&_object_label, 1, 2, 1, 2);

  _detailed_table.add(&_object_list, 1, 2, 2, 8);
  _object_list.signal_changed().connect(sigc::mem_fun(this, &DBObjectFilterFrame::update_button_enabled));

  _add1_button.set_text(">");
  _add1_button.set_size(30, -1);
  _add1_button.signal_clicked().connect(sigc::bind(sigc::mem_fun(this, &DBObjectFilterFrame::add_clicked), false));
  _del1_button.set_text("<");
  _del1_button.set_size(30, -1);
  _del1_button.signal_clicked().connect(sigc::bind(sigc::mem_fun(this, &DBObjectFilterFrame::del_clicked), false));

  _add2_button.set_text(">>");
  _add2_button.set_size(30, -1);
  _add2_button.signal_clicked().connect(sigc::bind(sigc::mem_fun(this, &DBObjectFilterFrame::add_clicked), true));
  _del2_button.set_text("<<");
  _del2_button.set_size(30, -1);
  _del2_button.signal_clicked().connect(sigc::bind(sigc::mem_fun(this, &DBObjectFilterFrame::del_clicked), true));
  
  _mask_button.set_text("+");
  _mask_button.set_size(30, -1);
  _mask_button.signal_clicked().connect(sigc::mem_fun(this, &DBObjectFilterFrame::add_mask));

  _detailed_table.add(&_add1_button, 2, 3, 2, 3, mforms::HFillFlag);
  _detailed_table.add(&_del1_button, 2, 3, 3, 4, mforms::HFillFlag);
  _detailed_table.add(&_add2_button, 2, 3, 4, 5, mforms::HFillFlag);
  _detailed_table.add(&_del2_button, 2, 3, 5, 6, mforms::HFillFlag);
  _detailed_table.add(&_mask_button, 2, 3, 6, 7, mforms::HFillFlag);


  _mask_label.set_text(_("Excluded Objects"));
  _detailed_table.add(&_mask_label, 3, 4, 1, 2);

  _detailed_table.add(&_mask_list, 3, 4, 2, 8);
  _mask_list.signal_changed().connect(sigc::mem_fun(this, &DBObjectFilterFrame::update_button_enabled));
  
  _hide_button.set_text(_("<< Hide Details"));
  _hide_button.set_size(150, -1);
  _hide_button.signal_clicked().connect(sigc::mem_fun(this, &DBObjectFilterFrame::hide_detailed));
  _detailed_table.add(&_hide_button, 3, 4, 8, 9, 0);
}



void DBObjectFilterFrame::set_object_class(const std::string &oclass, const std::string &caption_format)
{
  _filter_be.set_object_type_name(oclass);

  _summary_bold_label1.set_text(_filter_be.get_full_type_name());
  _summary_bold_label2.set_text("0 / 0");

  set_check_title(strfmt(caption_format.c_str(), _filter_be.get_full_type_name().c_str()));

  bec::IconId icon= _filter_be.icon_id(bec::Icon32);

  if (icon != 0)
  {
    std::string icon_path= bec::IconManager::get_instance()->get_icon_path(icon);
    if (!icon_path.empty())
      _icon.set_image(icon_path);
  }
}



void DBObjectFilterFrame::set_models(bec::GrtStringListModel *model, bec::GrtStringListModel *excl_model)
{
  _model= model;
  _exclude_model= excl_model;
  _filter_be.filter_model(_exclude_model);

  if (_model->total_items_count() == 0)
    set_active(false);
  else
    set_active(true);

  refresh();
}


void DBObjectFilterFrame::refresh()
{
  _model->refresh();
  _exclude_model->refresh();
  
  refill_list(_object_list, _model);
  refill_list(_mask_list, _exclude_model);

  _summary_bold_label2.set_text(strfmt("%i / %i", _model->active_items_count(), _model->total_items_count()));

  update_button_enabled();
}


void DBObjectFilterFrame::update_button_enabled()
{
  _add1_button.set_enabled(_object_list.get_selected_index() >= 0);
  _del1_button.set_enabled(_mask_list.get_selected_index() >= 0);
}


void DBObjectFilterFrame::show_detailed()
{
  _summary_table.show(false);
  _detailed_table.show();
}


void DBObjectFilterFrame::hide_detailed()
{
  _detailed_table.show(false);
  _summary_table.show();
}


void DBObjectFilterFrame::add_mask()
{
  mforms::View *prev= this, *view= get_parent();
  
  while (view)
  {
    prev= view;
    view= view->get_parent();
  }

  TextInputDialog dlg((mforms::Form*)prev);
  
  dlg.set_description("Pattern mask for objects to be ignored.\nYou may use wildcards such as * and ?");
  dlg.set_caption("Enter Pattern Mask:");

  if (dlg.run())
  {
    _exclude_model->add_item(dlg.get_value(), -1);
    _model->invalidate();
    
    refresh();
  }
}


void DBObjectFilterFrame::add_clicked(bool all)
{
  _filter_combo.set_selected(-1);

  std::vector<int> indices;

  if (all)
  {
    for (int c= _model->count(), i= 0; i < c; i++)
      indices.push_back(i);
  }
  else
    indices.push_back(_object_list.get_selected_index());

  _model->copy_items_to_val_masks_list(indices);
  _model->invalidate();

  refresh();
}


void DBObjectFilterFrame::del_clicked(bool all)
{
  _filter_combo.set_selected(-1);

  std::vector<int> indices;

  if (all)
  {
    for (int c= _exclude_model->count(), i= 0; i < c; i++)
      indices.push_back(i);
  }
  else
    indices.push_back(_mask_list.get_selected_index());

  _exclude_model->remove_items(indices);
  _model->invalidate();

  refresh();
}

