#ifndef _GUI_PLUGIN_BASE_H_
#define _GUI_PLUGIN_BASE_H_

#include "grt/grt_manager.h"

//! Base class for loadable GUI plugins
//! This object is created by plugins when the frontend calls the plugin entry point
//! function (that implements GUIPluginCreateFunction)
class GUIPluginBase
{
  bec::GRTManager *_grtm;

protected:
  bec::GRTManager *grtm() { return _grtm; }
 
  sigc::signal<void> _signal_closed; 
 
public:
  GUIPluginBase(bec::GRTManager* mgr, const grt::BaseListRef &args);
  virtual ~GUIPluginBase();
  
  sigc::signal<void> signal_closed() { return _signal_closed; }

  virtual void execute()= 0;
  
  virtual void show_plugin() {}
  virtual void hide_plugin() {}
};


typedef GUIPluginBase *(*GUIPluginCreateFunction)(bec::GRTManager* mgr, const grt::BaseListRef &args);


#endif /* _WB_PLUGIN_BASE_H_ */
