#ifndef _WIZARDPROGRESSPAGE_H_
#define _WIZARDPROGRESSPAGE_H_

#include "grt_wizard_form.h"

namespace grtui {

  class GRTUI_EXPORT WizardProgressPage : public WizardPage
  {
  public:
    WizardProgressPage(WizardForm *form, const std::string &id);
    virtual ~WizardProgressPage();

    virtual ::mforms::View *get_advanced_panel() { return &_log_panel; }

    sigc::signal<void,bool> &signal_tasks_finished() { return _signal_finished; }

  protected:
    enum TaskState {
      StateNormal,
      StateBusy,
      StateDone,
      StateError,
      StateDisabled
    };

    struct TaskRow
    {
      mforms::Box box;
      mforms::ImageBox icon;
      mforms::Label label;
      sigc::slot<bool> execute; //! return value indicates whether an asynchronous function was actually executed
      sigc::slot<bool> process_fail; //! return value indicates whether it can continue executing ok
      sigc::slot<void, grt::ValueRef> process_finish;
      std::string status_text;
      bool enabled;
      bool async;
      bool async_running;
      bool async_failed;
      bool async_errors;

      TaskRow() : box(true), enabled(true), async(false), async_running(false), async_failed(false), async_errors(0) {}
      
      void set_state(TaskState state);
      void set_enabled(bool flag);
    };

    mforms::Panel _progress;
    mforms::Box _content;
    mforms::Label _label1;
    mforms::Label _label2;

    std::vector<TaskRow*> _tasks;

    std::string _finish_message;

    mforms::Label _status_text;

    mforms::Box *_progress_box;
    mforms::ProgressBar *_progress_bar;
    mforms::Label *_progress_label;

    mforms::Panel _log_panel;
    mforms::TextBox _log_text;

    sigc::signal<void,bool> _signal_finished;

    int _current_task;
    bool _busy;
    bool _done;
    bool _got_warning_messages;
    bool _got_error_messages;
    bool _preview_before_execute;
    

    TaskRow *add_async_task(const std::string &caption,
                            const sigc::slot<bool> &execute,
                            const std::string &status_text);

    TaskRow *add_task(const std::string &caption,
                      const sigc::slot<bool> &execute,
                      const std::string &status_text);

    TaskRow *add_disabled_task(const std::string &caption);

    void end_adding_tasks(bool add_progressbar, 
                          const std::string &finish_message,
                          bool preview_tasks_before_execute= true);
    
    void reset_tasks();

    void start_tasks();

    void set_status_text(const std::string &text, bool is_error= false);

    void update_progress(float pct, const std::string &caption);

    void add_log_text(const std::string &text);
    
  private:
    TaskRow *add_task(bool async, 
                      const std::string &caption,
                      const sigc::slot<bool> &execute,
                      const std::string &status_text);

  public:
    void execute_grt_task(const sigc::slot<grt::ValueRef,grt::GRT*> &slot, bool sync);

    void process_grt_task_message(const grt::Message &msg);
    void process_grt_task_fail(const std::exception &error);
    void process_grt_task_finish(const grt::ValueRef &result);

  protected:
    void perform_tasks();

    virtual bool allow_cancel();
    virtual bool allow_next();

    virtual void execute();

    virtual bool override_next_action();
    virtual bool execute_overriden_next_action();
    virtual std::string next_button_caption();
  };

};
  
  
#endif /* _WIZARDPROGRESSPAGE_H_ */
