#include "wizard_schema_filter_page.h"

#include "grt/icon_manager.h"
#include "grt/common.h"
#include "grt/grt_manager.h"

/**
 * @file  wizard_schema_filter_page.cpp
 * @brief 
 */

using namespace grtui;

WizardSchemaFilterPage::WizardSchemaFilterPage(WizardForm *form, const char *name)
  : WizardPage(form, name), _header(true)
{
  _header.set_spacing(4);
    
  _image.set_image(bec::IconManager::get_instance()->get_icon_path("db.Schema.32x32.png"));
  _header.add(&_image, false);
    
  _label.set_text_align(mforms::BottomLeft);
  _label.set_text(_("Schemata to Process:"));
  _label.set_style(mforms::BoldStyle);
  _header.add(&_label, true, true);
  
  add(&_header, false, false);
    
  add(&_check_list, true, true);

  _check_list.signal_changed().connect(sigc::mem_fun(this, &WizardSchemaFilterPage::validate));
  
  signal_leave().connect(sigc::mem_fun(this, &WizardSchemaFilterPage::leave));
}
 

void WizardSchemaFilterPage::load()
{
  _check_list.set_strings(grt::StringListRef::cast_from(values().get("schemata")));
}


void WizardSchemaFilterPage::leave(grt::DictRef values)
{
  grt::StringListRef list(_form->grtm()->get_grt());
  std::vector<std::string> selection= _check_list.get_selection();
  
  for (std::vector<std::string>::const_iterator iter= selection.begin();
       iter != selection.end(); ++iter)
    list.insert(*iter);
  
  values.set("selectedSchemata", list);
}


bool WizardSchemaFilterPage::allow_next()
{
  return _check_list.has_selection();
}
