

#include "grt_wizard_form.h"
#include "grtpp_util.h"
#include "grt/common.h"
#include "wizard_view_text_page.h"

using namespace grtui;

ViewTextPage::ViewTextPage(WizardForm *form, const char *name, const std::string &filetype)
  : WizardPage(form, name), _scroll(true), _button_box(true), _filetype(filetype)
{
  add(&_scroll, true, true);
  _scroll.add(&_text);

  if (!_filetype.empty())
  {
    add(&_button_box, false, false);
    _button_box.add_end(&_save_button, false, true);
    _save_button.set_size(80, -1);
    _save_button.signal_clicked().connect(sigc::mem_fun(this, &ViewTextPage::clicked));
  }

  _save_button.set_text(_("Save to File..."));
}


void ViewTextPage::set_text(const std::string &text)
{
  _text.set_value(text);
}


std::string ViewTextPage::get_text()
{
  return _text.get_string_value();
}


void ViewTextPage::set_editable(bool flag)
{
  _text.set_read_only(!flag);
}


void ViewTextPage::clicked()
{
  mforms::FileChooser fsel(mforms::SaveFile);

  fsel.set_extensions(_filetype, _filetype);

  if (fsel.run_modal())
  {
    std::string text= _text.get_string_value();
    GError *error= 0;
    
    if (!g_file_set_contents(fsel.get_path().c_str(), text.data(), text.size(), &error))
    {
      mforms::Form::show_error(_("Save to File"),
                               strfmt(_("Could not save to file: %s"), error->message),
                               _("OK"));
      g_error_free(error);
    }
  }
}


void ViewTextPage::save_text_to(const std::string &path)
{
  char *filename= g_filename_from_utf8(path.c_str(), -1, NULL, NULL, NULL);
  std::string text= get_text();
  GError *error= NULL;

  if (!g_file_set_contents(filename, text.data(), text.size(), &error))
  {
    g_free(filename);
    std::string msg= strfmt(_("Could not save to file '%s': %s"), path.c_str(),
                            error->message);
    g_error_free(error);

    throw grt::os_error(msg);
  }
  g_free(filename);
}
