
#ifndef _CONNECTION_FIGURE_H_
#define _CONNECTION_FIGURE_H_


#include "mdc.h"

namespace wbfig {
  class ConnectionLineLayouter : public mdc::OrthogonalLineLayouter
  {
    typedef mdc::OrthogonalLineLayouter super;

    virtual std::vector<mdc::ItemHandle*> create_handles(mdc::Line *line, mdc::InteractionLayer *ilayer);

    virtual bool update_start_point();
    virtual bool update_end_point();

    virtual bool handle_dragged(mdc::Line *line, mdc::ItemHandle *handle, const mdc::Point &pos, bool dragging);


  public:
    ConnectionLineLayouter(mdc::Connector *sconn, mdc::Connector *econn);

  };


  class Connection : public mdc::Line
  {
    typedef mdc::Line super;
  public:
    enum DiamondType
    {
      None,
      Filled,
      LeftEmpty,
      RightEmpty,
      Empty
    };
    enum CaptionPos
    {
      Below,
      Above,
      Middle
    };

    Connection(mdc::Layer *layer);

    void set_splitted(bool flag);

    mdc::Point get_middle_caption_pos(const mdc::Size &size, CaptionPos pos);
    mdc::Point get_start_caption_pos(const mdc::Size &size);
    mdc::Point get_end_caption_pos(const mdc::Size &size);

    void set_start_dashed(bool flag);
    void set_end_dashed(bool flag);
    virtual void render(mdc::CairoCtx *cr);

    void set_start_figure(mdc::CanvasItem *item);
    void set_end_figure(mdc::CanvasItem *item);

    void set_diamond_type(DiamondType type);

    double get_segment_offset(int subline);
    void set_segment_offset(int subline, double offset);

    virtual bool on_click(mdc::CanvasItem *target, const mdc::Point &point, mdc::MouseButton button, mdc::EventState state);
    virtual bool on_enter(mdc::CanvasItem *target, const mdc::Point &point);
    virtual bool on_leave(mdc::CanvasItem *target, const mdc::Point &point);

    sigc::signal<void, bool> signal_activate() { return _signal_activate; }
    sigc::signal<void, bool> signal_crossing() { return _signal_crossing; }

    void set_center_captions(bool flag);
    bool get_center_captions() { return _center_captions; }

  private:
    mdc::CanvasItem *_start_figure;
    mdc::CanvasItem *_end_figure;

    mdc::Timestamp _last_click;
    DiamondType _diamond;
    bool _start_dashed;
    bool _end_dashed;
    bool _splitted;
    bool _center_captions;

    sigc::signal<void, bool> _signal_activate;
    sigc::signal<void, bool> _signal_crossing;

    virtual bool contains_point(const mdc::Point &point) const;

    virtual void stroke_outline(mdc::CairoCtx *cr, float offset= 0) const;
    //virtual void create_handles(mdc::InteractionLayer *ilayer);

    double get_middle_segment_angle();

    void update_layouter();

    virtual void mark_crossings(mdc::Line *line);
  };

};


#endif
