#ifndef _FIGURE_COMMON_H_
#define _FIGURE_COMMON_H_

#include "mdc.h"

#include <algorithm>

#include "wb_canvas_bridge_public_interface.h"


mdc::FontSpec parse_font_spec(const std::string &font);


namespace wbfig {

class MYSQLWBCANVASBRIDGE_PUBLIC Titlebar : public mdc::Box
{
public:
  Titlebar(mdc::Layer *layer, bool expander);
  virtual ~Titlebar();

  void set_icon(cairo_surface_t *icon);
  void set_title(const std::string &text);
  
  inline const std::string &get_title() const { return _icon_text.get_text(); }

  void set_color(const mdc::Color &color);
  void set_text_color(const mdc::Color &color);
  void set_font(const mdc::FontSpec &font);
  const mdc::FontSpec &get_font() { return _icon_text.get_font(); }
  void set_rounded(mdc::CornerMask corners);
  void set_border_color(const mdc::Color &color);

  void set_expanded(bool flag);
  bool get_expanded();

  virtual void set_auto_sizing(bool flag);

  void auto_size() { _icon_text.auto_size(); }

  sigc::signal<void,bool> signal_expand_toggle() { return _signal_expand_toggle; }
  sigc::signal<void,bool> signal_activate() { return _signal_activate; }

protected:
  mdc::IconTextFigure _icon_text;
  mdc::Button *_expander;
  mdc::Timestamp _last_click;

  mdc::Color _back_color;
  mdc::CornerMask _corners;
  mdc::Color _border_color;

  sigc::signal<void,bool> _signal_expand_toggle; 
  sigc::signal<void,bool> _signal_activate;
  
  void expand_toggled();

  virtual void render(mdc::CairoCtx *cr);

  virtual bool on_click(mdc::CanvasItem *target, const mdc::Point &point, mdc::MouseButton button, mdc::EventState state);
};


class MYSQLWBCANVASBRIDGE_PUBLIC TextFigure : public mdc::TextFigure
{
  sigc::signal<void, bool> _signal_crossing;

  virtual bool on_click(mdc::CanvasItem *target, const mdc::Point &point, mdc::MouseButton button, mdc::EventState state);
  virtual bool on_enter(mdc::CanvasItem *target, const mdc::Point &point);
  virtual bool on_leave(mdc::CanvasItem *target, const mdc::Point &point);

public:
  TextFigure(mdc::Layer *layer);
  sigc::signal<void, bool> signal_crossing() { return _signal_crossing; }
};



class MYSQLWBCANVASBRIDGE_PUBLIC FigureItem : public mdc::IconTextFigure
{
  typedef mdc::IconTextFigure super;

  std::string _object_id;
  bool _dirty;

  virtual void draw_state(mdc::CairoCtx *cr);
  virtual mdc::Point get_intersection_with_line_to(const mdc::Point &p);
  virtual mdc::Rect get_root_bounds() const;

public:
  FigureItem(mdc::Layer *layer);

  std::string get_id() { return _object_id; }
  void set_id(const std::string &id) { _object_id= id; }
  void set_dirty(bool flag= true) { _dirty= flag; }
  bool get_dirty() { return _dirty; }
};


class MYSQLWBCANVASBRIDGE_PUBLIC BaseFigure : public mdc::Box
{
  typedef mdc::Box super;

public:
  typedef std::list<FigureItem*> ItemList;

  // default implementation just sets background color
  virtual void unset_color();
  virtual void set_color(const mdc::Color &color);

  virtual void set_title_font(const mdc::FontSpec &font) {}
  virtual void set_content_font(const mdc::FontSpec &font);

  virtual void set_allow_manual_resizing(bool flag);

  sigc::signal<void, bool> signal_activate() { return _signal_activate; }

  sigc::signal<void,mdc::Rect> signal_interactive_resize() { return _signal_resize; }

  sigc::signal<void,FigureItem*> signal_item_added() { return _signal_item_added; }

  virtual void toggle(bool flag) {}
  virtual void set_state_drawing(bool flag);

protected:
  BaseFigure(mdc::Layer *layer);

  sigc::signal<void, bool> _signal_activate;
  sigc::signal<void,mdc::Rect> _signal_resize;
  sigc::signal<void,FigureItem*> _signal_item_added;
  mdc::Timestamp _last_click;

  mdc::Rect _initial_bounds;
  mdc::FontSpec _content_font;
  bool _manual_resizing;
  bool _resizing;

  void invalidate_min_sizes();
  static void invalidate_min_sizes(mdc::CanvasItem *item);
  
  typedef sigc::slot<FigureItem*, mdc::Layer*> CreateItemSlot;
  typedef sigc::slot<void,FigureItem*> UpdateItemSlot;

  virtual ItemList::iterator begin_sync(mdc::Box &box, ItemList &list);
  virtual ItemList::iterator sync_next(mdc::Box &box, ItemList &list,
                               ItemList::iterator iter,
                               const std::string &id,
                               cairo_surface_t *icon,
                               const std::string &text,
                               const CreateItemSlot &create_item= CreateItemSlot(),
                               const UpdateItemSlot &update_item= UpdateItemSlot());
  

  virtual void end_sync(mdc::Box &box, ItemList &list, ItemList::iterator iter);


  virtual bool on_click(mdc::CanvasItem *target, const mdc::Point &point, mdc::MouseButton button, mdc::EventState state);
  
  virtual bool on_drag_handle(mdc::ItemHandle *handle, const mdc::Point &pos, bool dragging);
};

};

#endif /* _FIGURE_COMMON_H_ */
