#include "stdafx.h"

#include "image_figure.h"

using namespace wbfig;

Image::Image(mdc::Layer *layer) : BaseFigure(layer), _image(layer)
{
  set_cache_toplevel_contents(false);
  set_accepts_focus();
  set_accepts_selection();
  set_allowed_resizing(true, true);

  add(&_image, true, true, false); 

  _keep_aspect_ratio= false;
}


static void constrain_aspect_ratio(mdc::ItemHandle *handle, mdc::Size &size, double ratio)
{
  if ((handle->get_tag() & (HDL_LEFT|HDL_RIGHT)) != 0)
    size.height= size.width/ratio;
  else
    size.width= size.height*ratio;
}


void Image::keep_aspect_ratio(bool flag)
{
  _keep_aspect_ratio= flag;

  if (flag)
  {
    cairo_surface_t *surf= _image.get_image();

    if (surf)
    {
      double aspect= get_aspect_ratio();
      double width= _image.get_size().width;
      double new_height= width / aspect;

      if (fabs(_image.get_size().height-new_height) > 1)
        set_fixed_size(mdc::Size(width, new_height));

      set_drag_handle_constrainer(sigc::bind<double>(sigc::ptr_fun(constrain_aspect_ratio), aspect));
    }
  }
  else
    set_drag_handle_constrainer(sigc::slot<void,mdc::ItemHandle*,mdc::Size&>());
}


double Image::get_aspect_ratio()
{
  mdc::Size size= _image.get_image_size();

  return size.width / size.height;
}


void Image::set_image(cairo_surface_t *image)
{
  _image.set_image(image);
}


bool Image::set_image(const std::string &filename)
{
  return _image.set_image(filename);
}


void Image::set_allow_manual_resizing(bool flag)
{
  _image.set_auto_sizing(!flag);

  if (!flag)
    relayout();
}


