#include "stdafx.h"

#include "model_figure_impl.h"
#include "model_diagram_impl.h"
#include "model_layer_impl.h"
#include "model_model_impl.h"

#include "grt/common.h"
#include "grt/grt_manager.h"
#include <grtpp_undo_manager.h>


model_Figure::ImplData::ImplData(model_Figure *owner)
  : model_Object::ImplData(owner), _realizing(false)
{
  _connected_update_options= false;
  
  owner->signal_changed().connect(sigc::mem_fun(this, &model_Figure::ImplData::member_changed));
}


void model_Figure::ImplData::update_options(const std::string &key)
{
  
}


void model_Figure::ImplData::set_layer(const model_LayerRef &nlayer)
{
  model_LayerRef oldLayer(self()->_layer);
  
  if (get_canvas_view())
    get_canvas_view()->lock_redraw();
  
  self()->_layer= nlayer;
  
  if (self()->_layer.is_valid())
  {
    mdc::CanvasItem *figure= get_canvas_item();
    
    model_Layer::ImplData *layer= nlayer->get_data();
    mdc::AreaGroup *ag= 0;
    if (layer)
      ag= layer->get_area_group();
    
    if (oldLayer.is_valid())
    {
      // update the position of the figure
      self()->_top= *self()->_top - *self()->_layer->top() + *oldLayer->top();
      self()->_left= *self()->_left - *self()->_layer->left() + *oldLayer->left();
    }
    else
    {
      self()->_top= *self()->_top - *self()->_layer->top();
      self()->_left= *self()->_left - *self()->_layer->left();
    }
    
    if (figure && ag)
    {        
      // will remove from old group if needed
      ag->add(figure);
      
      figure->move_to(mdc::Point(*self()->_left, *self()->_top));
    }
  }
  if (get_canvas_view())
    get_canvas_view()->unlock_redraw();

  try_realize();
}


void model_Figure::ImplData::member_changed(const std::string &name, const grt::ValueRef &ovalue)
{
  if (name == "name")
  {
    if (_in_view)
    {
      self()->owner()->signal_refreshDisplay().emit(self());
    }
  }
  else if (name == "color")
  {
    if (get_canvas_item())
    {
      if (self()->_color.is_valid() && *self()->_color.c_str())
        get_canvas_item()->set_color(mdc::Color::parse(*self()->_color));
      else
        get_canvas_item()->unset_color();
    }
  }
  else if (get_canvas_item()) // only valid if realized
  {
    if (name == "height")
    {
      mdc::Size size(get_canvas_item()->get_size());
      mdc::Size min_size(get_canvas_item()->get_min_size());

      self()->_manualSizing= 1;
      dynamic_cast<wbfig::BaseFigure*>(get_canvas_item())->set_allow_manual_resizing(true);

      if (*self()->_height < min_size.height)
        self()->_height= min_size.height;
     
      get_canvas_view()->lock();

      if (*self()->_height != size.height)
      {
        size.height= *self()->_height;
        if (*self()->_manualSizing)
          get_canvas_item()->set_fixed_size(size);
        else
          get_canvas_item()->resize_to(size);
      }

      get_canvas_view()->unlock();
    }
    else if (name == "width")
    {
      mdc::Size size(get_canvas_item()->get_size());
      mdc::Size min_size(get_canvas_item()->get_min_size());

      self()->_manualSizing= 1;
      dynamic_cast<wbfig::BaseFigure*>(get_canvas_item())->set_allow_manual_resizing(true);

      if (*self()->_width < min_size.width)
        self()->_width= min_size.width;

      get_canvas_view()->lock();

      if (*self()->_width != size.width)
      {
        size.width= *self()->_width;
        if (*self()->_manualSizing)
          get_canvas_item()->set_fixed_size(size);
        else
          get_canvas_item()->resize_to(size);
      }

      get_canvas_view()->unlock();
    }
    else if (name == "left")
    {      
      get_canvas_view()->lock();
      mdc::Point pos= get_canvas_item()->get_position();
      if (*self()->_left != pos.x)
      {
        pos.x= *self()->_left;
        get_canvas_item()->move_to(pos);
      }
      get_canvas_view()->unlock();
    }
    else if (name == "top")
    {      
      get_canvas_view()->lock();
      mdc::Point pos= get_canvas_item()->get_position();
      if (*self()->_top != pos.y)
      {
        pos.y= *self()->_top;
        get_canvas_item()->move_to(pos);
      }
      get_canvas_view()->unlock();
    }
    else if (name == "manualSizing")
    {
      dynamic_cast<wbfig::BaseFigure*>(get_canvas_item())->set_allow_manual_resizing(*self()->_manualSizing != 0);
    }
    else if (name == "expanded")
    {
      if (get_canvas_item())
        get_canvas_item()->toggle(*self()->_expanded!=0);
    }
    else if (name == "visible")
    {
      if (*self()->_visible)
        try_realize();
      else
        unrealize();
    }
  }
}


bool model_Figure::ImplData::is_realizable()
{
  if (_in_view && *self()->_visible && self()->layer().is_valid())
  {
    model_Layer::ImplData *layer= self()->_layer->get_data();
    model_Diagram::ImplData *view= self()->owner()->get_data();
  
    if (layer && layer->get_area_group() && view && view->get_canvas_view())
      return true;
  }
  return false;
}


void model_Figure::ImplData::figure_bounds_changed(const mdc::Rect &rect)
{
  mdc::Rect bounds= get_canvas_item()->get_bounds();

  self()->_left= grt::DoubleRef(bounds.xmin());
  self()->_top= grt::DoubleRef(bounds.ymin());
  self()->_width= grt::DoubleRef(bounds.width());
  self()->_height= grt::DoubleRef(bounds.height());
}


void model_Figure::ImplData::figure_resized(const mdc::Rect &rect)
{
  mdc::Rect bounds= get_canvas_item()->get_bounds();

  model_Model::ImplData *model= self()->owner()->owner()->get_data();
  bool skip_undo= true;
  if (model && !_realizing && rect != bounds)
    skip_undo= false;

  grt::AutoUndo undo(self()->get_grt(), skip_undo);

  self()->left(grt::DoubleRef(bounds.xmin()));
  self()->top(grt::DoubleRef(bounds.ymin()));
  self()->width(grt::DoubleRef(bounds.width()));
  self()->height(grt::DoubleRef(bounds.height()));

  self()->manualSizing(1);
  dynamic_cast<wbfig::BaseFigure*>(get_canvas_item())->set_allow_manual_resizing(true);
    
  undo.end(bec::fmt("Resize '%s'", self()->_name.c_str()));
}


void model_Figure::ImplData::finish_realize()
{
  mdc::Size size;
  mdc::Point pos;
  wbfig::BaseFigure *figure= get_canvas_item();

  _realizing= true;

  if (!_connected_update_options)
  {
    _connected_update_options= true;
    self()->owner()->owner()->get_data()->signal_options_changed().connect(sigc::mem_fun(this, &model_Figure::ImplData::update_options));
  }

  figure->set_tag(self()->id());

  figure->signal_bounds_changed().connect(sigc::mem_fun(this, &model_Figure::ImplData::figure_bounds_changed));
  figure->signal_interactive_resize().connect(sigc::mem_fun(this, &model_Figure::ImplData::figure_resized));

  pos.x= *self()->_left;
  pos.y= *self()->_top;

  
  if (*self()->_manualSizing)
  {
    figure->set_allow_manual_resizing(true);
    size.width= *self()->_width;
    size.height= *self()->_height;
    figure->set_fixed_size(size);
  }
  else
  {
    if (*self()->_width > 0 && *self()->_height > 0)
    {
      size.width= *self()->_width;
      size.height= *self()->_height;
      figure->resize_to(size);
    }
    else
    {
      // force relayout and fetch size
      figure->relayout();
      self()->_width= figure->get_size().width;
      self()->_height= figure->get_size().height;
    }
  }
  figure->move_to(pos);

  // check if the figure is in the views selection list and select it if so
  if (self()->owner()->selection().get_index(model_FigureRef(self())) != grt::BaseListRef::npos)
  {
    if (self()->owner()->selection().count() == 1)
      figure->get_view()->get_selection()->set(figure);
    else
      figure->get_view()->get_selection()->add(figure);
  }

  model_Diagram::ImplData *view= self()->owner()->get_data();
  model_Model::ImplData *model= self()->owner()->owner()->get_data();
  if (model)
  {
    std::string font;
    
    font= model->get_string_option(bec::fmt("%s:TitleFont", self()->class_name().c_str()), "");
    if (!font.empty())
      figure->set_title_font(parse_font_spec(font));

    font= model->get_string_option(bec::fmt("%s:ItemsFont", self()->class_name().c_str()), "");
    if (font.empty())
      font= model->get_string_option(bec::fmt("%s:TextFont", self()->class_name().c_str()), "");
    if (!font.empty())
      figure->set_content_font(parse_font_spec(font));
  }

  view->stack_figure(self(), get_canvas_item());

  _realizing= false;
}


void model_Figure::ImplData::activate(bool control)
{
  if (self()->owner().is_valid())
  {
    self()->owner()->signal_objectActivated().emit(self(), control?1:0);
  }
}


void model_Figure::ImplData::render_mini(mdc::CairoCtx *cr)
{
  cr->set_color(mdc::Color::parse(*self()->_color));
  cr->rectangle(*self()->_left, *self()->_top, *self()->_width, *self()->_height);
  cr->fill();
}


void model_Figure::ImplData::unrealize()
{
  if (get_canvas_item())
  {
    get_canvas_item()->get_view()->remove_item(get_canvas_item());
  }
}


mdc::CanvasView* model_Figure::ImplData::get_canvas_view() const
{
  if (self()->owner().is_valid())
  {
    model_Diagram::ImplData *view= self()->owner()->get_data();
    if (view)
      return view->get_canvas_view();
  }
  return 0;
}

