#ifndef _MODEL_FIGURE_IMPL_H_
#define _MODEL_FIGURE_IMPL_H_

#include <mdc.h>
#include <grtpp.h>

#ifdef _WIN32
#include <hash_map>
#define hash_map stdext::hash_map
#else
#include <ext/hash_map>
#define hash_map __gnu_cxx::hash_map
#endif

#include "grts/structs.model.h"

#include "wb_canvas_bridge_public_interface.h"

#include "model_object_impl.h"

#include "figure_common.h"

#ifdef __GNUC__
namespace __gnu_cxx
{
  using std::size_t;

  template<>
    struct hash<grt::internal::Object*>
    {
      size_t
      operator()(grt::internal::Object* __p) const
      { return (intptr_t)__p; }
    };
};
#endif

namespace bec {
  class UndoAction;
};


class MYSQLWBCANVASBRIDGE_PUBLIC model_Figure::ImplData : public model_Object::ImplData
{
  typedef BridgeBase super;

  friend class model_Diagram::ImplData;

protected:
  bool _resizable;
  bool _connected_update_options;
  bool _realizing;
  
  ImplData(model_Figure *owner);

  virtual void update_options(const std::string &key);
  
  void finish_realize();

  void member_changed(const std::string &member, const grt::ValueRef &ovalue);
  
  void figure_resized(const mdc::Rect &rect);
  void figure_bounds_changed(const mdc::Rect &rect);
  
  virtual bool is_realizable();
  
public:      
  void activate(bool control);
    
  mdc::CanvasView* get_canvas_view() const;
  
  void set_layer(const model_LayerRef &layer);
  
public:  
  virtual wbfig::BaseFigure* get_canvas_item() const= 0;
  
  virtual void render_mini(mdc::CairoCtx *cr);
  virtual void unrealize();
  
private:
  model_Figure *self() const { return (model_Figure*)_self; }
};


#endif // _MODEl_FIGURE_IMPL_H_
