#include "stdafx.h"

#include "grt/grt_manager.h"

#include "model_model_impl.h"

#include "model_diagram_impl.h"
#include "model_connection_impl.h"
#include "model_figure_impl.h"

#include "grts/structs.workbench.h"


model_Model::ImplData::ImplData(model_Model *owner)
  : _owner(owner)
{
  _reset_pending= false;
  
  
  GrtObjectRef object(_owner);
  while (object.is_valid() && !object.is_instance<app_Application>())
    object= object->owner();

  if (object.is_valid())
    app_ApplicationRef::cast_from(object)->options()->signal_dict_changed()
      .connect(sigc::mem_fun(this, &ImplData::option_changed));

  _owner->signal_list_changed().connect(sigc::mem_fun(this, &ImplData::list_changed));
}


void model_Model::ImplData::list_changed(grt::internal::OwnedList *list, bool added, const grt::ValueRef &value)
{
  if (!added && list == _owner->_diagrams.valueptr())
  {
    remove_diagram(model_DiagramRef::cast_from(value));
  }
}

void model_Model::ImplData::option_changed(grt::internal::OwnedDict *dict,
                                            bool added,
                                            const std::string &option)
{
  _options_changed_signal.emit(option);

  if (!_reset_pending && bec::has_suffix(option, "Font"))
  {
    _reset_pending= true;
    run_later(sigc::mem_fun(this, &model_Model::ImplData::reset_figures));
    run_later(sigc::mem_fun(this, &model_Model::ImplData::reset_connections));
  }
}

/*
void model_Model::ImplData::add_view(const model_DiagramRef &view)
{
  _owner->_diagrams.insert(view);

  view->get_data()->realize();
}
*/

void model_Model::ImplData::remove_diagram(const model_DiagramRef &view)
{
  view->get_data()->unrealize();

  _owner->_diagrams.remove_value(view);
}


void model_Model::ImplData::unrealize()
{
  const grt::ListRef<model_Diagram> &views(_owner->_diagrams);

  for (size_t c= views.count(), i= 0; i < c; i++)
  {
    views[i]->get_data()->unrealize();
  }
}


bool model_Model::ImplData::realize()
{
  model_Diagram::ImplData *view;

  // do not use __diagrams because it will get the wrong instance
  grt::ListRef<model_Diagram> views(_owner->_diagrams);

  for (size_t c= views.count(), i= 0; i < c; i++)
  {
    view= views[i]->get_data();
    if (view && !view->get_canvas_view())
      view->realize();
  }
  
  return true;
}


void model_Model::ImplData::reset_connections()
{
  _reset_pending= false;
  grt::ListRef<model_Diagram> views(_owner->_diagrams);

  for (size_t c= views.count(), i= 0; i < c; i++)
  {
    model_DiagramRef view(views[i]);
    grt::ListRef<model_Connection> conns(view->connections());

    for (size_t cc= conns.count(), ci= 0; ci < cc; ++ci)
    {
      model_Connection::ImplData *obj= conns.get(ci)->get_data();
      if (obj && obj->get_canvas_item())
      {
        obj->unrealize();
        obj->realize();
      }
    }
  }
}


void model_Model::ImplData::reset_figures()
{
  _reset_pending= false;
  grt::ListRef<model_Diagram> views(_owner->_diagrams);

  for (size_t c= views.count(), i= 0; i < c; i++)
  {
    model_DiagramRef view(views[i]);
    grt::ListRef<model_Figure> figs(view->figures());

    for (size_t cc= figs.count(), ci= 0; ci < cc; ++ci)
    {
      model_Figure::ImplData *obj= figs.get(ci)->get_data();
      if (obj && obj->get_canvas_item())
      {
        obj->unrealize();
        obj->realize();
      }
    }
  }
}



app_PageSettingsRef model_Model::ImplData::get_page_settings()
{
  GrtObjectRef object(_owner);

  while (object.is_valid() && !object.is_instance<workbench_Document>())
    object= object->owner();

  if (object.is_valid())
    return workbench_DocumentRef::cast_from(object)->pageSettings();

  return app_PageSettingsRef();
}


grt::DictRef model_Model::ImplData::get_options_dict()
{
  GrtObjectRef object(_owner);

  while (object.is_valid() && !object.is_instance<app_Application>())
    object= object->owner();

  if (object.is_valid())
    return app_ApplicationRef::cast_from(object)->options()->options();

  return grt::DictRef();
}



std::string model_Model::ImplData::get_string_option(const std::string &name, const std::string &defvalue)
{
  return _owner->_options.get_string(name, get_options_dict().get_string(name, defvalue));
}


int model_Model::ImplData::get_int_option(const std::string &name, int defvalue)
{
  return _owner->_options.get_int(name, get_options_dict().get_int(name, defvalue));
}
