#ifndef __TABLE_FIGURE_H__
#define __TABLE_FIGURE_H__

#include "wb_config.h"

#include "figure_common.h"
#include <set>
#include "wb_canvas_bridge_public_interface.h"

namespace wbbridge { namespace physical { class TableFigure; }; };

namespace wbfig {


  enum ColumnFlags {
    ColumnPK = (1<<0),
    ColumnFK = (1<<1),
    ColumnNotNull = (1<<2),
    ColumnAutoIncrement = (1<<3),
    ColumnUnsigned = (1<<4),

    ColumnListTruncated = (1<<5)
  };
  
  class Table;

  class TableColumnItem : public FigureItem
  {
    ColumnFlags _flags;

    bool check_column_connection(mdc::Connector *connector);

    virtual mdc::Size calc_min_size();
    virtual void draw_contents(mdc::CairoCtx *cr);

  public:
    TableColumnItem(mdc::Layer *layer);
    
    void set_column_flags(ColumnFlags flags);
  };

  
  class TableIndexItem : public FigureItem
  {
    Table *_owner;

    virtual bool on_enter(mdc::CanvasItem *target, const mdc::Point &point);
    virtual bool on_leave(mdc::CanvasItem *target, const mdc::Point &point);

  public:
    TableIndexItem(mdc::Layer *layer, Table *owner);
  };
  

  class MYSQLWBCANVASBRIDGE_PUBLIC Table : public BaseFigure
  {
    typedef BaseFigure super;
  public:
    Table(mdc::Layer *layer, bool collapsible);

    Titlebar *get_title() { return &_title; }
    virtual Titlebar *get_index_title() { return 0; }
    virtual Titlebar *get_trigger_title() { return 0; }

    void set_show_flags(bool flag);
    
    virtual void set_dependant(bool flag)= 0;

    virtual ItemList *get_columns()= 0;

    virtual ItemList::iterator begin_columns_sync()= 0;
    virtual ItemList::iterator sync_next_column(ItemList::iterator iter,
                                        const std::string &id,
                                        ColumnFlags type,
                                        const std::string &text)= 0;
    virtual void end_columns_sync(ItemList::iterator iter)= 0;

    virtual ItemList *get_indexes() { return 0; }

    virtual ItemList::iterator begin_indexes_sync()= 0;
    virtual ItemList::iterator sync_next_index(ItemList::iterator iter,
                                        const std::string &id,
                                        const std::string &text)= 0;
    virtual void end_indexes_sync(ItemList::iterator iter)= 0;


    virtual ItemList::iterator begin_triggers_sync()= 0;
    virtual ItemList::iterator sync_next_trigger(ItemList::iterator iter,
                                        const std::string &id,
                                        const std::string &text)= 0;
    virtual void end_triggers_sync(ItemList::iterator iter)= 0;

    virtual void set_highlighted(bool flag)
    {
      _background.set_highlighted(flag);
      set_needs_render();
    }

    virtual void set_title_font(const mdc::FontSpec &font);
    virtual void set_section_font(const mdc::FontSpec &font);
    virtual void set_content_font(const mdc::FontSpec &font);

    sigc::signal<void> signal_enter() { return _signal_enter; }
    sigc::signal<void> signal_leave() { return _signal_leave; }

    // index of index, true= enter, false= leave
    sigc::signal<void,int,bool> signal_index_crossed() { return _signal_index_crossed; }


    bool columns_hidden() { return _hide_columns; }
    bool indexes_hidden() { return _hide_indexes; }
    bool triggers_hidden() { return _hide_triggers; }

    mdc::BoxSideMagnet *get_sides_magnet() { return _sides_magnet; }

    virtual void toggle(bool flag) {}
    virtual void toggle_indexes(bool flag) {}
    virtual void toggle_triggers(bool flag) {}

  protected:
    mdc::RectangleFigure _background;
    sigc::signal<void> _signal_enter;
    sigc::signal<void> _signal_leave;
    sigc::signal<void,int,bool> _signal_index_crossed;

    mdc::BoxSideMagnet *_sides_magnet;

    Titlebar _title;
    double _original_column_box_height;

    bool _hide_columns;
    bool _hide_indexes;
    bool _hide_triggers;

    bool _show_flags;

    wbfig::FigureItem *create_column_item(mdc::Layer *layer);
    void update_column_item(wbfig::FigureItem *item, ColumnFlags flags);

    wbfig::FigureItem *create_index_item(mdc::Layer *layer);

    bool compare_connection_position(mdc::Connector *a, mdc::Connector *b, 
      mdc::BoxSideMagnet::Side vertical);

    virtual bool get_expanded() { return true; }

    virtual bool get_indexes_expanded() { return true; }
    virtual bool get_triggers_expanded() { return true; }

    virtual bool on_enter(mdc::CanvasItem *target, const mdc::Point &point);
    virtual bool on_leave(mdc::CanvasItem *target, const mdc::Point &point);
  };
};

#endif

