#include "stdafx.h"

#include "workbench_model_imagefigure_impl.h"

#include "model_layer_impl.h"
#include "model_model_impl.h"
#include "model_diagram_impl.h"


workbench_model_ImageFigure::ImplData::ImplData(workbench_model_ImageFigure *self)
  : super(self), _figure(0)
{
  _resizable= true;
  _last_click= 0;
}


std::string workbench_model_ImageFigure::ImplData::set_filename(const std::string &fn)
{  
  if (fn != *self()->_filename)
  {
    std::string internal_name;
    
    if (fn != "")
    {
      internal_name= self()->owner()->owner()->get_data()->get_delegate()->attach_image(fn);
      if (internal_name.empty())
      {
        g_warning("Image '%s' could not be attached to document.", fn.c_str());
        return "";
      }

      self()->owner()->owner()->get_data()->get_delegate()->release_image(self()->_filename);
    }

    if (_figure)
    {
      cairo_surface_t *img= self()->owner()->owner()->get_data()->get_delegate()->fetch_image(internal_name);

      if (!img)
        g_warning("Could not load image '%s' for '%s'", fn.c_str(),
                  self()->name().c_str());
    
      _figure->set_image(img);

      shrink_if_needed();

      self()->_width= _figure->get_size().width;
      self()->_height= _figure->get_size().height;
    }

    self()->_filename= internal_name;
    
    return internal_name;
  }
  else
    return self()->_filename;
}


void workbench_model_ImageFigure::ImplData::set_keep_aspect_ratio(bool flag)
{
  self()->_keepAspectRatio= flag ? 1 : 0;
  
  if (_figure)
    dynamic_cast<wbfig::Image*>(_figure)->keep_aspect_ratio(*self()->_keepAspectRatio!=0);
}
  

/*  else if (_figure && name == "width" && *__keepAspectRatio!=0)
  {
    double ratio= dynamic_cast<wbfig::Image*>(_figure)->get_aspect_ratio();

    __width= grt::DoubleRef::cast_from(value);
    __height= *__width / ratio;
    if (__height < 1)
      __height= 1;

    _figure->set_fixed_size(mdc::Size(__width, __height));
  }
  else if (_figure && name == "height" && *__keepAspectRatio!=0)
  {
    double ratio= dynamic_cast<wbfig::Image*>(_figure)->get_aspect_ratio();

    __height= grt::DoubleRef::cast_from(value);
    __width= *__height * ratio;
    if (__width < 1)
      __width= 1;

    _figure->set_fixed_size(mdc::Size(__width, __height));
  }*/


void workbench_model_ImageFigure::ImplData::unrealize()
{
  super::unrealize();
  
  delete _figure;
  _figure= 0;
}


bool workbench_model_ImageFigure::ImplData::shrink_if_needed()
{
  mdc::Size size(_figure->calc_min_size());
  mdc::Size max_size(get_canvas_view()->get_total_view_size());
  bool resized= false;

  max_size.width-= 20;
  max_size.height-= 20;

  // shrink image if too big
  if (size.width > max_size.width)
  {
    size.width= max_size.width;
    resized= true;
  }
  if (size.height > max_size.height)
  {
    size.height= max_size.height;
    resized= true;
  }
  if (resized)
  {
    self()->_manualSizing= 1;
    _figure->set_fixed_size(size);
  }
  else
    _figure->resize_to(size);

  return resized;
}


bool workbench_model_ImageFigure::ImplData::realize()
{
  if (_figure) return true;
  if (!is_realizable()) return false;
  
  if (!is_main_thread())
  {
    run_later(sigc::hide_return(sigc::mem_fun(this, &ImplData::realize)));
    return true;
  }
  
  if (!_figure)
  {
    mdc::CanvasView *view= self()->owner()->get_data()->get_canvas_view();
    mdc::AreaGroup *agroup;
  
    view->lock();
    
    wbfig::Image *image= _figure= new wbfig::Image(view->get_current_layer());
    image->keep_aspect_ratio(*self()->_keepAspectRatio!=0);

    image->signal_activate().connect(sigc::mem_fun(this, &super::activate));

    agroup= self()->layer()->get_data()->get_area_group();
    
    view->get_current_layer()->add_item(_figure, agroup);
    
    cairo_surface_t *img= self()->owner()->owner()->get_data()->get_delegate()->fetch_image(self()->_filename);
    if (!img)
      g_warning("Could not load image '%s' for '%s'", self()->_filename.c_str(),
                self()->name().c_str());

    _figure->set_image(img);

    if (shrink_if_needed())
    {
      self()->_width= _figure->get_size().width;
      self()->_height= _figure->get_size().height;
    }

    finish_realize();
    
    view->unlock();

    self()->owner()->get_data()->notify_object_realize(self());
  }
  return true;
}
