
#include "stdafx.h"

#include "workbench_physical_routinegroupfigure_impl.h"
#include "workbench_physical_model_impl.h"

#include "workbench_physical_diagram_impl.h"
#include "model_layer_impl.h"


workbench_physical_RoutineGroupFigure::ImplData::ImplData(workbench_physical_RoutineGroupFigure *self)
  : super(self), _figure(0)
{
  _resizable= false;
  
  self->signal_changed().connect(sigc::mem_fun(this, &ImplData::member_changed));
}


void workbench_physical_RoutineGroupFigure::ImplData::routinegroup_member_changed(const std::string &name, const grt::ValueRef &ovalue)
{
  if (name == "name")
  {
    self()->_name= self()->_routineGroup->name();

    if (_figure)
      _figure->set_title(*self()->_name, bec::fmt("%i routines", self()->_routineGroup->routines().count()));
  }
}


void workbench_physical_RoutineGroupFigure::ImplData::contents_changed()
{
  sync_routines();
}



void workbench_physical_RoutineGroupFigure::ImplData::set_in_view(bool flag)
{
  if (!self()->owner().is_valid())
    throw std::logic_error("adding figure to view before setting owner");

  if (flag)
  {
    if (self()->_routineGroup.is_valid())
      workbench_physical_DiagramRef::cast_from(self()->_owner)->get_data()->add_mapping(self()->_routineGroup, self());
  }
  else
  {
    if (self()->_routineGroup.is_valid())
      workbench_physical_DiagramRef::cast_from(self()->_owner)->get_data()->remove_mapping(self()->_routineGroup);
  }
  
  model_Figure::ImplData::set_in_view(flag);
}


void workbench_physical_RoutineGroupFigure::ImplData::set_routine_group(const db_RoutineGroupRef &rgroup)
{
  self()->_routineGroup= rgroup;
  
  //if (_figure_conn)
//    _figure_conn.disconnect();
  
  if (self()->routineGroup().is_valid())
  {
    if (self()->_owner.is_valid())
      workbench_physical_DiagramRef::cast_from(self()->_owner)->get_data()->add_mapping(rgroup, self());

    //_figure_conn= self()->routineGroup()->signal_changed().connect(sigc::mem_fun(this, &ImplData::rgroup_member_changed));
    self()->_name= self()->routineGroup()->name();
  }

  if (!_figure)
    try_realize();
  else if (_figure)
  {
    if (self()->routineGroup().is_valid())
    {
      _figure->set_title(*self()->_name, bec::fmt("%i routines", self()->_routineGroup->routines().count()));
    }
    else
      unrealize();
  }
}


void workbench_physical_RoutineGroupFigure::ImplData::member_changed(const std::string &name, const grt::ValueRef &ovalue)
{
  /*QQQ
  else if (name == "color" && _figure &&
    get_model()->get_int_option("SynchronizeObjectColors", 0))
  {
    super::set_item(name, value);

    // temporarily disable to avoid recursion
    get_model()->__options.gset("SynchronizeObjectColors", 0);

    // change color of all RoutineGroups in all views that point to the same db RoutineGroup
    model_Model model(__owner.owner());
    for (size_t vc= model.views().count(), vi= 0; vi < vc; vi++)
    {
      grt::ListRef<model_Figure> figures(model.views()[vi]->figures());
      for (size_t c= figures.count(), i= 0; i < c; i++)
      {
        model_Figure figure(figures[i]);

        if (figure != get_self() &&
          figure.is_instance(workbench_physical_workbench_physical_RoutineGroupFigure::ImplData::static_class_name())
          && workbench_physical_workbench_physical_RoutineGroupFigure::ImplData::cast_from(figure).routineGroup() == __routineGroup
          && strcmp(figure.color().c_str(), __color.c_str()) != 0)
        {
          figure.color(__color);
        }
      }
    }
    get_model()->__options.gset("SynchronizeObjectColors", 1);
  }
   * */
}


bool workbench_physical_RoutineGroupFigure::ImplData::is_realizable()
{
  if (!super::is_realizable())
    return false;

  if (self()->_routineGroup.is_valid())
    return true;

  return false;
}


void workbench_physical_RoutineGroupFigure::ImplData::unrealize()
{
  super::unrealize();
  
  delete _figure;
  _figure= 0;
}


bool workbench_physical_RoutineGroupFigure::ImplData::realize()
{
  if (_figure) return true;
  if (!is_realizable())
    return false;

  if (!is_main_thread())
  {
    run_later(sigc::hide_return(sigc::mem_fun(this, &ImplData::realize)));
    return true;
  }

  if (!_figure)
  {
    mdc::CanvasView *view= self()->owner()->get_data()->get_canvas_view();
    mdc::AreaGroup *agroup;

    view->lock();

    _figure= new wbfig::RoutineGroup(view->get_current_layer());

    agroup= self()->layer()->get_data()->get_area_group();
    
    view->get_current_layer()->add_item(_figure, agroup);
    
    _figure->set_color(mdc::Color::parse(*self()->_color));

    _figure->set_title(*self()->_name, bec::fmt("%i routines", self()->_routineGroup->routines().count()));

    _figure->get_title()->signal_activate().
      connect(sigc::mem_fun(this, &super::activate));

    sync_routines();

    finish_realize();

    view->unlock();

    self()->owner()->get_data()->notify_object_realize(self());
  }
  return true;
}


void workbench_physical_RoutineGroupFigure::ImplData::sync_routines()
{
  if (_figure)
  {
    wbfig::BaseFigure::ItemList::iterator iter= _figure->begin_routines_sync();

    grt::ListRef<db_Routine> routines(self()->_routineGroup->routines());
    size_t maxnlength= self()->owner()->owner()->get_data()->get_int_option("workbench.physical.RoutineGroupFigure:MaxRoutineNameLength", 20);

    for (size_t c= routines.count(), i= 0; i < c; i++)
    {
      db_RoutineRef routine(routines.get(i));
      std::string text;

      text= *routine->name();
      if (text.length() > maxnlength)
        text= text.substr(0, maxnlength)+"...";

      iter= _figure->sync_next_routine(iter,
                                      routine.id(),
                                      text);
    }
    _figure->end_routines_sync(iter);

    _figure->set_title(*self()->_routineGroup->name(), bec::fmt("%i routines", self()->_routineGroup->routines().count()));
  }
}
