#include "stdafx.h"

#include "workbench_physical_tablefigure_impl.h"
#include "workbench_physical_diagram_impl.h"
#include "workbench_physical_model_impl.h"
#include "model_layer_impl.h"

#include "table_figure.h"
#include "table_figure_wb.h"

#include "grt/common.h"
#include "grt/grt_manager.h"
#include "grtdb/db_object_helpers.h"



workbench_physical_TableFigure::ImplData::ImplData(workbench_physical_TableFigure *self)
  : super(self), _figure(0)
{
  _pending_columns_sync= false;
  _pending_index_sync= false;

  self->signal_changed().connect(sigc::mem_fun(this, &ImplData::member_changed));
}


void workbench_physical_TableFigure::ImplData::update_options(const std::string &key)
{
  if (bec::has_prefix(key, "workbench.physical.ObjectFigure:"))
  {
    if (_figure)
      sync_columns();
  }
}


void workbench_physical_TableFigure::ImplData::set_in_view(bool flag)
{
  if (!self()->owner().is_valid())
    throw std::logic_error("adding figure to view before setting owner");

  if (flag)
  {
    if (self()->_table.is_valid())
      workbench_physical_DiagramRef::cast_from(self()->_owner)->get_data()->add_mapping(self()->_table, self());
  }
  else
  {
    if (self()->_table.is_valid())
      workbench_physical_DiagramRef::cast_from(self()->_owner)->get_data()->remove_mapping(self()->_table);
  }
  
  model_Figure::ImplData::set_in_view(flag);
}


void workbench_physical_TableFigure::ImplData::set_table(const db_TableRef &table)
{
  // Check if we had a valid table before and revert the previous setup if so.
  if (self()->_table.is_valid())
  {
    if (self()->_owner.is_valid())
      workbench_physical_DiagramRef::cast_from(self()->_owner)->get_data()->remove_mapping(self()->_table);
    
    // TODO: For disconnection of the signals we'd need the iterators returned by "connect". Do we need to disconnect at all?
    // self()->_table->signal_refreshDisplay().disconnect();
    // self()->_table->signal_changed().disconnect();
  }

  // Now replace the table and run its setup.
  self()->_table= table;
  
  if (self()->_table.is_valid())
  {
    if (self()->_owner.is_valid())
      workbench_physical_DiagramRef::cast_from(self()->_owner)->get_data()->add_mapping(table, self());
    
    table->signal_refreshDisplay().
      connect(sigc::mem_fun(this, &workbench_physical_TableFigure::ImplData::content_changed));
    
    table->signal_changed().
      connect(sigc::mem_fun(this, &workbench_physical_TableFigure::ImplData::table_member_changed));

    self()->_name= self()->_table->name();

    if (_figure)
    {
      // Refresh everything because table has been replaced.
      _figure->get_title()->set_title(*self()->_table->name());
      run_later(sigc::mem_fun(this, &workbench_physical_TableFigure::ImplData::sync_columns));
      run_later(sigc::mem_fun(this, &workbench_physical_TableFigure::ImplData::sync_indexes));
    }
    else
      try_realize();
  }
  else
    unrealize();
}


void workbench_physical_TableFigure::ImplData::table_member_changed(const std::string &name, const grt::ValueRef &ovalue)
{
  if (name == "name")
  {
    self()->_name= self()->_table->name();

    if (_figure)
      _figure->get_title()->set_title(*self()->_table->name());
  }
  else if (name == "primaryKey")
  {
    if (_figure)
    {
      if (!_pending_columns_sync)
      {
        _pending_columns_sync= true;
        run_later(sigc::mem_fun(this, &workbench_physical_TableFigure::ImplData::sync_columns));
      }
    }
  }
}


void workbench_physical_TableFigure::ImplData::content_changed(const std::string &where)
{
  if ((where == "column" || where == "foreignKey") &&
      _figure && !_pending_columns_sync)
  {
    _pending_columns_sync= true;
    run_later(sigc::mem_fun(this, &workbench_physical_TableFigure::ImplData::sync_columns));
  }

  if (where == "index" && _figure && !_pending_index_sync)
  {
    _pending_index_sync= true;
    run_later(sigc::mem_fun(this, &workbench_physical_TableFigure::ImplData::sync_indexes));
  }
}


void workbench_physical_TableFigure::ImplData::member_changed(const std::string &name, const grt::ValueRef &ovalue)
{
/* not good  if (name == "name")
  {
    if (self()->_table.is_valid())
      self()->_table->name(self()->_name);
  }
  */
  /*
  else if (name == "columnsExpanded")
  {
    return; // ignore
  }*/
  /*else*/ if (name == "indicesExpanded")
  {
    if (_figure)
      _figure->toggle_indexes(*self()->_indicesExpanded!=0);
  }
  else if (name == "triggersExpanded")
  {
    if (_figure)
      _figure->toggle_triggers(*self()->_triggersExpanded!=0);
  }
  /*QQQ
  else if (name == "color" && _figure &&
    get_model()->get_int_option("SynchronizeObjectColors", 0))
  {

    // temporarily disable to avoid recursion
    get_model()->self()->_options.gset("SynchronizeObjectColors", 0);

    // change color of all tables in all views that point to the same db table
    model_Model model(self()->_owner.owner());
    for (size_t vc= model.views().count(), vi= 0; vi < vc; vi++)
    {
      grt::ListRef<model_Figure> figures(model.views()[vi]->figures());
      for (size_t c= figures.count(), i= 0; i < c; i++)
      {
        model_Figure figure(figures[i]);

        if (figure != get_self() &&
          figure.is_instance(wo::static_class_name())
          && ImplData::cast_from(figure).table() == self()->_table
          && strcmp(figure.color().c_str(), self()->_color.c_str()) != 0)
        {
          figure.color(self()->_color);
        }
      }
    }
    get_model()->self()->_options.gset("SynchronizeObjectColors", 1);
  }*/
}


void workbench_physical_TableFigure::ImplData::sync_columns()
{
  if (_figure)
  {
    wbfig::Table::ItemList::iterator iter= _figure->begin_columns_sync();

    grt::ListRef<db_Column> columns(self()->_table->columns());
    bool show_type= self()->owner()->owner()->get_data()->get_int_option("workbench.physical.TableFigure:ShowColumnTypes", 1)!=0;
    bool show_flags= self()->owner()->owner()->get_data()->get_int_option("workbench.physical.TableFigure:ShowColumnFlags", 0)!=0;
    int max_count= self()->owner()->owner()->get_data()->get_int_option("workbench.physical.TableFigure:MaxColumnsDisplayed", 20);
    bool keys_only= false;

    if (max_count > 0 && max_count < (int)columns.count())
      keys_only= true;

    for (size_t c= columns.count(), i= 0; i < c; i++)
    {
      db_ColumnRef column= columns.get(i);
      std::string text;
      wbfig::ColumnFlags flags= (wbfig::ColumnFlags)0;
      
      if (self()->_table->isPrimaryKeyColumn(column))
        flags= (wbfig::ColumnFlags)(flags|wbfig::ColumnPK);
      if (self()->_table->isForeignKeyColumn(column))
        flags= (wbfig::ColumnFlags)(flags|wbfig::ColumnFK);

      if (keys_only && flags == (wbfig::ColumnFlags)0)
        continue;

      if (column->isNotNull())
        flags= (wbfig::ColumnFlags)(flags|wbfig::ColumnNotNull);

      if (column->flags().get_index("UNSIGNED") != grt::BaseListRef::npos)
        flags= (wbfig::ColumnFlags)(flags|wbfig::ColumnUnsigned);
      if (column.has_member("autoIncrement") && column.get_integer_member("autoIncrement") != 0)
        flags= (wbfig::ColumnFlags)(flags|wbfig::ColumnAutoIncrement);

      text= *column->name();
      if (show_type)
        text.append(" ").append(bec::ColumnHelper::format_column_type(column));

      iter= _figure->sync_next_column(iter,
                                      column.id(),
                                      flags,
                                      text);
    }
    
    // add an item to indicate there are more columns in the table
    if (keys_only)
      iter= _figure->sync_next_column(iter, "", wbfig::ColumnListTruncated, "...");

    _figure->set_show_flags(show_flags);
    _figure->end_columns_sync(iter);

    _figure->set_dependant(self()->_table->isDependantTable() != 0);

    _pending_columns_sync= false;
  }
}


void workbench_physical_TableFigure::ImplData::sync_indexes()
{
  if (_figure)
  {
    wbfig::Table::ItemList::iterator iter= _figure->begin_indexes_sync();

    grt::ListRef<db_Index> indexes(self()->_table->indices());

    for (size_t c= indexes.count(), i= 0; i < c; i++)
    {
      db_IndexRef index(indexes.get(i));
      std::string text;
      
      text= *index->name();
      
      iter= _figure->sync_next_index(iter,
                                     index.id(),
                                     text);
    }
    _figure->end_indexes_sync(iter);

    if (_figure->get_index_title() && !_figure->indexes_hidden())
      _figure->get_index_title()->set_visible(indexes.count() > 0);

    _pending_index_sync= false;
  }
}


bool workbench_physical_TableFigure::ImplData::is_realizable()
{
  if (!super::is_realizable())
    return false;

  if (self()->_table.is_valid())
    return true;

  return false;
}


void workbench_physical_TableFigure::ImplData::unrealize()
{
  super::unrealize();

  delete _figure;
  _figure= 0;
}


bool workbench_physical_TableFigure::ImplData::realize()
{
  if (_figure) return true;
  if (!is_realizable()) return false;
  
  if (!is_main_thread())
  {
    run_later(sigc::hide_return(sigc::mem_fun(this, &ImplData::realize)));
    return true;
  }
  
  if (!_figure)
  {
    mdc::CanvasView *view= get_canvas_view();
    mdc::AreaGroup *agroup;

    view->lock();
    
    _figure= new wbfig::WBTable(view->get_current_layer());

    dynamic_cast<workbench_physical_Diagram::ImplData*>(self()->owner()->get_data())->notify_table_realized(self());
    
    agroup= self()->_layer->get_data()->get_area_group();

    view->get_current_layer()->add_item(_figure, agroup);

    _figure->set_color(mdc::Color::parse(*self()->_color));
    
    _figure->get_title()->set_title(*self()->_table->name());

    _figure->get_title()->signal_activate().
      connect(sigc::mem_fun(this, &super::activate));

    _figure->get_title()->signal_expand_toggle().connect(
      sigc::bind<wbfig::Titlebar*>(sigc::mem_fun(this, &workbench_physical_TableFigure::ImplData::toggle_title), _figure->get_title()));
    if (_figure->get_index_title())
    {
       _figure->get_index_title()->signal_activate().
        connect(sigc::mem_fun(this, &super::activate));

      _figure->get_index_title()->signal_expand_toggle().connect(
        sigc::bind<wbfig::Titlebar*>(sigc::mem_fun(this, &workbench_physical_TableFigure::ImplData::toggle_title), _figure->get_index_title()));
    }
    if (_figure->get_trigger_title())
    {
       _figure->get_trigger_title()->signal_activate().
        connect(sigc::mem_fun(this, &super::activate));

      _figure->get_trigger_title()->signal_expand_toggle().connect(
        sigc::bind<wbfig::Titlebar*>(sigc::mem_fun(this, &workbench_physical_TableFigure::ImplData::toggle_title), _figure->get_trigger_title()));
    }

    _figure->set_dependant(self()->_table->isDependantTable() != 0);

    std::string sfont= self()->owner()->owner()->get_data()->get_string_option(bec::fmt("%s:SectionFont", self()->class_name().c_str()), "");
    if (!sfont.empty())
      _figure->set_section_font(parse_font_spec(sfont));

    _figure->toggle(*self()->_expanded!=0);
    _figure->toggle_indexes(*self()->_indicesExpanded!=0);

    _figure->signal_index_crossed().connect(sigc::mem_fun(this, &ImplData::index_crossed));
    
    if (self()->_table->columns().count() > 0)
      sync_columns();
    if (self()->_table->indices().count() > 0)
      sync_indexes();

    finish_realize();
    
    view->unlock();

    notify_realized();
  }
  return true;
}


void workbench_physical_TableFigure::ImplData::set_highlighted(bool flag)
{
  if (_figure)
    _figure->set_highlighted(flag);
}


void workbench_physical_TableFigure::ImplData::toggle_title(bool expanded, wbfig::Titlebar *sender)
{
  if (sender == _figure->get_title())
  {
    grt::AutoUndo undo(self()->get_grt());
    self()->expanded(expanded);
    undo.end(expanded ? _("Expand Table") : _("Collapse Table"));
  }
  else if (sender == _figure->get_index_title())
  {
    grt::AutoUndo undo(self()->get_grt());
    self()->indicesExpanded(expanded);
    undo.end(expanded ? _("Expand Table Indices") : _("Collapse Table Indices"));
  }
  else if (sender == _figure->get_trigger_title())
  {
    grt::AutoUndo undo(self()->get_grt());
    self()->triggersExpanded(expanded);
    undo.end(expanded ? _("Expand Table Triggers") : _("Collapse Table Triggers"));
  }
}


db_ColumnRef workbench_physical_TableFigure::ImplData::get_column_at(const mdc::Point &pos)
{
  if (_figure && !_figure->get_columns()->empty())
  {
    size_t i= 0;
    mdc::Point p= pos - _figure->get_columns()->front()->get_parent()->convert_point_to(mdc::Point(), _figure);

    for (wbfig::Table::ItemList::const_iterator iter= _figure->get_columns()->begin();
      iter != _figure->get_columns()->end();
      ++iter, ++i)
    {
      if ((*iter)->contains_point(p))
        return grt::find_object_in_list(self()->_table->columns(), (*iter)->get_id());
    }
  }
  return db_ColumnRef();
}


db_IndexRef workbench_physical_TableFigure::ImplData::get_index_at(const mdc::Point &pos)
{
  if (_figure && _figure->get_indexes() && !_figure->get_indexes()->empty())
  {
    size_t i= 0;
    mdc::Point p= pos - _figure->get_indexes()->front()->get_parent()->convert_point_to(mdc::Point(), _figure);

    for (wbfig::Table::ItemList::const_iterator iter= _figure->get_indexes()->begin();
      iter != _figure->get_indexes()->end();
      ++iter, ++i)
    {
      if ((*iter)->contains_point(p))
        return grt::find_object_in_list(self()->_table->indices(), (*iter)->get_id());
    }
  }
  return db_IndexRef();
}


void workbench_physical_TableFigure::ImplData::set_column_highlighted(const db_ColumnRef &column, bool flag)
{
  if (_figure)
  {
    for (wbfig::Table::ItemList::const_iterator iter= _figure->get_columns()->begin();
        iter != _figure->get_columns()->end();
        ++iter)
    {
      if (!column.is_valid() || (*iter)->get_id() == column.id())
      {
        (*iter)->set_highlighted(flag);
        break;
      }
    }
  }
}



void workbench_physical_TableFigure::ImplData::index_crossed(int index_i, bool entered)
{
  // highlight columns that are in the index + the index itself
  if (_figure && index_i < (int)self()->_table->indices().count())
  {
    wbfig::Table::ItemList *indexes= _figure->get_indexes();
    wbfig::Table::ItemList *columns= _figure->get_columns();
    if (indexes && columns)
    {
      db_IndexRef index(self()->_table->indices()[index_i]);
      wbfig::Table::ItemList::iterator iter;
      iter= indexes->begin();
      int i= index_i;
      while (iter != indexes->end() && --i >= 0) ++iter;
      if (iter != indexes->end())
      {
        // highlight the index
        (*iter)->set_highlighted(entered);
      }

      // highlight columns in the index
      for (wbfig::Table::ItemList::const_iterator iter= columns->begin();
           iter != columns->end(); ++iter)
      {
        std::string column_id= (*iter)->get_id();
        bool ok= false;

        // lookup the index for matching columns
        for (grt::ListRef<db_IndexColumn>::const_iterator ic= index->columns().begin();
             ic != index->columns().end(); ++ic)
        {
          if ((*ic)->referencedColumn().is_valid() && (*ic)->referencedColumn().id() == column_id)
          {
            ok= true;
            break;
          }
        }
        if (ok)
          (*iter)->set_highlighted(entered);
      }
    }
  }
}

