#ifndef _WORKBENCH_PHYSICAL_TABLEFIGURE_IMPL_H_
#define _WORKBENCH_PHYSICAL_TABLEFIGURE_IMPL_H_

#include "wb_config.h"

#include "model_figure_impl.h"
#include "grts/structs.workbench.physical.h"

#include "table_figure.h"


/** Table figure implementation.
 * Implements inner working of db table figures.
 * 
 * Figure contents are updated when the following changes:
 * 
 * <li>figure attributes (color, state etc)
 * <li>table name
 * <li>column list (column added, removed)
 * <li>column name changed
 * <li>column type changed
 * <li>primary key
 * <li>foreign key list (fk added, removed)
 * <li>foreign key columns list (column added, removed)
 * <li>index list (index added, removed)
 * <li>index name changed
 * 
 * @ingroup Figures
 */
class MYSQLWBCANVASBRIDGE_PUBLIC workbench_physical_TableFigure::ImplData : public model_Figure::ImplData
{
  typedef model_Figure::ImplData super;
  
protected:
  wbfig::Table *_figure;

  bool _pending_columns_sync;
  bool _pending_index_sync;
  
  std::vector<int> _list_listeners;
  
  virtual bool realize();
  
  virtual void update_options(const std::string &key);

  void sync_columns();
  void sync_indexes();

  void toggle_title(bool expanded, wbfig::Titlebar *sender);

  void member_changed(const std::string &name, const grt::ValueRef &ovalue);
  void table_member_changed(const std::string &name, const grt::ValueRef &ovalue);

  void content_changed(const std::string &where);
  
  void fk_list_changed(bool added, const db_ForeignKeyRef &fk);
  void index_columns_list_changed(bool added, const db_IndexColumnRef &column);

  virtual void set_in_view(bool flag);
  
  void index_crossed(int index, bool entered);
  
public:
  ImplData(workbench_physical_TableFigure *self);
  
  void set_table(const db_TableRef &table);
  
  db_ColumnRef get_column_at(const mdc::Point &pos);
  db_IndexRef get_index_at(const mdc::Point &pos);
  
  void set_column_highlighted(const db_ColumnRef &column, bool flag);
  
  virtual wbfig::BaseFigure* get_canvas_item() const { return _figure; }
  virtual bool is_realizable();

  void set_highlighted(bool flag);

  virtual void unrealize();

private:
  workbench_physical_TableFigure *self() const { return (workbench_physical_TableFigure*)_self; }
};


#endif
