
#include "stdafx.h"

#include "workbench_physical_viewfigure_impl.h"

#include "model_layer_impl.h"
#include "workbench_physical_diagram_impl.h"


workbench_physical_ViewFigure::ImplData::ImplData(workbench_physical_ViewFigure *owner)
  : super(owner), _figure(0)
{
  _resizable= false;
}


void workbench_physical_ViewFigure::ImplData::view_member_changed(const std::string &name, 
                                                                  const grt::ValueRef &ovalue)
{
  if (name == "name")
  {
    self()->_name= self()->view()->name();

    if (_figure)
      _figure->set_title(*self()->_name);
  }
}


void workbench_physical_ViewFigure::ImplData::set_in_view(bool flag)
{
  if (!self()->owner().is_valid())
    throw std::logic_error("adding figure to diagram before setting owner");

  if (flag)
  {
    if (self()->_view.is_valid())
      workbench_physical_DiagramRef::cast_from(self()->_owner)->get_data()->add_mapping(self()->_view, self());
  }
  else
  {
    if (self()->_view.is_valid())
      workbench_physical_DiagramRef::cast_from(self()->_owner)->get_data()->remove_mapping(self()->_view);
  }
  
  model_Figure::ImplData::set_in_view(flag);
}


void workbench_physical_ViewFigure::ImplData::set_view(const db_ViewRef &view)
{
  self()->_view= view;
  
  if (_figure_conn)
    _figure_conn.disconnect();
  
  if (self()->view().is_valid())
  {
    if (self()->_owner.is_valid())
      workbench_physical_DiagramRef::cast_from(self()->_owner)->get_data()->add_mapping(view, self());

    _figure_conn= self()->view()->signal_changed().connect(sigc::mem_fun(this, &ImplData::view_member_changed));
    self()->_name= self()->view()->name();
  }

  if (!_figure)
    try_realize();
  else if (_figure)
  {
    if (self()->view().is_valid())
      _figure->set_title(*self()->view()->name());
    else
      unrealize();
  }
}


void workbench_physical_ViewFigure::ImplData::member_changed(const std::string &name, const grt::ValueRef &ovalue)
{
    /*  dont rename view from figure
  else if (name == "name")
  {
    if (grt::StringRef::cast_from(value) == self()->_view->name())
    {
      //if (__view.is_valid())
      //  __view.name(__name);
    }
  }*/
  if (name == "color" && _figure &&
           self()->owner()->owner()->get_data()->get_int_option("SynchronizeObjectColors", 0))
  {
    /*QQQ implement this at WB level
    // temporarily disable to avoid recursion
    get_model()->__options.gset("SynchronizeObjectColors", 0);

    // change color of all tables in all views that point to the same db table
    model_Model model(__owner.owner());
    for (size_t vc= model.views().count(), vi= 0; vi < vc; vi++)
    {
      grt::ListRef<model_Figure> figures(model.views()[vi]->figures());
      for (size_t c= figures.count(), i= 0; i < c; i++)
      {
        model_Figure figure(figures[i]);

        if (figure != get_self() &&
          figure.is_instance(workbench_physical_workbench_physical_ViewFigure::ImplData::static_class_name())
          && workbench_physical_workbench_physical_ViewFigure::ImplData::cast_from(figure).view() == __view
          && strcmp(figure.color().c_str(), __color.c_str()) != 0)
        {
          figure.color(__color);
        }
      }
    }
    get_model()->__options.gset("SynchronizeObjectColors", 1);
     */
  }
}


bool workbench_physical_ViewFigure::ImplData::is_realizable()
{
  if (!super::is_realizable())
    return false;

  if (self()->_view.is_valid())
    return true;

  return false;
}


void workbench_physical_ViewFigure::ImplData::unrealize()
{
  super::unrealize();
  
  delete _figure;
  _figure= 0;
}



bool workbench_physical_ViewFigure::ImplData::realize()
{
  if (_figure) return true;
  if (!is_realizable()) return false;
  
  if (!is_main_thread())
  {
    run_later(sigc::hide_return(sigc::mem_fun(this, &ImplData::realize)));
    return true;
  }
  
  if (!_figure)
  {
    mdc::CanvasView *view= self()->owner()->get_data()->get_canvas_view();
    mdc::AreaGroup *agroup;

    if (!self()->_view.is_valid())
      throw std::logic_error("Realizing table figure without table object");

    view->lock();
    
    _figure= new wbfig::View(view->get_current_layer());

    agroup= self()->layer()->get_data()->get_area_group();

    view->get_current_layer()->add_item(_figure, agroup);
    
    _figure->set_color(mdc::Color::parse(*self()->_color));

    _figure->set_title(*self()->_view->name());

    _figure->get_title()->signal_activate().
      connect(sigc::mem_fun(this, &super::activate));
    
    finish_realize();

    view->unlock();

    self()->owner()->get_data()->notify_object_realize(self());
  }
  return true;
}

