#include "document_properties_form.h"
#include "workbench/wb_context_ui.h"

/**
 * @file  document_properties_form.cpp
 * @brief 
 */


DocumentPropertiesForm::DocumentPropertiesForm(wb::WBContextUI *wbui)
  : _top_box(false), _bottom_box(true),
    _btn_size_group(true, false)
{
  _wbui= wbui;
  
  set_title(_("Document Properties"));
  
  set_content(&_top_box);
  _top_box.set_padding(12);
  _top_box.set_spacing(8);

  add_control(_("Name:"), &_entry1);
  add_control(_("Version:"), &_entry2);
  add_control(_("Author:"), &_entry3);
  add_control(_("Project:"), &_entry4);
  add_control(_("Created:"), &_entry5);
  add_control(_("Last Changed:"), &_entry6);
  add_control(_("Description:"), &_text);


  _entry5.set_enabled(false);
  _entry6.set_enabled(false);
  
  _top_box.add(&_bottom_box, false, false);
  _bottom_box.set_spacing(8);

  _ok_button.signal_clicked().connect(sigc::mem_fun(this, &DocumentPropertiesForm::ok_clicked));
  _cancel_button.signal_clicked().connect(sigc::mem_fun(this, &DocumentPropertiesForm::cancel_clicked));
  
  _btn_size_group.add(&_ok_button);
  _btn_size_group.add(&_cancel_button);

  _bottom_box.add_end(&_cancel_button, false, false);
  _cancel_button.set_text(_("Cancel"));

  _bottom_box.add_end(&_ok_button, false, false);
  _ok_button.set_text(_(" OK "));

//  _btn_size_group.resize();

  pull_values();
}


void DocumentPropertiesForm::push_values()
{

}


void DocumentPropertiesForm::pull_values()
{
  std::string caption, version, author, project, dateCreated, dateChanged, description;

  //fetch values from backend
  _wbui->get_doc_properties(caption, version, author, project, dateCreated, dateChanged, description);

  _entry1.set_value(caption);
  _entry2.set_value(version);
  _entry3.set_value(author);
  _entry4.set_value(project);
  _entry5.set_value(dateChanged);
  _entry6.set_value(dateCreated);
  
  _text.set_value(description);
}


static void destroy(mforms::Object *object)
{
  delete object;
}


DocumentPropertiesForm::~DocumentPropertiesForm()
{
  std::for_each(_widgets.begin(), _widgets.end(), sigc::ptr_fun(destroy));
}



void DocumentPropertiesForm::add_control(const std::string &caption, mforms::View *control)
{
  mforms::Box *box;
  mforms::Label *label;

  box= new mforms::Box(true);
  label= new mforms::Label();

  _top_box.add(box, false, false);
  box->set_spacing(4);

  label->set_text(caption);
  label->set_text_align(mforms::TopRight);

  box->add(label, false, true);
  box->add(control, true, true);

  _widgets.push_back(box);
  _widgets.push_back(label);
}


void DocumentPropertiesForm::ok_clicked()
{
  push_values();

  show(false);
  _handle_close();
}


void DocumentPropertiesForm::cancel_clicked()
{
  show(false);
  _handle_close();
}


void DocumentPropertiesForm::set_closed_handler(const sigc::slot<void> &slot)
{
  _handle_close= slot;
}
