/* Copyright (C) 2004 MySQL AB

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */


#include "MGGladeXML.h"
#include "wbdbg.h"

#include <gtk/gtk.h>

static GtkWidget *custom_handler(GladeXML *xml,
                                 gchar *func_name,
                                 gchar *name,
                                 gchar *string1,
                                 gchar *string2,
                                 gint int1,
                                 gint int2,
                                 gpointer user_data);

static void add_custom_handlers();

//------------------------------------------------------------------------------
MGGladeXML::MGGladeXML(const std::string &file, const Glib::ustring &root, 
                        const std::string &translation_domain)
  : _xml(0)
{
  static bool initialized= false;
  
  if (!initialized)
  {
    initialized= true;
    glade_init();
    
    glade_set_custom_handler(custom_handler, 0);
    
    add_custom_handlers();
  }

  if (translation_domain.empty())
    _xml= glade_xml_new(file.c_str(), root.empty()?NULL:root.c_str(), NULL);
  else
    _xml= glade_xml_new(file.c_str(), root.empty()?NULL:root.c_str(), translation_domain.c_str());

  if (!_xml)
  {
    throw Error("could not open glade file "+Glib::filename_to_utf8(file));
  }
}

//------------------------------------------------------------------------------
MGGladeXML::MGGladeXML(const char* buffer, const int  buffer_size, const char *root, const char* domain)
           : _xml(0)
{
  static bool initialized= false;
  
  if (!initialized)
  {
    initialized= true;
    glade_init();
  }

  _xml= glade_xml_new_from_buffer(buffer, buffer_size, root, domain);
  
  if (!_xml)
    throw Error("could not load glade from buffer");
}

//------------------------------------------------------------------------------
MGGladeXML::~MGGladeXML()
{
  g_object_unref(_xml);
  _xml = 0;
}


//Gtk::Widget *MGGladeXML::get_widget(const Glib::ustring &name) const
//------------------------------------------------------------------------------
Gtk::Widget* MGGladeXML::get(const Glib::ustring& name, Gtk::Widget** obj) const
{
  GtkWidget *w= glade_xml_get_widget(_xml, name.c_str());
  if (!w)
  {
    g_warning("request for bad widget '%s' in glade object", name.c_str());
    return 0;
  }
  
  return (*obj = Glib::wrap(w));
}


//Gtk::Window *MGGladeXML::get_window(const Glib::ustring &name) const
//------------------------------------------------------------------------------
Gtk::Window* MGGladeXML::get(const Glib::ustring& name, Gtk::Window** obj) const
{
  *obj = static_cast<Gtk::Window*>(get_widget(name));
  if (!GTK_IS_WINDOW((*obj)->gobj()))
  {
    g_warning("request for bad widget '%s' in glade object", name.c_str());
    return 0;
  }
  return (*obj);
}


//Gtk::Entry *MGGladeXML::get_entry(const Glib::ustring &name) const
//------------------------------------------------------------------------------
Gtk::Entry* MGGladeXML::get(const Glib::ustring& name, Gtk::Entry** obj) const
{
  *obj = static_cast<Gtk::Entry*>(get_widget(name));
  
  if (!GTK_IS_ENTRY((*obj)->gobj()))
  {
    g_warning("requested widget '%s' with the wrong type", name.c_str());
    return 0;
  }
  
  return (*obj);
}


//Gtk::ToggleButton *MGGladeXML::get_toggle(const Glib::ustring &name) const
//------------------------------------------------------------------------------
Gtk::ToggleButton* MGGladeXML::get(const Glib::ustring& name, Gtk::ToggleButton** obj) const
{
  *obj = static_cast<Gtk::ToggleButton*>(get_widget(name));
  
  if (!GTK_IS_TOGGLE_BUTTON((*obj)->gobj()))
  {
    g_warning("requested widget '%s' with the wrong type", name.c_str());
    return 0;
  }
  
  return (*obj);
}


//Gtk::TreeView *MGGladeXML::get_tree(const Glib::ustring &name) const
//------------------------------------------------------------------------------
Gtk::TreeView* MGGladeXML::get(const Glib::ustring& name, Gtk::TreeView** obj) const
{
  *obj = static_cast<Gtk::TreeView*>(get_widget(name));
  
  if (!GTK_IS_TREE_VIEW((*obj)->gobj()))
  {
    g_warning("requested widget '%s' with the wrong type", name.c_str());
    return 0;
  }
  
  return (*obj);
}


//Gtk::Button *MGGladeXML::get_button(const Glib::ustring &name) const
//------------------------------------------------------------------------------
Gtk::Button* MGGladeXML::get(const Glib::ustring& name, Gtk::Button** obj) const
{
  *obj = static_cast<Gtk::Button*>(get_widget(name));
  
  if (!GTK_IS_BUTTON((*obj)->gobj()))
  {
    g_warning("requested widget '%s' with the wrong type", name.c_str());
    return 0;
  }
  
  return (*obj);
}


//Gtk::Label *MGGladeXML::get_label(const Glib::ustring &name) const
//------------------------------------------------------------------------------
Gtk::Label* MGGladeXML::get(const Glib::ustring& name, Gtk::Label** obj) const
{
  *obj = static_cast<Gtk::Label*>(get_widget(name));
  
  if (!GTK_IS_LABEL((*obj)->gobj()))
  {
    g_warning("requested widget '%s' with the wrong type", name.c_str());
    return 0;
  }
  
  return (*obj);
}


//Gtk::SpinButton *MGGladeXML::get_spin(const Glib::ustring &name) const
//------------------------------------------------------------------------------
Gtk::SpinButton* MGGladeXML::get(const Glib::ustring& name, Gtk::SpinButton** obj) const
{
  *obj = static_cast<Gtk::SpinButton*>(get_widget(name));
  
  if (!GTK_IS_SPIN_BUTTON((*obj)->gobj()))
  {
    g_warning("requested widget '%s' with the wrong type", name.c_str());
    return 0;
  }
  
  return (*obj);
}

//Gtk::TextView *MGGladeXML::get_text(const Glib::ustring &name) const
//------------------------------------------------------------------------------
Gtk::TextView* MGGladeXML::get(const Glib::ustring& name, Gtk::TextView** obj) const
{
  *obj = static_cast<Gtk::TextView*>(get_widget(name));
  
  if (!GTK_IS_TEXT_VIEW((*obj)->gobj()))
  {
    g_warning("requested widget '%s' with the wrong type", name.c_str());
    return 0;
  }
  
  return (*obj);
}



//Gtk::OptionMenu *MGGladeXML::get_option(const Glib::ustring &name) const
//------------------------------------------------------------------------------
Gtk::OptionMenu* MGGladeXML::get(const Glib::ustring& name, Gtk::OptionMenu** obj) const
{
  *obj = static_cast<Gtk::OptionMenu*>(get_widget(name));

  if (!GTK_IS_OPTION_MENU((*obj)->gobj()))
  {
    g_warning("requested widget '%s' with the wrong type", name.c_str());
    return 0;
  }
  return (*obj);
}


//Gtk::ComboBox *MGGladeXML::get_combo(const Glib::ustring &name) const
//------------------------------------------------------------------------------
Gtk::ComboBox* MGGladeXML::get(const Glib::ustring& name, Gtk::ComboBox** obj) const
{
  *obj = static_cast<Gtk::ComboBox*>(get_widget(name));
  
  if (!GTK_IS_COMBO_BOX((*obj)->gobj()))
  {
    g_warning("requested widget '%s' with the wrong type", name.c_str());
    return 0;
  }  
  return (*obj);
}

Gtk::ComboBoxText* MGGladeXML::get(const Glib::ustring& name, Gtk::ComboBoxText** obj) const
{
  *obj = static_cast<Gtk::ComboBoxText*>(get_widget(name));
  
  if (!GTK_IS_COMBO_BOX((*obj)->gobj()))
  {
    g_warning("requested widget '%s' with the wrong type", name.c_str());
    return 0;
  }  
  return (*obj);
}

//Gtk::ComboBoxEntry *MGGladeXML::get_combo_entry(const Glib::ustring &name) const
//------------------------------------------------------------------------------
Gtk::ComboBoxEntry* MGGladeXML::get(const Glib::ustring& name, Gtk::ComboBoxEntry** obj) const
{
  *obj = static_cast<Gtk::ComboBoxEntry*>(get_widget(name));
  
  if (!GTK_IS_COMBO_BOX_ENTRY((*obj)->gobj()))
  {
    g_warning("requested widget '%s' with the wrong type", name.c_str());
    return 0;
  }  
  return (*obj);
}

//------------------------------------------------------------------------------
Gtk::ComboBoxEntryText* MGGladeXML::get(const Glib::ustring& name, Gtk::ComboBoxEntryText** obj) const
{
  *obj = static_cast<Gtk::ComboBoxEntryText*>(get_widget(name));
  
  if (!GTK_IS_COMBO_BOX_ENTRY((*obj)->gobj()))
  {
    g_warning("requested widget '%s' with the wrong type", name.c_str());
    return 0;
  }  
  return (*obj);
}

//Gtk::Entry *MGGladeXML::get_combo_entry_entry(const Glib::ustring &name) const
//------------------------------------------------------------------------------
Gtk::Entry* MGGladeXML::get_entry(const Glib::ustring& name, Gtk::ComboBoxEntry** obj) const
{
  *obj = static_cast<Gtk::ComboBoxEntry*>(get_widget(name));
  
  if (!GTK_IS_COMBO_BOX_ENTRY((*obj)->gobj()))
  {
    g_warning("requested widget '%s' with the wrong type", name.c_str());
    return 0;
  }
  return (Gtk::Entry*)(*obj)->get_child();
}

//Gtk::Container *MGGladeXML::get_container(const Glib::ustring &name) const
//------------------------------------------------------------------------------
Gtk::Container* MGGladeXML::get(const Glib::ustring& name, Gtk::Container** obj) const
{
  *obj = static_cast<Gtk::Container*>(get_widget(name));
  
  if (!GTK_IS_CONTAINER((*obj)->gobj()))
  {
    g_warning("requested widget '%s' with the wrong type", name.c_str());
    return 0;
  }
  
  return (*obj);
}

//Gtk::Notebook *MGGladeXML::get_note(const Glib::ustring &name) const
//------------------------------------------------------------------------------
Gtk::Notebook* MGGladeXML::get(const Glib::ustring& name, Gtk::Notebook** obj) const
{
  *obj = static_cast<Gtk::Notebook*>(get_widget(name));
  
  if (!GTK_IS_NOTEBOOK((*obj)->gobj()))
  {
    g_warning("requested widget '%s' with the wrong type", name.c_str());
    return 0;
  }
  
  return (*obj);
}


//Gtk::MenuItem *MGGladeXML::get_menu_item(const Glib::ustring &name) const
//------------------------------------------------------------------------------
Gtk::MenuItem* MGGladeXML::get(const Glib::ustring& name, Gtk::MenuItem** obj) const
{
  *obj = static_cast<Gtk::MenuItem*>(get_widget(name));
  
  if (!GTK_IS_MENU_ITEM((*obj)->gobj()))
  {
    g_warning("requested widget '%s' with the wrong type", name.c_str());
    return 0;
  }
  
  return (*obj);
}


//Gtk::CheckMenuItem *MGGladeXML::get_check_menu_item(const Glib::ustring &name) const
//------------------------------------------------------------------------------
Gtk::CheckMenuItem* MGGladeXML::get(const Glib::ustring& name, Gtk::CheckMenuItem** obj) const
{
  *obj = static_cast<Gtk::CheckMenuItem*>(get_widget(name));
  
  if (!GTK_IS_CHECK_MENU_ITEM((*obj)->gobj()))
  {
    g_warning("requested widget '%s' with the wrong type", name.c_str());
    return 0;
  }
  
  return (*obj);
}


//Gtk::Paned *MGGladeXML::get_paned(const Glib::ustring &name) const
//------------------------------------------------------------------------------
Gtk::Paned* MGGladeXML::get(const Glib::ustring& name, Gtk::Paned** obj) const
{
  *obj = static_cast<Gtk::Paned*>(get_widget(name));

  if (!GTK_IS_PANED((*obj)->gobj()))
  {
    g_warning("requested widget '%s' with the wrong type", name.c_str());
    return 0;
  }

  return (*obj);
}


//Gtk::Box *MGGladeXML::get_box(const Glib::ustring &name) const
//------------------------------------------------------------------------------
Gtk::Box* MGGladeXML::get(const Glib::ustring& name, Gtk::Box** obj) const
{
  *obj = static_cast<Gtk::Box*>(get_widget(name));

  if (!GTK_IS_BOX((*obj)->gobj()))
  {
    g_warning("requested widget '%s' with the wrong type", name.c_str());
    return 0;
  }

  return (*obj);
}

//Gtk::IconView *MGGladeXML::get_icon_view(const Glib::ustring &name) const
//------------------------------------------------------------------------------
Gtk::IconView* MGGladeXML::get(const Glib::ustring& name, Gtk::IconView** obj) const
{
  *obj = static_cast<Gtk::IconView*>(get_widget(name));

  if (!GTK_IS_ICON_VIEW((*obj)->gobj()))
  {
    g_warning("requested widget '%s' with the wrong type", name.c_str());
    return 0;
  }

  return (*obj);
}

//Gtk::Image *MGGladeXML::get_image(const Glib::ustring &name) const
//------------------------------------------------------------------------------
Gtk::Image* MGGladeXML::get(const Glib::ustring& name, Gtk::Image** obj) const
{
  *obj = static_cast<Gtk::Image*>(get_widget(name));

  if (!GTK_IS_IMAGE((*obj)->gobj()))
  {
    g_warning("requested widget '%s' with the wrong type", name.c_str());
    return 0;
  }
  return (*obj);
}

//------------------------------------------------------------------------------
Gtk::EventBox* MGGladeXML::get(const Glib::ustring& name, Gtk::EventBox** obj) const
{
  *obj = static_cast<Gtk::EventBox*>(get_widget(name));

  if (!GTK_IS_EVENT_BOX((*obj)->gobj()))
  {
    g_warning("requested widget '%s' with the wrong type", name.c_str());
    return 0;
  }
  return (*obj);
}

//------------------------------------------------------------------------------
Gtk::VBox* MGGladeXML::get(const Glib::ustring& name, Gtk::VBox** obj) const
{
  *obj = static_cast<Gtk::VBox*>(get_widget(name));

  if (!GTK_IS_VBOX((*obj)->gobj()))
  {
    g_warning("requested widget '%s' with the wrong type", name.c_str());
    return 0;
  }
  return (*obj);
}

//------------------------------------------------------------------------------
Gtk::HBox* MGGladeXML::get(const Glib::ustring& name, Gtk::HBox** obj) const
{
  *obj = static_cast<Gtk::HBox*>(get_widget(name));

  if (!GTK_IS_HBOX((*obj)->gobj()))
  {
    g_warning("requested widget '%s' with the wrong type", name.c_str());
    return 0;
  }
  return (*obj);
}

//------------------------------------------------------------------------------
Gtk::Statusbar* MGGladeXML::get(const Glib::ustring& name, Gtk::Statusbar** obj) const
{
  *obj = static_cast<Gtk::Statusbar*>(get_widget(name));

  if (!GTK_IS_STATUSBAR((*obj)->gobj()))
  {
    g_warning("requested widget '%s' with the wrong type", name.c_str());
    return 0;
  }
  return (*obj);
}

//------------------------------------------------------------------------------
Gtk::MenuBar* MGGladeXML::get(const Glib::ustring& name, Gtk::MenuBar** obj) const
{
  *obj = static_cast<Gtk::MenuBar*>(get_widget(name));

  if (!GTK_IS_MENU_BAR((*obj)->gobj()))
  {
    g_warning("requested widget '%s' with the wrong type", name.c_str());
    return 0;
  }
  return (*obj);
}

//------------------------------------------------------------------------------
Gtk::HandleBox* MGGladeXML::get(const Glib::ustring& name, Gtk::HandleBox** obj) const
{
  *obj = static_cast<Gtk::HandleBox*>(get_widget(name));

  if (!GTK_IS_HANDLE_BOX((*obj)->gobj()))
  {
    g_warning("requested widget '%s' with the wrong type", name.c_str());
    return 0;
  }
  return (*obj);
}


//------------------------------------------------------------------------------
Gtk::Toolbar* MGGladeXML::get(const Glib::ustring& name, Gtk::Toolbar** obj) const
{
  *obj = static_cast<Gtk::Toolbar*>(get_widget(name));

  if (!GTK_IS_TOOLBAR((*obj)->gobj()))
  {
    g_warning("requested widget '%s' with the wrong type", name.c_str());
    return 0;
  }
  return (*obj);
}


//------------------------------------------------------------------------------
Gtk::Frame* MGGladeXML::get(const Glib::ustring& name, Gtk::Frame** obj) const
{
  *obj = static_cast<Gtk::Frame*>(get_widget(name));

  if (!GTK_IS_FRAME((*obj)->gobj()))
  {
    g_warning("requested widget '%s' with the wrong type", name.c_str());
    return 0;
  }
  return (*obj);
}

//------------------------------------------------------------------------------
Gtk::Alignment* MGGladeXML::get(const Glib::ustring& name, Gtk::Alignment** obj) const
{
  *obj = static_cast<Gtk::Alignment*>(get_widget(name));

  if (!GTK_IS_ALIGNMENT((*obj)->gobj()))
  {
    g_warning("requested widget '%s' with the wrong type", name.c_str());
    return 0;
  }
  return (*obj);
}

//------------------------------------------------------------------------------
Gtk::ScrolledWindow* MGGladeXML::get(const Glib::ustring& name, Gtk::ScrolledWindow** obj) const
{
  *obj = static_cast<Gtk::ScrolledWindow*>(get_widget(name));

  if (!GTK_IS_SCROLLED_WINDOW((*obj)->gobj()))
  {
    g_warning("requested widget '%s' with the wrong type", name.c_str());
    return 0;
  }
  return (*obj);
}

void MGGladeXML::bind_accelerators(const Glib::RefPtr<Gtk::AccelGroup> &group,
                                   const AccelList &accels)
{
  for (AccelList::const_iterator iter= accels.begin(); iter != accels.end(); ++iter)
  {
    guint accel;
    Gdk::ModifierType mods;

    Gtk::AccelGroup::parse(iter->second, accel, mods);

    Gtk::MenuItem *mitem(0);
    get(iter->first, &mitem);
    mitem->add_accelerator("activate",
                            group,
                            accel, mods, 
                            Gtk::ACCEL_VISIBLE);
  }
}



static std::map<Glib::ustring, MGGladeXML::CustomWidgetFunc> custom_widgets;


static GtkWidget *create_combo_box_text(gchar *name, gchar *string1, gchar *string2, gint int1, gint int2)
{
  Gtk::ComboBoxText *combo= new Gtk::ComboBoxText();

  return GTK_WIDGET(combo->gobj());
}

static GtkWidget *create_combo_box_entry_text(gchar *name, gchar *string1, gchar *string2, gint int1, gint int2)
{
  Gtk::ComboBoxEntryText *combo= new Gtk::ComboBoxEntryText();

  return GTK_WIDGET(combo->gobj());
}


static void add_custom_handlers()
{
  custom_widgets["GtkComboBoxText"]      = create_combo_box_text;
  custom_widgets["GtkComboBoxEntryText"] = create_combo_box_entry_text;
}


static GtkWidget *custom_handler(GladeXML *xml,
                                 gchar *func_name,
                                 gchar *name,
                                 gchar *string1,
                                 gchar *string2,
                                 gint int1,
                                 gint int2,
                                 gpointer user_data)
{
  if (custom_widgets.find(func_name) == custom_widgets.end())
    throw std::runtime_error(std::string("Unknown custom handler name in glade xml ").append(func_name));
  
  return (custom_widgets[func_name])(name, string1, string2, int1, int2);
}

