/* Copyright (C) 2004 MySQL AB

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */


#ifndef _MGGLADEXML_H_
#define _MGGLADEXML_H_

#include <glade/glade.h>
#include <gtkmm.h>


class MGGladeXML : public Glib::ObjectBase {
  public:    
    class Error : public Glib::Exception {
        Glib::ustring _msg;
      public:
        Error(const Glib::ustring &error) : _msg(error) {};
        virtual ~Error() throw() {};
        virtual Glib::ustring what() const { return _msg; };
    };
    
    typedef std::pair<Glib::ustring,Glib::ustring> AccelPair;
    typedef std::list<AccelPair> AccelList;

    typedef GtkWidget *(*CustomWidgetFunc)(gchar *widget_name, gchar *string1, gchar *string2, gint int1, gint int2);

  private:
    
    GladeXML *_xml;

  public:
    MGGladeXML(const std::string &file, const Glib::ustring &root=Glib::ustring(), const std::string &domain="");
    MGGladeXML(const char* buffer, const int  buffer_size, const char *root = 0, const char* domain = 0);
    virtual ~MGGladeXML();

    Gtk::Widget* get_widget(const Glib::ustring & name) const;
    Gtk::Widget* get(const Glib::ustring & name, Gtk::Widget** obj) const;
    Gtk::Window* get(const Glib::ustring & name, Gtk::Window** obj) const;
    Gtk::Entry* get(const Glib::ustring & name, Gtk::Entry** obj) const;
    Gtk::ToggleButton* get(const Glib::ustring & name, Gtk::ToggleButton** obj) const;
    Gtk::TreeView* get(const Glib::ustring & name, Gtk::TreeView** obj) const;
    Gtk::Button* get(const Glib::ustring & name, Gtk::Button** obj) const;
    Gtk::Label* get(const Glib::ustring & name, Gtk::Label** obj) const;
    Gtk::SpinButton* get(const Glib::ustring & name, Gtk::SpinButton** obj) const;
    Gtk::TextView* get(const Glib::ustring & name, Gtk::TextView** obj) const;
    Gtk::OptionMenu* get(const Glib::ustring & name, Gtk::OptionMenu** obj) const;
    Gtk::ComboBox* get(const Glib::ustring & name, Gtk::ComboBox** obj) const;
    Gtk::ComboBoxEntry* get(const Glib::ustring & name, Gtk::ComboBoxEntry** obj) const;
    Gtk::ComboBoxEntryText* get(const Glib::ustring & name, Gtk::ComboBoxEntryText** obj) const;
    Gtk::Entry* get_entry(const Glib::ustring & name, Gtk::ComboBoxEntry** obj) const;
    Gtk::ComboBoxText* get(const Glib::ustring & name, Gtk::ComboBoxText** obj) const;
    Gtk::Container* get(const Glib::ustring & name, Gtk::Container** obj) const;
    Gtk::Notebook* get(const Glib::ustring & name, Gtk::Notebook** obj) const;
    Gtk::MenuItem* get(const Glib::ustring & name, Gtk::MenuItem** obj) const;
    Gtk::CheckMenuItem* get(const Glib::ustring & name, Gtk::CheckMenuItem** obj) const;
    Gtk::Paned* get(const Glib::ustring & name, Gtk::Paned** obj) const;
    Gtk::Box* get(const Glib::ustring & name, Gtk::Box** obj) const;
    Gtk::IconView* get(const Glib::ustring & name, Gtk::IconView** obj) const;
    Gtk::Image* get(const Glib::ustring & name, Gtk::Image** obj) const;
    Gtk::EventBox* get(const Glib::ustring & name, Gtk::EventBox** obj) const;
    Gtk::VBox* get(const Glib::ustring & name, Gtk::VBox** obj) const;
    Gtk::HBox* get(const Glib::ustring & name, Gtk::HBox** obj) const;
    Gtk::Statusbar* get(const Glib::ustring & name, Gtk::Statusbar** obj) const;
    Gtk::MenuBar* get(const Glib::ustring & name, Gtk::MenuBar** obj) const;
    Gtk::HandleBox* get(const Glib::ustring & name, Gtk::HandleBox** obj) const;
    Gtk::Toolbar* get(const Glib::ustring & name, Gtk::Toolbar** obj) const;
    Gtk::Frame* get(const Glib::ustring & name, Gtk::Frame** obj) const;
    Gtk::Alignment* get(const Glib::ustring & name, Gtk::Alignment** obj) const;
    Gtk::ScrolledWindow* get(const Glib::ustring & name, Gtk::ScrolledWindow** obj) const;
  
    void bind_accelerators(const Glib::RefPtr<Gtk::AccelGroup> &group,
                           const AccelList &accels);
    
    template<class T> 
      void get_widget_derived(const Glib::ustring &name, T *&widget)
      {
          typedef typename T::BaseObjectType cwidget_type;
          cwidget_type *w= (cwidget_type*)glade_xml_get_widget(_xml, name.c_str());
          if (!w)
          {
            g_error("error retrieving widget %s from glade xml", name.c_str());
          }
          widget= new T(w);
      };

    template<class T> 
      void get_widget_derived2(const Glib::ustring &name, T *&widget)
      {
          typedef typename T::BaseObjectType cwidget_type;
          cwidget_type *w= (cwidget_type*)glade_xml_get_widget(_xml, name.c_str());
          if (!w)
          {
            g_error("error retrieving widget %s from glade xml", name.c_str());
          }
          widget= new T(w, this);
      };
};

//------------------------------------------------------------------------------
inline Gtk::Widget* MGGladeXML::get_widget(const Glib::ustring & name) const
{
  Gtk::Widget* w(0);
  return get(name, &w); // see overloaded get(..., Gtk::Widget**...);
}

#endif /* _MGGLADEXML_H_ */
