#ifndef _GTK_HELPERS_H_
#define _GTK_HELPERS_H_

#include <gtkmm.h>
#include <vector>
#include <string>
#include <tree_model.h>

class TreeModelWrapper;
void expand_tree_nodes_as_in_be(const Glib::RefPtr<TreeModelWrapper> &model, Gtk::TreeView *tv);

class WidgetsAutoCleaner
{
  public:
    virtual ~WidgetsAutoCleaner();
    template <typename T>
    T* manage(T* w)
    {
      add(w);
      return w;
    }
    void delete_widgets();
  private:
    void add(Gtk::Widget* w);
    std::vector<Gtk::Widget*>	_widgets;
};

struct TextListColumnsModel : public Gtk::TreeModelColumnRecord
{
  public:
    TextListColumnsModel() {add(item); }
    Gtk::TreeModelColumn<std::string>  item;
};

//==============================================================================
class AutoCompletable
{
  public:
    AutoCompletable(Gtk::Entry* entry = 0);
    void add_completion_text(const std::string& s);
    void add_to_entry(Gtk::Entry* entry);
    void clear();
  private:
    TextListColumnsModel                _completion_columns;
    Glib::RefPtr<Gtk::ListStore>        _completion_model;
    Glib::RefPtr<Gtk::EntryCompletion>  _completion;
};

Gtk::HBox &create_icon_label(const std::string &icon, const std::string &label);

void open_url(const std::string &url);

Glib::RefPtr<Gtk::ListStore> model_from_string_list(const std::vector<std::string>& list, TextListColumnsModel** columns = 0);
void recreate_model_from_string_list(Glib::RefPtr<Gtk::ListStore>, const std::vector<std::string>& list);

void set_glib_string(Glib::ValueBase& value, const std::string& str);
void set_glib_int(Glib::ValueBase& value, const int i);
void set_glib_bool(Glib::ValueBase& value, const bool b);
void set_glib_double(Glib::ValueBase& value, const double d);

void fill_combo_from_string_list(Gtk::ComboBoxText* combo, const std::vector<std::string>& list);
void fill_combo_from_string_list(Gtk::ComboBoxEntryText* combo, const std::vector<std::string>& list);

// get_mainwindow is declared here as extern while it is implemented in frontend/linux/workbench/Program.cpp
// and frontend/linux/shell/shell.cpp. get_mainwindow is needed by some functions/methods to
// set dialog transient.
extern Gtk::Window* get_mainwindow();
extern std::string open_file_chooser();
extern std::string save_file_chooser();

std::string run_string_dialog(const std::string& title, const std::string& init_value);

#endif /* _GTK_HELPERS_H_ */
