#ifndef __ERRORS_LIST_H__
#define __ERRORS_LIST_H__

#include <gtkmm.h>
#include <sigc++/sigc++.h>
#include "editor_dbobject.h"

//==============================================================================
//
//==============================================================================
class ErrorsList
{
  public:
    ErrorsList(bec::DBObjectEditorBE* be);
    ~ErrorsList();
    
    Gtk::TreeView& widget();
    
    void clear_list();
    void add_error(const int line, const std::string &msg);
    typedef sigc::signal<void, const int, const std::string&> ErrorSelectedSignal;
    ErrorSelectedSignal& signal_error_selected() { return _error_selected_signal; };

  private:
    void error_selected();

    struct ErrorColumns : public Gtk::TreeModel::ColumnRecord
    {
      ErrorColumns()
      {
        add(lineno);
        add(msg);
      }
      
      Gtk::TreeModelColumn<int>         lineno;
      Gtk::TreeModelColumn<std::string> msg;
    };

    Gtk::TreeView                            *_errors_tv;
    ErrorColumns                              _errors_columns;
    Glib::RefPtr<Gtk::ListStore>              _errors_model;
    ErrorSelectedSignal                       _error_selected_signal;
    bec::DBObjectEditorBE                    *_be;
};

//==============================================================================
//
//==============================================================================
class TrackedPane
{
  public:
    TrackedPane(Gtk::Paned* pane, bec::DBObjectEditorBE* be);
    void show2nd(const int default_2nd_size);
    void max1st();
    
  private:
    void editor_parse_log_cb(const std::vector<std::string>& list);
    void track_pane();

    Gtk::Paned             *_pane;
    int                     _pane_position;
    sigc::connection        _pane_pos_changed_conn;
    bec::DBObjectEditorBE  *_be;    
};

#endif
