#include <gtk/gtk.h>
#include "sql_editor_fe.h"

//------------------------------------------------------------------------------
SqlEditorFE::SqlEditorFE()
{
  _editor = scintilla_new();
  _sci    = SCINTILLA(_editor);
  _widget = Glib::wrap(_editor);

  send_editor(SCI_SETSCROLLWIDTH, 400);  
  send_editor(SCI_SETMARGINTYPEN, 0, SC_MARGIN_NUMBER);
  send_editor(SCI_SETMARGINWIDTHN, 0, 20);
  send_editor(SCI_SETSTYLEBITS, send_editor(SCI_GETSTYLEBITSNEEDED));
  init_lexer();
  
  gtk_signal_connect(GTK_OBJECT(_editor), SCINTILLA_NOTIFY, GTK_SIGNAL_FUNC(&SqlEditorFE::notify_signal), this);
}

//------------------------------------------------------------------------------
void SqlEditorFE::notify_signal(GtkWidget *w, gint wParam, gpointer lParam, SqlEditorFE *editor)
{
  SCNotification *event = reinterpret_cast<SCNotification *>(lParam);
  if ( event->nmhdr.code == SCN_MODIFIED )
  {
    const int mod_type = event->modificationType;
    if ( mod_type & SC_MOD_INSERTTEXT || mod_type & SC_MOD_DELETETEXT )
      editor->_text_changed_signal.emit();
  }
}

//------------------------------------------------------------------------------
Gtk::Widget& SqlEditorFE::widget()
{
  return *_widget;
}

//------------------------------------------------------------------------------
void SqlEditorFE::set_text(const std::string& text)
{
  const int pos = send_editor(SCI_GETCURRENTPOS);
  send_editor(SCI_SETTEXT, 0, (sptr_t)text.c_str());
  send_editor(SCI_GOTOPOS, pos);
  send_editor(SCI_SCROLLCARET);
}

//------------------------------------------------------------------------------
std::string SqlEditorFE::get_text()
{
  char *buffer(0);
  const int length = send_editor(SCI_GETLENGTH);
  if ( length > 0 )
  {
    buffer = new char[length+1];
    try
    {
      send_editor(SCI_GETTEXT, length+1, (sptr_t)buffer);
      send_editor(SCI_SETSAVEPOINT);
    }
    catch (...)
    {
      delete[] buffer;
      buffer = 0;
    }
  }
  
  return buffer ? buffer : "";
}

//------------------------------------------------------------------------------
void SqlEditorFE::scroll_to(const int line, const std::string& msg)
{
  send_editor(SCI_GOTOLINE, line);
}

//------------------------------------------------------------------------------
sptr_t SqlEditorFE::send_editor(unsigned int msg, uptr_t uparam, sptr_t sparam)
{
  return scintilla_send_message(_sci, msg, uparam, sparam);
}

static const char* keywords = 
"absolute action add admin after aggregate "
"alias all allocate alter and any are array as asc "
"assertion at authorization "
"before begin binary bit blob boolean both breadth by "
"call cascade cascaded case cast catalog char character "
"check class clob close collate collation column commit "
"completion connect connection constraint constraints "
"constructor continue corresponding create cross cube current "
"current_date current_path current_role current_time current_timestamp "
"current_user cursor cycle "
"data date day deallocate dec decimal declare default "
"deferrable deferred delete depth deref desc describe descriptor "
"destroy destructor deterministic dictionary diagnostics disconnect "
"distinct domain double drop dynamic "
"each else end end-exec equals escape every except "
"exception exec execute external "
"false fetch first float for foreign found from free full "
"function "
"general get global go goto grant group grouping "
"having host hour "
"identity if ignore immediate in indicator initialize initially "
"inner inout input insert int integer intersect interval "
"into is isolation iterate "
"join "
"key "
"language large last lateral leading left less level like "
"limit local localtime localtimestamp locator "
"map match minute modifies modify module month "
"names national natural nchar nclob new next no none "
"not null numeric "
"object of off old on only open operation option "
"or order ordinality out outer output "
"pad parameter parameters partial path postfix precision prefix "
"preorder prepare preserve primary "
"prior privileges procedure public "
"read reads real recursive ref references referencing relative "
"restrict result return returns revoke right "
"role rollback rollup routine row rows "
"savepoint schema scroll scope search second section select "
"sequence session session_user set sets size smallint some| space "
"specific specifictype sql sqlexception sqlstate sqlwarning start "
"state statement static structure system_user "
"table temporary terminate than then time timestamp "
"timezone_hour timezone_minute to trailing transaction translation "
"treat trigger true "
"under union unique unknown "
"unnest update usage user using "
"value values varchar variable varying view "
"when whenever where with without work write "
"year "
"zone";

//------------------------------------------------------------------------------
void SqlEditorFE::init_lexer()
{
  send_editor(SCI_SETLEXER, SCLEX_SQL);
  send_editor(SCI_SETKEYWORDS, 0, (sptr_t)keywords);

  send_editor(SCI_STYLESETFORE, SCE_SQL_DEFAULT, 0x808080);
  send_editor(SCI_STYLESETFORE, SCE_SQL_COMMENT, 0x007F00);
  send_editor(SCI_STYLESETFORE, SCE_SQL_COMMENTLINE, 0x007F00);
  send_editor(SCI_STYLESETFORE, SCE_SQL_COMMENTDOC, 0x7F7F7F);
  send_editor(SCI_STYLESETFORE, SCE_SQL_NUMBER, 0x007F7F);
  send_editor(SCI_STYLESETFORE, SCE_SQL_WORD, 0x7F0000);
  send_editor(SCI_STYLESETBOLD, SCE_SQL_WORD, 1);
  send_editor(SCI_STYLESETFORE, SCE_SQL_STRING, 0x7F007F);
  send_editor(SCI_STYLESETFORE, SCE_SQL_CHARACTER, 0x7F007F);
  send_editor(SCI_STYLESETFORE, SCE_SQL_SQLPLUS, 0x808080);
  send_editor(SCI_STYLESETFORE, SCE_SQL_SQLPLUS_PROMPT, 0x007F00);//,$(font.monospace),back:0xE0FFE0,eolfilled
  send_editor(SCI_STYLESETBOLD, SCE_SQL_OPERATOR, 1);
  //send_editor(SCI_STYLESETFORE, SCE_SQL_IDENTIFIER, );
  //send_editor(SCI_STYLESETFORE, SCE_SQL_SQLPLUS_COMMENT, :0x000000,$(font.monospace),back:0xE0C0E0,eolfilled
  send_editor(SCI_STYLESETFORE, SCE_SQL_SQLPLUS_COMMENT, 0x007F00);
  send_editor(SCI_STYLESETFORE, SCE_SQL_COMMENTLINEDOC, 0x007F00);
  send_editor(SCI_STYLESETFORE, SCE_SQL_WORD2, 0xB00040);
  //send_editor(SCI_STYLESETFORE, SCE_SQL_17=fore:0x3060A0,$(font.code.comment.doc)
  //send_editor(SCI_STYLESETFORE, SCE_SQL_18=fore:0x804020,$(font.code.comment.doc)
  //send_editor(SCI_STYLESETFORE, SCE_SQL_19=fore:0x4B0082
  //send_editor(SCI_STYLESETFORE, SCE_SQL_20=fore:0xB00040
  //send_editor(SCI_STYLESETFORE, SCE_SQL_21=fore:0x8B0000
  //send_editor(SCI_STYLESETFORE, SCE_SQL_22=fore:0x800080
}
