#include "stdafx.h"

#ifndef _WIN32
#include <list>
#include <string>
#endif

#include "driver_manager.h"
#include <gmodule.h>
#include <cppconn/connection.h>
#include <cppconn/statement.h>
#include <cppconn/exception.h>


namespace sql {


DriverManager *DriverManager::getDriverManager()
{
  static DriverManager *dm= new DriverManager;
  return dm;
}


DriverManager::DriverManager()
:
_driver_path(".")
{
}


void DriverManager::set_driver_dir(const std::string &path)
{
  _driver_path= path;
}


std::auto_ptr<Connection> DriverManager::getConnection(const db_mgmt_ConnectionRef &connectionProperties)
{
  // 0. determine correct driver filename

  std::string library= connectionProperties->driver()->driverLibraryName();

#ifdef _WIN32
  library.append(".dll");
#elif defined(__APPLE__)
  library.append(".dylib");
#else
  library.append(".so");
#endif

  // 1. find driver

  GModule *gmodule= g_module_open((_driver_path + "/" + library).c_str(), G_MODULE_BIND_LOCAL);
  if (NULL == gmodule)
    throw SQLException(std::string("Database driver: Failed to open library '").append(_driver_path + "/" + library).append("'. Check settings.").c_str());

  Driver *(* get_driver_instance)()= NULL;
  g_module_symbol(gmodule, "get_driver_instance", (gpointer*)&get_driver_instance);
  if (NULL == get_driver_instance)
    throw SQLException("Database driver: Failed to get library instance. Check settings.");

  // 2. call driver->connect()

  grt::DictRef conn_params= connectionProperties->parameterValues();

  //! to be replaced with name-value list
  std::string hostname= conn_params.get_string("host");
  std::string username= conn_params.get_string("username");
  std::string password= conn_params.get_string("password");

  //grt::StringRef host(conn_params.has_key("host") ? (grt::internal::String*)conn_params["host"].valueptr() : NULL);
  //grt::IntegerRef port(conn_params.has_key("port") ?  (grt::internal::Integer*)conn_params["port"].valueptr() : NULL);
  //grt::StringRef username(conn_params.has_key("username") ?  (grt::internal::String*)conn_params["username"].valueptr() : NULL);
  //grt::StringRef password(conn_params.has_key("password") ?  (grt::internal::String*)conn_params["password"].valueptr() : NULL);
  //grt::StringRef schema(conn_params.has_key("schema") ?  (grt::internal::String*)conn_params["schema"].valueptr() : NULL);
  //grt::StringRef socket(conn_params.has_key("socket") ?  (grt::internal::String*)conn_params["socket"].valueptr() : NULL);

  Driver *driver= get_driver_instance();
  if (NULL == driver)
    throw SQLException("Database driver: Failed to get driver instance. Check  settings.");

  std::auto_ptr<Connection> conn(driver->connect(hostname, username, password));

  std::auto_ptr<Statement> stmt(conn->createStatement());
  stmt->execute("SET CHARACTER SET utf8");
  stmt->execute("SET NAMES utf8");

  return conn;
}


} // namespace sql
