
#include <mforms/form.h>
#include <mforms/mforms.h>

using namespace mforms;

Form::Form(Form *owner)
{
  _form_impl= &ControlFactory::get_instance()->_form_impl;
  
  _content= 0;
  _fixed_size= false;
  _form_impl->create(this, owner);
}

void Form::set_title(const std::string &title)
{
  _form_impl->set_title(this, title);
}

bool Form::run_modal(Button *accept, Button *cancel)
{
  return _form_impl->run_modal(this, accept, cancel);
}

void Form::show_modal(Button *accept, Button *cancel)
{
  _form_impl->show_modal(this, accept, cancel);
}

void Form::close()
{
  _form_impl->close(this);
}


void Form::set_content(View *view)
{
  _content= view;
  _form_impl->set_content(this, view);
  view->set_parent(this);
}


void Form::needs_resize()
{
  relayout();
}


void Form::relayout()
{
  if (_content)
  {
    _content->set_size(get_width(), get_height());
    _content->relayout();
  }
}


void Form::flush_events()
{
  _form_impl->flush_events(this);
}



int Form::show_message(const std::string &title, const std::string &text,
                       const std::string &ok, const std::string &cancel,
                       const std::string &other)
{
  return ControlFactory::get_instance()->_form_impl.show_message(title, text, ok, cancel, other);
}


int Form::show_error(const std::string &title, const std::string &text,
                       const std::string &ok, const std::string &cancel,
                       const std::string &other)
{
  return ControlFactory::get_instance()->_form_impl.show_error(title, text, ok, cancel, other);
}


int Form::show_warning(const std::string &title, const std::string &text,
                       const std::string &ok, const std::string &cancel,
                       const std::string &other)
{
  return ControlFactory::get_instance()->_form_impl.show_warning(title, text, ok, cancel, other);
}

