#ifndef _LF_PANEL_H_
#define _LF_PANEL_H_


#include "mforms/panel.h"

#include "lfi_container.h"

namespace mforms
{
namespace gtk
{

class PanelImpl : public ContainerImpl
{
  Gtk::Frame *_frame;
  Gtk::EventBox *_evbox;
  Gtk::CheckButton *_title_check;

  Gtk::RadioButtonGroup _radio_group;
  bool _radio_group_set;

  virtual Gtk::Widget *get_outer() const { if (_frame) return _frame; else return _evbox; }
  virtual Gtk::Widget *get_inner() const { return _evbox; }

protected:
  PanelImpl(::mforms::Panel *self, bool bordered)
    : ContainerImpl(self),  _radio_group_set(false)
  {
    _title_check= 0;

    // if possible, don't create a borderless frame because
    // that adds a stupid 1px border around it that nobody has asked for....
    if (bordered)
      _frame= Gtk::manage(new Gtk::Frame());
    else
      _frame= 0;
    _evbox= Gtk::manage(new Gtk::EventBox());
    _evbox->show();
    if (bordered)
    {
      _frame->add(*_evbox);
      _frame->set_border_width(0);
      _frame->set_shadow_type(bordered ? Gtk::SHADOW_ETCHED_IN : Gtk::SHADOW_NONE);
    }
  }

  static bool create(::mforms::Panel *self, bool bordered)
  {
    return new PanelImpl(self, bordered);
  }

  static void set_title(::mforms::Panel *self, const std::string &title)
  {
    PanelImpl *panel= self->get_data<PanelImpl>();

    panel->_title_check= 0;
    if (!panel->_frame)
    {
      panel->_frame= Gtk::manage(new Gtk::Frame());
      panel->_frame->add(*panel->_evbox);
    }
    panel->_frame->set_label(title.c_str());
  }

  static void set_check_title(::mforms::Panel *self, const std::string &title)
  {
    PanelImpl *panel= self->get_data<PanelImpl>();

    if (panel->_title_check)
      panel->_title_check->set_label(title);
    else
    {
      panel->_title_check= Gtk::manage(new Gtk::CheckButton(title));
      if (!panel->_frame)
      {
        panel->_frame= Gtk::manage(new Gtk::Frame());
        panel->_frame->add(*panel->_evbox);
      }
      panel->_frame->set_label_widget(*panel->_title_check);
      panel->_title_check->show();
    }
  }


  static void set_active(::mforms::Panel *self, bool flag)
  {
    PanelImpl *panel= self->get_data<PanelImpl>();

    if (panel->_title_check)
      panel->_title_check->set_active(flag);
  }

  
  static bool get_active(::mforms::Panel *self)
  {
    PanelImpl *panel= self->get_data<PanelImpl>();

    if (panel->_title_check)
      return panel->_title_check->get_active();
    return false;
  }

  static void set_back_color(::mforms::Panel *self, const std::string &color)
  {
    PanelImpl *panel= self->get_data<PanelImpl>();

    Gdk::Color c(color);
    panel->_evbox->get_colormap()->alloc_color(c);

    panel->_evbox->modify_bg(Gtk::STATE_NORMAL, c);
    panel->_evbox->modify_base(Gtk::STATE_NORMAL, c);
  }


  static void set_padding(::mforms::Panel *self, int pad)
  {
    PanelImpl *panel= self->get_data<PanelImpl>();
    
    panel->_evbox->set_border_width(pad);
  }
  

  static void add(::mforms::Panel *self, ::mforms::View *child)
  {
    PanelImpl *panel= self->get_data<PanelImpl>();

    panel->_evbox->add(*child->get_data<ViewImpl>()->get_outer());
  }

  static void remove(::mforms::Panel *self, ::mforms::View *child)
  {
    PanelImpl *panel= self->get_data<PanelImpl>();

    panel->_evbox->remove();
  }

 public:
  static void init()
  {
    ::mforms::ControlFactory *f = ::mforms::ControlFactory::get_instance();

    f->_panel_impl.create         = &PanelImpl::create;
    f->_panel_impl.set_title      = &PanelImpl::set_title;
    f->_panel_impl.set_back_color = &PanelImpl::set_back_color;
    
    f->_panel_impl.set_padding    = &PanelImpl::set_padding;

    f->_panel_impl.add            = &PanelImpl::add;
    f->_panel_impl.remove         = &PanelImpl::remove;
    
    f->_panel_impl.set_active     = &PanelImpl::set_active;
    f->_panel_impl.get_active     = &PanelImpl::get_active;
    f->_panel_impl.set_check_title= &PanelImpl::set_check_title;
  }


  // called by radiobutton to add itself to a radio group
  void add_to_radio_group(Gtk::RadioButton *radio)
  {
    if (!_radio_group_set)
    {
      _radio_group_set= true;
      _radio_group= radio->get_group();
    }
    else
      radio->set_group(_radio_group);
  }
};

};
};


#endif
