#ifndef _LF_RADIOBUTTON_H_
#define _LF_RADIOBUTTON_H_

#include <mforms/radiobutton.h>


#include "lf_view.h"
#include "lf_panel.h"

namespace mforms {
namespace gtk {

class RadioButtonImpl : public ButtonImpl
{
 protected:
  Gtk::RadioButton *_radio;

  virtual Gtk::Widget *get_outer() const { return _radio; }

  RadioButtonImpl(::mforms::RadioButton *self)
    : ButtonImpl(self)
  {
    _radio= Gtk::manage(new Gtk::RadioButton());
    _button= _radio;

    _radio->signal_parent_changed().connect(sigc::mem_fun(this, &RadioButtonImpl::parent_changed));
    _radio->signal_toggled().connect(sigc::mem_fun(*self, &::mforms::RadioButton::callback));
  }

  static bool create(::mforms::RadioButton *self)
  {
    return new RadioButtonImpl(self);
  }

  static bool get_active(::mforms::RadioButton *self)
  {
    RadioButtonImpl* button = self->get_data<RadioButtonImpl>();
    
    if ( button )
    {
      return button->_radio->get_active();
    }
    return false;
  }
  
  
  static void set_active(::mforms::RadioButton *self, bool flag)
  {
    RadioButtonImpl* button = self->get_data<RadioButtonImpl>();
    
    if ( button )
    {
      button->_radio->set_active(flag);
    }
  }


  void parent_changed(Gtk::Widget *old_parent)
  {
    // find panel parent
    View *view= (View*)owner;
    Panel *panel;

    while (view && (panel= dynamic_cast<Panel*>(view)) == 0)
      view= view->get_parent();

    if (panel)
      panel->get_data<PanelImpl>()->add_to_radio_group(_radio);
  }

public:
  static void init()
  {
    ::mforms::ControlFactory *f = ::mforms::ControlFactory::get_instance();

    f->_radio_impl.create= &RadioButtonImpl::create;
    f->_radio_impl.get_active= &RadioButtonImpl::get_active;
    f->_radio_impl.set_active= &RadioButtonImpl::set_active;
  }
};

}
}



#endif /* _LF_RADIOBUTTON_H_ */
