#ifndef _MFORMS_FORM_H_
#define _MFORMS_FORM_H_

#include <mforms/base.h>
#include <mforms/view.h>


namespace mforms {
  enum FormFlag
  {
    FormResizable=  (1<<0),
    FormMinimizable=(1<<1)
  };

  class Form;
  class Button;

  struct MFORMS_EXPORT FormImplPtrs
  {
    bool (__stdcall *create)(Form *self, Form *owner);
    void (__stdcall *set_title)(Form *self, const std::string &title);
    void (__stdcall *show_modal)(Form *self, Button *accept, Button *cancel);
    bool (__stdcall *run_modal)(Form *self, Button *accept, Button *cancel);
    void (__stdcall *set_flags)(Form *self, FormFlag flag);
    void (__stdcall *set_content)(Form *self, View *view);
    void (__stdcall *close)(Form *self);

    void (__stdcall *flush_events)(Form *self);
    
    int (__stdcall *show_message)(const std::string &title, const std::string &text,
                                  const std::string &ok, const std::string &cancel,
                                  const std::string &other);
    int (__stdcall *show_error)(const std::string &title, const std::string &text,
                                  const std::string &ok, const std::string &cancel,
                                  const std::string &other);
    int (__stdcall *show_warning)(const std::string &title, const std::string &text,
                                  const std::string &ok, const std::string &cancel,
                                  const std::string &other);
  };

  class MFORMS_EXPORT Form : public View
  {
    friend class ControlFactory;

    FormImplPtrs *_form_impl;
    View *_content;
    bool _fixed_size;
  public:
    Form(Form *owner= 0);

    virtual void set_content(View *view);
    virtual void set_title(const std::string &title);
    virtual void show_modal(Button *accept, Button *cancel);
    virtual bool run_modal(Button *accept, Button *cancel);
    virtual void close();

    virtual void flush_events();

    virtual void relayout();


    static int show_message(const std::string &title, const std::string &text,
                            const std::string &ok, const std::string &cancel="",
                            const std::string &other="");
    static int show_error(const std::string &title, const std::string &text,
                            const std::string &ok, const std::string &cancel="",
                            const std::string &other="");
    static int show_warning(const std::string &title, const std::string &text,
                            const std::string &ok, const std::string &cancel="",
                            const std::string &other="");

  protected:
    virtual void needs_resize();
  };
};

#endif
