#ifndef _MFORMS_GRTTREEVIEW_H_
#define _MFORMS_GRTTREEVIEW_H_

#include <mforms/base.h>
#include <mforms/control.h>


namespace bec {
  class TreeModel;
  struct NodeId;
};


namespace mforms {
  class GRTTreeView;
  
  enum GRTTreeColumnType
  {
    StringGRTColumnType,
    IconStringGRTColumnType,
    IntegerGRTColumnType,
    CheckGRTColumnType,
    IconGRTColumnType
  };


  struct GRTTreeViewImplPtrs
  {
    bool (__stdcall *create)(GRTTreeView *self);
    
    void (__stdcall *set_model)(GRTTreeView *self, bec::TreeModel *model);

    int (__stdcall *add_column)(GRTTreeView *self, GRTTreeColumnType type, int model_column, 
                                const std::string &name);

    int (__stdcall *add_column_editable)(GRTTreeView *self, GRTTreeColumnType type, int model_column, 
                                const std::string &name);
    
    void (__stdcall *set_column_width)(GRTTreeView *self, int model_column, int width);

    void (__stdcall *refresh)(GRTTreeView *self, const bec::NodeId &node);
  };


  class MFORMS_EXPORT GRTTreeView : public Control
  {
  public:
    GRTTreeView();
    
    void set_model(bec::TreeModel *model);

    int add_column(GRTTreeColumnType type, int model_column, const std::string &name);
    int add_column_editable(GRTTreeColumnType type, int model_column, const std::string &name);
    
    void set_column_width(int model_column, int width);

    void refresh(const bec::NodeId &node); // call with bec::NodeId() for a full refresh

    sigc::signal<void, bec::NodeId, int> signal_row_activate() { return _activate_signal; }
    
  public:
    void row_activate_callback(const bec::NodeId &node, int column);
    
  protected:
    GRTTreeViewImplPtrs *_grttreeview_impl;
    sigc::signal<void, bec::NodeId, int> _activate_signal;
  };
}

#endif /* _GRTTREEVIEW_H_ */
