#ifndef _MFORMS_PANEL_H_
#define _MFORMS_PANEL_H_

#include <mforms/base.h>
#include <mforms/view.h>


namespace mforms {
  enum BorderType {
    None,
    SingleLine,
    Ridge,
    Groove,
    Sunken,
    Raised
  };

  class Panel;

  struct MFORMS_EXPORT PanelImplPtrs
  {
    bool (__stdcall *create)(Panel*, bool bordered);
    void (__stdcall *set_back_color)(Panel*, const std::string &);
    void (__stdcall *set_title)(Panel*, const std::string&);
    void (__stdcall *set_check_title)(Panel*, const std::string&);

    void (__stdcall *set_active)(Panel*, bool);
    bool (__stdcall *get_active)(Panel*);
    
    void (__stdcall *set_padding)(Panel*, int);

    void (__stdcall *add)(Panel *self,View*);
    void (__stdcall *remove)(Panel *self,View*);
  };

  class MFORMS_EXPORT Panel : public View
  {
  public:
    Panel(bool bordered);
    void set_title(const std::string &title);
    void set_back_color(const std::string &color);

    void set_check_title(const std::string &title);
    void set_active(bool);
    bool get_active();
    
    void add(View *subview);
    virtual void remove(View *subview);
    void set_padding(int pad);

    virtual void relayout() {}

    View *find_subview(const std::string &name)
    {
      for (std::list<View*>::const_iterator iter= _subviews.begin(); iter != _subviews.end(); ++iter)
      {
        if ((*iter)->get_name() == name)
          return *iter;

        View *sv= dynamic_cast<Panel*>((*iter))->find_subview(name);
        if (sv)
          return sv;
      }
      return 0;
    }
    
  protected:
    PanelImplPtrs *_panel_impl;
  };
};

#endif
