#ifndef _MFORMS_SIMPLEFORM_H_
#define _MFORMS_SIMPLEFORM_H_

#include <mforms/base.h>
#include <string>
#include <list>
#include <map>

namespace mforms {

  class Form;
  class Label;
  class View;
  class Button;
  class Control;
  class Panel;
  class Box;
  class Table;

  class MFORMS_EXPORT SimpleForm
  {
  public:
    SimpleForm(const std::string &title, const std::string &ok_caption);
    ~SimpleForm();

    void parse_definition(const std::string &definition);

    //void begin_group();
    //void end_group();

    void add_label(const std::string &text, bool bold);

    void add_file_picker(const std::string &name, const std::string &caption, 
      const std::string &default_value="");

    void add_text_entry(const std::string &name, const std::string &caption, 
      const std::string &default_value="");

    void add_text_area(const std::string &name, const std::string &caption, 
      int rows, const std::string &default_value="");

    void add_checkbox(const std::string &name, const std::string &caption, 
      bool default_value= false);

    void add_select(const std::string &name, const std::string &caption,
      const std::list<std::string> &items, int defualt_index=-1);

    bool show();

    std::map<std::string,Control*> get_controls();
    std::string get_string_value(const std::string &name);
    bool get_bool_value(const std::string &name);
    int get_int_value(const std::string &name);

  private:
    struct Row
    {
      Label *caption;
      Control *control;
      int spacing;
      bool fullwidth;
    };
    std::list<Row> _rows;

    Form *_form;
    Table *_content;
    Box *_button_box;
    Button *_ok_button;
    Button *_cancel_button;
    std::string _ok_caption;
    int _caption_width;
    int _control_width;
    int _title_width;
  };

};

#endif
