#ifndef _MFORMS_TABLE_H_
#define _MFORMS_TABLE_H_

#include <mforms/base.h>
#include <mforms/view.h>


namespace mforms {

  class Table;
  
  enum TableItemFlags
  {
    VExpandFlag = (1 << 0),
    HExpandFlag = (1 << 1),
    VFillFlag = (1 << 2),
    HFillFlag = (1 << 3),
  };

  struct MFORMS_EXPORT TableImplPtrs
  {
    bool (__stdcall *create)(Table*);

    void (__stdcall *set_row_count)(Table *self, int);
    void (__stdcall *set_column_count)(Table *self, int);

    void (__stdcall *add)(Table *self,View*, int, int, int, int, int);
    void (__stdcall *remove)(Table *self, View*);
    
    void (__stdcall *set_padding)(Table *self, int);

    void (__stdcall *set_row_spacing)(Table *self, int);
    void (__stdcall *set_column_spacing)(Table *self, int);

    void (__stdcall *set_homogeneous)(Table *self, bool);
  };

  class MFORMS_EXPORT Table : public View
  {
  public:
    Table();

    void set_row_count(int c);
    void set_column_count(int c);

    void set_row_spacing(int s);
    void set_column_spacing(int s);
    
    void set_padding(int pad);
    void set_homogeneous(bool value);

    void add(View *subview, int left, int right, int top, int bottom, int flags= HExpandFlag | VExpandFlag | HFillFlag | VFillFlag);

    virtual void remove(View *subview);
    
    virtual void relayout() {}

    View *find_subview(const std::string &name);

  protected:
    TableImplPtrs *_table_impl;
  };
};


#endif /* _MFORMS_TABLE_H_ */
