#ifndef _MFORMS_TREEVIEW_H_
#define _MFORMS_TREEVIEW_H_

#include <mforms/base.h>
#include <mforms/control.h>


namespace mforms {
  class TreeView;
  
  enum TreeColumnType
  {
    StringColumnType,
    IntegerColumnType,
    CheckColumnType
  };


  struct TreeViewImplPtrs
  {
    bool (__stdcall *create)(TreeView *self, bool show_header);
    
    int (__stdcall *add_column)(TreeView *self, TreeColumnType type, const std::string &name, bool editable);
    void (__stdcall *end_columns)(TreeView *self);
    
    void (__stdcall *clear_rows)(TreeView *self);

    void (__stdcall *delete_row)(TreeView *self, int row);
    int (__stdcall *add_row)(TreeView *self);

    int (__stdcall *count)(TreeView *self);

    void (__stdcall *set_string)(TreeView *self, int row, int column, const std::string &value);
    void (__stdcall *set_int)(TreeView *self, int row, int column, int value);
    void (__stdcall *set_check)(TreeView *self, int row, int column, bool value);

    std::string (__stdcall *get_string)(TreeView *self, int row, int column);
    int (__stdcall *get_int)(TreeView *self, int row, int column);
    bool (__stdcall *get_check)(TreeView *self, int row, int column);
  };


  class MFORMS_EXPORT TreeView : public Control
  {
  public:
    TreeView(bool show_header);

    int add_column(TreeColumnType type, const std::string &name, bool editable);
    void end_columns();

    void clear_rows();
    void delete_row(int row);
    int add_row();

    int count();

    void set(int row, int column, const std::string &value);
    void set(int row, int column, int value);
    void set(int row, int column, bool check);

    std::string get_string(int row, int column);
    int get_int(int row, int column);
    bool get_check(int row, int column);
    
  protected:
    TreeViewImplPtrs *_treeview_impl;
  };
}

#endif /* _TREEVIEW_H_ */
