#ifndef _MFORMS_VIEW_H_
#define _MFORMS_VIEW_H_

#include <mforms/base.h>
#include <sigc++/sigc++.h>

namespace mforms {

  class View;

  struct ViewImplPtrs
  {
    int (__stdcall *get_width)(View *self);
    int (__stdcall *get_height)(View *self);
    int (__stdcall *get_preferred_width)(View *self);
    int (__stdcall *get_preferred_height)(View *self);
    void (__stdcall *set_size)(View *self, int, int);

    int (__stdcall *get_x)(View *self);
    int (__stdcall *get_y)(View *self);
    void (__stdcall *set_position)(View *self, int, int);

    void (__stdcall *show)(View *self, bool);
    bool (__stdcall *is_shown)(View *self);
    
    void (__stdcall *set_tooltip)(View *self, const std::string&);
    void (__stdcall *set_name)(View *self, const std::string&);
    
    void (__stdcall *set_enabled)(View *self, bool);
    void (__stdcall *relayout)(View *self);
  };

  // Describes in which direction a control might be resized by the layouters.
  enum AutoResizeMode {
    ResizeNone,
    ResizeVertical,
    ResizeHorizontal,
    ResizeBoth
  };

  class MFORMS_EXPORT View : public Object
  {
    friend class ControlFactory;

    ViewImplPtrs *_view_impl;
  private:
    std::string _name;
    AutoResizeMode _resize_mode;
  protected:
    View();

    View *_parent;
    std::list<View*> _subviews;

    sigc::signal<void> _signal_resized;

  public:
    virtual ~View() { 
#ifdef _DEBUG
      printf("destroy view %p\n", this); 
#endif
    };
    
    AutoResizeMode get_resize_mode();
    void set_resize_mode(AutoResizeMode mode);

    sigc::signal<void> signal_resized() { return _signal_resized; }
    
    void set_name(const std::string &name)
    {
      _name= name;

      // Optional implementation.
      if (_view_impl->set_name)
        _view_impl->set_name(this, name);
    }
    
    void set_tooltip(const std::string &text)
    {
      _view_impl->set_tooltip(this, text);
    }

    std::string get_name() { return _name; }

    void set_parent(View *parent)
    {
      _parent= parent;
    }
    
    View *get_parent() const { return _parent; }

    virtual int get_width()
    {
      return (*_view_impl->get_width)(this);
    }

    virtual int get_height()
    {
      return (*_view_impl->get_height)(this);
    }

    virtual int get_preferred_width()
    {
      return (*_view_impl->get_preferred_width)(this);
    }

    virtual int get_preferred_height()
    {
      return (*_view_impl->get_preferred_height)(this);
    }

    virtual int get_x()
    {
      return (*_view_impl->get_x)(this);
    }

    virtual int get_y()
    {
      return (*_view_impl->get_y)(this);
    }

    virtual void set_position(int x, int y)
    {
      (*_view_impl->set_position)(this, x, y);
    }

    virtual void set_size(int width, int height)
    {
      (*_view_impl->set_size)(this, width, height);
    }

    void show(bool flag= true)
    {
      (*_view_impl->show)(this, flag);
    }
    
    bool is_shown()
    {
      return (*_view_impl->is_shown)(this);
    }

    void set_enabled(bool flag)
    {
      (*_view_impl->set_enabled)(this, flag);
    }
    
    virtual void needs_resize()
    {
      if (_parent)
        _parent->needs_resize();
    }
    
    virtual void relayout()
    {
      if (_view_impl->relayout)
        (*_view_impl->relayout)(this);
    }
    
    virtual void on_resize()
    {
      needs_resize();
      _signal_resized.emit();
    }
  };
};

#endif
