


#include "grtpp.h"
#include "grtpp_module_cpp.h"
#include "grtpp_util.h"

using namespace grt;

//--------------------------------------------------------------------------------

// A bundle (as in mac os x bundles) is a directory tree containing all files needed for
// a self contained plugin, including dynamic libraries and data files.
std::string GRT::module_path_in_bundle(const std::string &path)
{
  if (!g_str_has_suffix(path.c_str(), ".mwbplugin") || !g_file_test(path.c_str(), G_FILE_TEST_IS_DIR))
    return "";
#ifdef __APPLE__
  if (!g_file_test((path+"/Contents/Info.plist").c_str(), G_FILE_TEST_IS_REGULAR)) // not a bundle
    return "";
  
  std::string module_file;

  FILE *file= fopen((path+"/Contents/Info.plist").c_str(), "r");
  if (file)
  {
    char line[256];
    
    while (fgets(line, sizeof(line), file))
    {
      // find out the db.mysql.foobar.grt.dylib name
      if (strstr(line, "<key>GRTModuleFileName</key>"))
      {
        if (fgets(line, sizeof(line), file))
        {
          char *name= strstr(line, "<string>");
          if (name)
          {
            char *end;
            name+= strlen("<string>");
            end= strrchr(name, '<');
            if (end)
            {
              *end= 0;
              module_file= std::string(path).append("/Contents/Frameworks/").append(name);
            }
          }
        }
        break;
      }
    }
  
    fclose(file);
  }
  
  if (!module_file.empty())
    return module_file;
#elif defined(_WIN32)
  
#else
  
#endif
  
  return "";
}



//--------------------------------------------------------------------------------
// Module Loader



//--------------------------------------------------------------------------------
// Modules


Module::Module(ModuleLoader *loader)
  : _loader(loader), _is_bundle(false)
{
}


ValueRef Module::call_function(const std::string &name, const grt::BaseListRef &args)
{
  const Function *f= get_function(name);

  if (!f)
    throw grt::module_error(std::string("Module ").append(_name).append(" doesn't have function ").append(name));

  return f->call(args);
}




void Module::add_function(const Function &func)
{
  _functions.push_back(func);
}



const Module::Function *Module::get_function(const std::string &name) const
{
  for (std::vector<Function>::const_iterator iter= _functions.begin();
       iter != _functions.end(); ++iter)
  {
    if (iter->name == name)
      return &*iter;
  }

  if (!_extends.empty())
  {
    Module *module= get_grt()->get_module(_extends);
    if (!module)
      throw std::runtime_error(strfmt("Parent module '%s' of module '%s' was not found", _extends.c_str(), _name.c_str()));
    
    return module->get_function(name);
  }
  return 0;
}


void Module::validate() const
{
  if (name().empty()) throw std::runtime_error("Invalid module");

  // validate if registered functions conform to interfaces
  for (Interfaces::const_iterator iter= _interfaces.begin();
       iter != _interfaces.end(); ++iter)
  {
    const Interface *iface= get_grt()->get_interface(*iter);
    if (iface)
    {
      if (!iface->check_conformance(this))
        throw std::logic_error(std::string("Module ").append(name()).append(" does not conform to interface ").append(*iter));
    }
    else
      g_warning("Interface '%s' implemented by module '%s' is not registered",
                iter->c_str(), name().c_str());
  }
}




