#ifndef _GRTPP_MODULE_LUA_H_
#define _GRTPP_MODULE_LUA_H_

#include "grtpp.h"

#include "lua_context.h"

namespace grt {
  
  class LuaModuleLoader;
  
  class MYSQLGRT_PUBLIC LuaModule : public Module
  {
    friend class LuaModuleLoader;
  public:
    LuaModule(LuaModuleLoader *loader);
    virtual ~LuaModule();

    /** Parse a Lua module function specification.
     * 
     * Function parameter list syntax:
     * @code
     * param_list::= param[, param_list]
     * param::= type [ label]
     * label::= a name for the parameter
     * type::= i | r | s | l[<content_spec>] | d[<content_spec>] | o[@struct]
     * content_spec::= i | r | s | o@struct
     * struct::= name of a valid struct
     * @endcode
     *
     * Ex.:
     *  doSomething:s:i count,l<i> poslist,o<db.mysql.Table> table,d args
     */
    virtual bool add_parse_function_spec(const std::string &spec);
  protected:

  };


  class MYSQLGRT_PUBLIC LuaModuleLoader : public ModuleLoader
  {
  public:
    LuaModuleLoader(GRT *grt);
    virtual ~LuaModuleLoader();
    
    virtual std::string get_loader_name() { return "lua"; }
    
    virtual Module *init_module(const std::string &path);

    virtual void refresh();
    
    virtual bool load_library(const std::string &file);

    LuaContext *get_lua_context() { return &_lua; }
    
    virtual bool check_file_extension(const std::string &path);
    
  protected:
    friend class LuaModule;

    LuaContext _lua;


    ValueRef call_function(const BaseListRef &args, LuaModule *module, const Module::Function &function);
  };

  
};

#endif
