#ifndef _GRTPP_SHELL_H_
#define _GRTPP_SHELL_H_

#include <grtpp.h>

namespace grt {
  
  typedef enum {
    ShellCommandUnknown = -1,
      ShellCommandExit = 0,
      ShellCommandAll,
      ShellCommandError,
      ShellCommandStatement,
      ShellCommandHelp,
      ShellCommandLs,
      ShellCommandCd,
      ShellCommandRun
  } ShellCommand;


#define MYX_SHELL_CURNODE "current"
  
  class MYSQLGRT_PUBLIC Shell
  {
  public:
    Shell(GRT *grt);
    virtual ~Shell();

    bool set_disable_quit(bool flag);

    ShellCommand execute(const std::string &linebuf);

    virtual void init()= 0;
    virtual void print_welcome()= 0;
    virtual std::string get_prompt()= 0;
    virtual int execute_script(const std::string &linebuf)= 0;
    virtual int run_file(const std::string &file_name, bool interactive)= 0;
    virtual void show_help(const std::string &topic)= 0;

    // for GCompletion
    virtual GList *get_tokens()= 0;

    virtual ValueRef get_global_var(const std::string &var_name)= 0;
    virtual int set_global_var(const std::string &var_name, const ValueRef &value)= 0;


    static std::string get_abspath(const std::string &cwd, const std::string &npath);


    virtual void print(const std::string &str);

  protected:
    GRT *_grt;
    
    bool _disable_quit;
  };

};

#endif
