#include <stdio.h>
#include <sstream>
#include "myx_sql_parser_public_interface.h"
#include "myx_lex_helpers.h"
#include "myx_sql_parser.tab.hh"
#include "myx_sql_tree_item.h"

//#include "sql_lex.h"

std::istream* lex_input_stream= 0;
std::string err_msg;

void yyerror(const char *msg) { ::err_msg= msg; printf("Parse error: %s\n", msg); }

MYX_PUBLIC_FUNC const std::string & myx_get_err_msg()
{
  return err_msg;
}

MYX_PUBLIC_FUNC void *myx_get_parser_tree()
{
  return tree;
}

MYX_PUBLIC_FUNC void myx_set_parser_input(std::istream *sqlstream)
{
  lex_input_stream = sqlstream;
}

MYX_PUBLIC_FUNC void myx_set_parser_source(const char *sql)
{
  lex_input_stream = new std::istringstream(sql);
}

MYX_PUBLIC_FUNC void myx_set_parser_source(std::istream *sqlstream)
{
  lex_input_stream = sqlstream;
}

MYX_PUBLIC_FUNC void myx_free_parser_source(void)
{
  delete lex_input_stream;
  MyxSQLTreeItem::make_items_cleanup();
}

MYX_PUBLIC_FUNC void myx_parse(void)
{
  err_msg.clear();
  yyparse();
}

extern int MYSQLlex(void **arg, void *yyl);

extern "C" {

struct Lex_args lex_args;

int yylex(void **yylval) 
{
  //struct Lex_args *p= (struct Lex_args *)ptr_to_arg_pair;
  //return MYSQLlex(lex_args.arg1, lex_args.arg2); 
  
  int state= MYSQLlex(yylval, lex_args.arg2);
  //int state= myx_map_lexer_value(MYSQLlex(yylval, lex_args.arg2)); 
  //return state == END_OF_INPUT ? 0 : state;
  return state;
}


int yywrap() { return 1; }  // strop after EOF

void yy_custom_input(char *buf, int* result, int max_size) 
{
  lex_input_stream->read(buf, max_size);
  *result= lex_input_stream->gcount();
}

int yy_token_match(int token, const char *value)
{
  //printf("lexer token %s\n", value);
  return token;
}

int yy_unknown_token(const char *value)
{
  //printf("error %s", value);
  return 0;
}

// server replacement routines
//

char *strmake_root(const char *str, unsigned int len)
{
  char *pos;
  if ((pos=(char *)malloc(len+1)))
  {
    memcpy(pos,str,len);
    pos[len]=0;
  }
  return pos;
}

char *strdup_root(const char *str)
{
  return strmake_root(str, (unsigned int) strlen(str));
} 

char *alloc_root(unsigned int size)
{
  return (char *)malloc(size);
}

char *memdup_root(const char *str, unsigned int len)
{
  char *pos;
  if ((pos=alloc_root(len)))
    memcpy(pos,str,len);
  return pos;
}

} // extern C
