#include "stdafx.h"

#include "mysql_sql_facade.h"
#include "mysql_sql_parser.h"
#include "mysql_sql_normalizer.h"
#include "mysql_sql_schema_rename.h"
#include "mysql_sql_syntax_check.h"
#include "mysql_invalid_sql_parser.h"
#include "mysql_sql_script_splitter.h"


using namespace bec;


int MysqlSqlFacade::splitSqlScript(const std::string &sql_script, std::list<std::string> &statements)
{
  return Mysql_sql_script_splitter().process(sql_script, statements);
}


std::auto_ptr<Sql_parser> MysqlSqlFacade::sqlParser()
{
  return std::auto_ptr<Sql_parser>(new Mysql_sql_parser());
}


int MysqlSqlFacade::parseSqlScriptString(db_CatalogRef catalog, const std::string sql_script)
{
  return parseSqlScriptStringEx(catalog, sql_script, DictRef());
}


int MysqlSqlFacade::parseSqlScriptStringEx(db_CatalogRef catalog, const std::string sql_script, const grt::DictRef options)
{
  return Mysql_sql_parser().parse_sql_script(
    db_mysql_CatalogRef::cast_from(catalog), sql_script.c_str(), options);
}


int MysqlSqlFacade::parseSqlScriptFile(db_CatalogRef catalog, const std::string sql_script_filename)
{
  return parseSqlScriptFileEx(catalog, sql_script_filename, DictRef());
}


int MysqlSqlFacade::parseSqlScriptFileEx(db_CatalogRef catalog, const std::string sql_script_filename, const grt::DictRef options)
{
  return Mysql_sql_parser().parse_sql_script_file(
    db_mysql_CatalogRef::cast_from(catalog), sql_script_filename.c_str(), options);
}


std::auto_ptr<Invalid_sql_parser> MysqlSqlFacade::invalidSqlParser()
{
  return std::auto_ptr<Invalid_sql_parser>(new Mysql_invalid_sql_parser());
}


int MysqlSqlFacade::parseInserts(db_TableRef table, const std::string sql_script)
{
  return Mysql_invalid_sql_parser().parse_inserts(db_mysql_TableRef::cast_from(table), sql_script.c_str());
}


int MysqlSqlFacade::parseTriggers(db_TableRef table, const std::string sql_script)
{
  return Mysql_invalid_sql_parser().parse_triggers(db_mysql_TableRef::cast_from(table), sql_script.c_str());
}


int MysqlSqlFacade::parseRoutine(db_RoutineRef routine, const std::string sql_script)
{
  return Mysql_invalid_sql_parser().parse_routine(db_mysql_RoutineRef::cast_from(routine), sql_script.c_str());
}


int MysqlSqlFacade::parseRoutines(db_RoutineGroupRef routineGroup, const std::string sql_script)
{
  return Mysql_invalid_sql_parser().parse_routines(db_mysql_RoutineGroupRef::cast_from(routineGroup), sql_script.c_str());
}


int MysqlSqlFacade::parseView(db_ViewRef view, const std::string sql_script)
{
  return Mysql_invalid_sql_parser().parse_view(db_mysql_ViewRef::cast_from(view), sql_script.c_str());
}


int MysqlSqlFacade::checkTriggerSyntax(const std::string sql_script_string)
{
  return Mysql_sql_syntax_check().check_trigger_syntax(sql_script_string.c_str());
}


int MysqlSqlFacade::checkViewSyntax(const std::string sql_script_string)
{
  return Mysql_sql_syntax_check().check_view_syntax(sql_script_string.c_str());
}


int MysqlSqlFacade::checkRoutineSyntax(const std::string sql_script_string)
{
  return Mysql_sql_syntax_check().check_routine_syntax(sql_script_string.c_str());
}


std::string MysqlSqlFacade::normalizeSqlStatement(const std::string sql, const std::string schema_name)
{
  return Mysql_sql_normalizer().normalize(sql, schema_name);
}


int MysqlSqlFacade::renameSchemaReferences(db_CatalogRef catalog, const std::string old_schema_name, const std::string new_schema_name)
{
  return Mysql_sql_schema_rename().rename_schema_references(
    db_mysql_CatalogRef::cast_from(catalog), old_schema_name, new_schema_name);
}
