#ifndef _MYSQL_SQL_FACADE_H_
#define _MYSQL_SQL_FACADE_H_


#include "mysql_sql_parser_public_interface.h"
#include "grtsqlparser/sql_facade.h"
#include "mysql_sql_parser_base.h"
#include "grts/structs.db.mysql.h"
#include "grtpp_module_cpp.h"


#define MysqlSqlFacade_VERSION "1.0"
 

/** Implements DBMS specifics.
 * 
 * @ingroup sqlparser
 */
class MYSQL_SQL_PARSER_PUBLIC_FUNC MysqlSqlFacade
  : public SqlFacade, public grt::ModuleImplBase
{
public:
  MysqlSqlFacade(grt::CPPModuleLoader *ldr);

  DEFINE_INIT_MODULE(MysqlSqlFacade_VERSION, "MySQL AB", grt::ModuleImplBase,
    DECLARE_MODULE_FUNCTION(MysqlSqlFacade::getPluginInfo),
    NULL);
  virtual grt::ListRef<app_Plugin> getPluginInfo();

  int splitSqlScript(const std::string &sql_script, std::list<std::string> &statements);

  std::auto_ptr<Sql_parser> sqlParser();
  int parseSqlScriptString(db_CatalogRef catalog, const std::string sql_script);
  int parseSqlScriptStringEx(db_CatalogRef catalog, const std::string sql_script, const grt::DictRef options);
  int parseSqlScriptFile(db_CatalogRef catalog, const std::string sql_script_filename);
  int parseSqlScriptFileEx(db_CatalogRef catalog, const std::string sql_script_filename, const grt::DictRef options);

  std::auto_ptr<Invalid_sql_parser> invalidSqlParser();
  int parseInserts(db_TableRef table, const std::string sql_script);
  int parseTriggers(db_TableRef table, const std::string sql_script);
  int parseRoutine(db_RoutineRef routine, const std::string sql_script);
  int parseRoutines(db_RoutineGroupRef routineGroup, const std::string sql_script);
  int parseView(db_ViewRef view, const std::string sql_script);

  int checkTriggerSyntax(const std::string sql_script_string);
  int checkViewSyntax(const std::string sql_script_string);
  int checkRoutineSyntax(const std::string sql_script_string);

  std::string normalizeSqlStatement(const std::string sql_script, const std::string schema_name);

  int renameSchemaReferences(db_CatalogRef catalog, const std::string old_schema_name, const std::string new_schema_name);
};


#endif // _MYSQL_SQL_FACADE_H_
