#ifndef _MYSQL_SQL_PARSER_BASE_H_
#define _MYSQL_SQL_PARSER_BASE_H_


#include "mysql_sql_parser_public_interface.h"
#include "grtsqlparser/sql_parser_base.h"
#include "grts/structs.db.mysql.h"


class Mysql_sql_parser_fe;
class MyxStatementParser;
class MyxSQLTreeItem;


/** Provides DBMS specific functionality for processing of SQL statements/scripts.
 * 
 * @ingroup sqlparser
 */
class Mysql_sql_parser_base : virtual public Sql_parser_base
{
private:
  static int process_sql_statement(void *sql_parser_ptr, const MyxStatementParser *splitter, const char *sql_statement, const MyxSQLTreeItem *tree, int lineno, const std::string &err_msg); //callback
  friend int process_sql_statement(void *, const MyxStatementParser *, const char *, const MyxSQLTreeItem *, int, const std::string &);

protected:
  Mysql_sql_parser_base();
  virtual ~Mysql_sql_parser_base() {}

  // misc
  virtual int total_line_count();

  // aux types
  typedef sigc::slot<int, const MyxSQLTreeItem *> Process_sql_statement;

  // basic functionality
  int parse_sql_script(Mysql_sql_parser_fe &sql_parser_fe, const char *sql_script);
  int parse_sql_script_file(Mysql_sql_parser_fe &sql_parser_fe, const char *sql_script_filename);

  // data members
  const MyxStatementParser *_splitter;
  Process_sql_statement _process_sql_statement;
  db_mysql_CatalogRef _catalog;
  db_mysql_SchemaRef _active_schema;
  int _lineno;
  std::string _err_msg;

  class Null_state_keeper : public Sql_parser_base::Null_state_keeper
  {
  public:
    Null_state_keeper(Mysql_sql_parser_base *sql_parser)
      : Sql_parser_base::Null_state_keeper(sql_parser), _sql_parser(sql_parser) {}
    virtual ~Null_state_keeper();
  private:
    Mysql_sql_parser_base *_sql_parser;
  };
  friend class Null_state_keeper;
};


#endif // _MYSQL_SQL_PARSER_BASE_H_
