#ifndef _MYSQL_SQL_SCHEMA_RENAME_H_
#define _MYSQL_SQL_SCHEMA_RENAME_H_


#include "mysql_sql_parser_base.h"


/** Searches catalog objects' statements for occurences of old schema name and replaces them with new one.
 * 
 * @ingroup sqlparser
 */
class Mysql_sql_schema_rename : protected Mysql_sql_parser_base
{
public:
  Mysql_sql_schema_rename();

  int rename_schema_references(db_mysql_CatalogRef catalog, const std::string &old_schema_name, const std::string &new_schema_name);

protected:
  template<typename T>
  void rename_schema_references(
    grt::ListRef<T> obj_list,
    grt::StringRef (T::*sql_text_prop_r)() const,
    void (T::*sql_text_prop_w)(const grt::StringRef&),
    int delim_wraping,
    Mysql_sql_parser_fe &parser_fe);
  
  int process_sql_statement(const MyxSQLTreeItem *tree);
  void process_sql_statement_item(const MyxSQLTreeItem *item);
  void process_schema_reference_candidate(const MyxSQLTreeItem *item, int dot_count);
  bool rename_schema_references(std::string &sql_text);

  std::string _old_schema_name;
  std::string _new_schema_name;
  std::list<int> _schema_names_offsets;
  bool _case_sensitive_names;

  class Null_state_keeper : Mysql_sql_parser_base::Null_state_keeper
  {
  public:
    Null_state_keeper(Mysql_sql_schema_rename *sql_parser)
      : Mysql_sql_parser_base::Null_state_keeper(sql_parser), _sql_parser(sql_parser) {}
    ~Null_state_keeper();
  private:
    Mysql_sql_schema_rename *_sql_parser;
  };
  friend class Null_state_keeper;
};


#endif // _MYSQL_SQL_SCHEMA_RENAME_H_
