
#include "stdafx.h"

#include <glib/gstdio.h>

#include "myx_util_functions.h"
#include "wb_model.h"
#include "reporting.h"

#include "grti/sqlgenerator.h"
#include "grts/structs.workbench.h"

#include "../res/reporting_includes/overview.html.tpl.varnames.h"
#include "../res/reporting_includes/table_details.html.tpl.varnames.h"
#include "../res/reporting_includes/full_list.html.tpl.varnames.h"
#include "../res/reporting_includes/overview_list.html.tpl.varnames.h"
#include "../res/reporting_includes/details_list.html.tpl.varnames.h"

using namespace std;

using google::Template;
using google::TemplateDictionary;

/**
 * Initializes the template engine with our templates. They are registered so we can get error reports for them
 * if something is wrong and also can reload them if they have been changed while the application is running.
 */
void WbModelImpl::initializeReporting()
{
  // Enumerate reporting folder for all stored report types.
}

/*
 * @brief returns the list of available template file directories
 *
 * @param model - the workbench_physical_ModelRef to process
 * @param templates - a GRT List the available templates will be added to
 * @return 1 on success, 0 on error
 */
int WbModelImpl::getAvailableReportingTemplates(grt::StringListRef templates)
{
  // get pointer to the GRT
  grt::GRT *grt= get_grt();
  string basedir= bec::GRTManager::get_instance_for(grt)->get_basedir();
  string template_base_dir= bec::make_path(basedir, "modules/data/wb_model_reporting");
  GDir *dir;
  const char *entry;

  dir= g_dir_open(template_base_dir.c_str(), 0, NULL);
  if (dir)
  {
    while ((entry= g_dir_read_name(dir)) != NULL)
    {
      char *path= g_build_filename(template_base_dir.c_str(), entry, NULL);

      if (g_file_test(path, (GFileTest)(G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)) && 
        g_str_has_suffix(entry, ".tpl"))
      {
        // reformat the template name, replace _ with spaces
        char *temp= g_strdup(entry);
        char *ptr= temp; 
        while ((ptr= strchr(ptr, '_'))) 
          *ptr= ' ';

        // remove .tpl suffix
        ptr= strrchr(temp, '.'); *ptr= 0;

        templates.insert(temp);

        g_free(temp);
      }

      g_free(path);
    }

    g_dir_close(dir);
  }

  return 1;
}

/*
 * @brief returns the template info for the given template
 *
 * @param model - the workbench_physical_ModelRef to process
 * @param template_name - the name of the template
 * @return the template info object
 */
workbench_model_reporting_TemplateInfoRef WbModelImpl::getReportingTemplateInfo(const std::string& template_name)
{
  string template_dir= getTemplateDirFromName(template_name);

  string template_info_path= bec::make_path(template_dir, "info.xml");
  if (g_file_test(template_info_path.c_str(), (GFileTest)(G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR)))
    return workbench_model_reporting_TemplateInfoRef::cast_from(get_grt()->unserialize(template_info_path));
  else
    return workbench_model_reporting_TemplateInfoRef();
}



workbench_model_reporting_TemplateStyleInfoRef WbModelImpl::get_template_style_from_name(grt::GRT *grt, 
                                                                         std::string template_name, std::string template_style_name)
{
  if (template_style_name == "")
    return workbench_model_reporting_TemplateStyleInfoRef();

  string template_dir= getTemplateDirFromName(template_name);

  string template_info_path= bec::make_path(template_dir, "info.xml");
  if (g_file_test(template_info_path.c_str(), (GFileTest)(G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR)))
  {
    workbench_model_reporting_TemplateInfoRef info= 
      workbench_model_reporting_TemplateInfoRef::cast_from(grt->unserialize(template_info_path));

    for( size_t i= 0; i < info->styles().count(); i++)
    {
      workbench_model_reporting_TemplateStyleInfoRef styleInfo=
        info->styles().get(i);

      if (template_style_name == (std::string)styleInfo->name())
        return styleInfo;
    }
  }

  return workbench_model_reporting_TemplateStyleInfoRef();
}

void assign_dict_field_if_exist(bool& var, const char* field, const grt::DictRef& dict)
{
  if (dict.has_key(field))
    var= dict.get_int(field) != 0;
}

void assign_dict_field_if_exist(std::string& var, const char* field, const grt::DictRef& dict)
{
  if (dict.has_key(field))
    var= dict.get_string(field);
}

/**
 * Fills the given column dictionary with data from the given column definition.
 */
void fillColumnDict(const db_mysql_ColumnRef& col, const db_mysql_TableRef& table, TemplateDictionary *col_dict, bool detailed)
{
  if (*table->isPrimaryKeyColumn(col))
  {
    if (*table->isForeignKeyColumn(col))
      col_dict->SetValue(ktd_COLUMN_KEY, "PFK");
    else
      col_dict->SetValue(ktd_COLUMN_KEY, "PK");
  }

  col_dict->SetValue(ktd_COLUMN_NAME, col->name().c_str());

  col_dict->SetValue(ktd_COLUMN_NOTNULL, (col->isNotNull() == 1) ? "Yes" : "No");
  col_dict->SetValue(ktd_COLUMN_DEFAULTVALUE, (col->defaultValueIsNull() == 1) ? "NULL" : 
    col->defaultValue().c_str());
  col_dict->SetValue(ktd_COLUMN_COMMENT, col->comment().c_str());
  col_dict->SetValue(ktd_COLUMN_DATATYPE, bec::ColumnHelper::format_column_type(col));

  if (detailed)
  {
    col_dict->SetValue(ktd_TABLE_NAME, *table->name());
    
    std::string key_part= "";
    if (table->isPrimaryKeyColumn(col))
      key_part += "Primary key, ";
    if (table->isForeignKeyColumn(col))
      key_part += "Foreign key, ";

    col_dict->SetValue(kfl_COLUMN_KEY_PART, key_part.substr(0, key_part.size() - 2));
    col_dict->SetValue(kfl_COLUMN_NULLABLE, (col->isNotNull() == 1) ? "No" : "Yes");
    col_dict->SetValue(kfl_COLUMN_AUTO_INC, (col->autoIncrement() == 1) ? "Yes" : "No");
    if (!col->characterSetName().empty())
      col_dict->SetValue(kfl_COLUMN_CHARSET, col->characterSetName().c_str());
    else
      col_dict->SetValue(kfl_COLUMN_CHARSET, "Schema Default");
    
    if (!col->collationName().empty())
      col_dict->SetValue(kfl_COLUMN_COLLATION, col->collationName().c_str());
    else
      col_dict->SetValue(kfl_COLUMN_COLLATION, "Schema Default");

    if (col->userType().is_valid())
      col_dict->SetValue(kfl_COLUMN_IS_USERTYPE, "Yes");
    else 
      col_dict->SetValue(kfl_COLUMN_IS_USERTYPE, "No");
  }
}

void fillIndexDict(const db_mysql_IndexRef& idx, const db_mysql_TableRef& table, TemplateDictionary *idx_dict, bool detailed)
{
  idx_dict->SetValue(ktd_INDEX_NAME, idx->name().c_str());

  idx_dict->SetValue(ktd_INDEX_PRIMARY, (idx->isPrimary() == 1) ? "Yes" : "No");
  idx_dict->SetValue(ktd_INDEX_UNIQUE, (idx->unique() == 1) ? "Yes" : "No");
  idx_dict->SetValue(ktd_INDEX_TYPE, idx->indexType().c_str());
  idx_dict->SetValue(ktd_INDEX_KIND, idx->indexKind().c_str());
  idx_dict->SetValue(ktd_INDEX_COMMENT, idx->comment().c_str());

  for (size_t l= 0; l < idx->columns().count(); l++)
  {
    db_mysql_IndexColumnRef idx_col= idx->columns().get(l);

    TemplateDictionary *idx_col_dict= idx_dict->AddSectionDictionary(kfl_INDEX_COLUMNS);
    idx_col_dict->SetValue(ktd_INDEX_COLUMN_NAME, idx_col->referencedColumn()->name().c_str());
    idx_col_dict->SetValue(ktd_INDEX_COLUMN_ORDER, (idx_col->descend() == 1) ? "Descending" : "Ascending");
    if (idx_col->comment().empty())
      idx_col_dict->SetValue(ktd_INDEX_COLUMN_COMMENT, "no comment");
    else
      idx_col_dict->SetValue(ktd_INDEX_COLUMN_COMMENT, idx_col->comment().c_str());
  }

  if (detailed)
  {
    idx_dict->SetValue(ktd_TABLE_NAME, *table->name());
    idx_dict->SetIntValue(kfl_INDEX_KEY_BLOCK_SIZE, idx->keyBlockSize());
  }
}

void fillForeignKeyDict(const db_mysql_ForeignKeyRef& fk, const db_mysql_TableRef& table, TemplateDictionary *fk_dict, bool detailed)
{
  fk_dict->SetValue(ktd_REL_NAME, fk->name().c_str());
  fk_dict->SetValue(ktd_REL_TYPE, bec::TableHelper::is_identifying_foreign_key(table, fk) ? "Identifying" : "Non-Identifying");
  if (fk->referencedTable().is_valid())
    fk_dict->SetValue(ktd_REL_PARENTTABLE, fk->referencedTable()->name().c_str());
  fk_dict->SetValue(ktd_REL_CHILDTABLE, table->name().c_str());
  fk_dict->SetValue(ktd_REL_CARD, (fk->many() == 1) ? "1:n" : "1:1");

  if (detailed)
  {
    fk_dict->SetValue(ktd_TABLE_NAME, table->name().c_str());
    fk_dict->SetValue(kfl_FK_DELETE_RULE, fk->deleteRule().c_str());
    fk_dict->SetValue(kfl_FK_UPDATE_RULE, fk->updateRule().c_str());
    fk_dict->SetValue(kfl_FK_MANDATORY, fk->mandatory() ? "Yes" : "No");
  }
}

/**
 * Copies all files non-recursively from source to target. Target will be created on the fly.
 */
void copy_folder(const std::string source_folder, const std::string target_folder)
{
  // Create target folder.
  g_mkdir(target_folder.c_str(), 0600);

  const char *entry;
  GDir *dir= g_dir_open(source_folder.c_str(), 0, NULL);
  if (dir)
  {
    while ((entry= g_dir_read_name(dir)) != NULL)
    {
      char* source= g_build_filename(source_folder.c_str(), entry, NULL);
      char* target= g_build_filename(target_folder.c_str(), entry, NULL);
      copy_file(source, target);
    }
  }
}

void set_ddl(TemplateDictionary *target, SQLGeneratorInterfaceWrapper* sqlgenModule, const GrtNamedObjectRef& object, 
             bool ddl_enabled)
{
  if (ddl_enabled && sqlgenModule != NULL)
  {
    std::string sql= sqlgenModule->makeCreateScriptForObject(object);
    std::string fixed_line_breaks= bec::replace_string(sql, "\n", "<br />");

    // The DDL script is wrapped in an own section dir to allow switching it off entirely (including
    // the surrounding HTML code).
    target->SetValueAndShowSection(ktd_DDL_SCRIPT, fixed_line_breaks, ktd_DDL_LISTING);
  }
}

void set_table_comment(TemplateDictionary *target, const std::string comment)
{
  target->SetValueAndShowSection(ktd_TABLE_COMMENT, comment, ktd_TABLE_COMMENT_LISTING);
}

/*
 * @brief Generates a schema report for the model passed in workbench_physical_Model.
 *
 * @param model - the workbench_physical_ModelRef to process
 * @param options - various options that customize the output, including output template
 * @param dirname - directory where to output generated files
 * @return 1 on success, 0 on error
 */
int WbModelImpl::generateReport(workbench_physical_ModelRef model, const grt::DictRef& options, 
                                const std::string& dirname)
{
  // get pointer to the GRT
  grt::GRT *grt= model.get_grt();
  string basedir= bec::GRTManager::get_instance_for(grt)->get_basedir();
  string template_base_dir= bec::make_path(basedir, "modules/data/wb_model_reporting");

  db_mysql_CatalogRef catalog= db_mysql_CatalogRef::cast_from(model->catalog());

  // helper variables
  map<std::string, vector<db_mysql_ForeignKeyRef> > tbl_fk_map;

  // Process options
  string template_name= "HTML Basic Frames";
  string template_style_name= "";
  string title= "MySQL Schema Report";
  string output_dirname= "./Report";
  bool columns_show= true;
  bool indices_show= true;
  bool fks_show= true;
  bool fks_show_referred_fks= true;
  bool show_ddl= true;

  if (options.is_valid())
  {
    assign_dict_field_if_exist(template_name, "template_name", options);
    assign_dict_field_if_exist(template_style_name, "template_style_name", options);
    assign_dict_field_if_exist(title, "title", options);
    assign_dict_field_if_exist(output_dirname, "output_dirname", options);
    assign_dict_field_if_exist(columns_show, "columns_show", options);
    assign_dict_field_if_exist(indices_show, "indices_show", options);
    assign_dict_field_if_exist(fks_show, "fks_show", options);
    assign_dict_field_if_exist(fks_show_referred_fks, "fks_show_referred_fks", options);
    assign_dict_field_if_exist(show_ddl, "show_ddl", options);
  }

  // ensure the output dir exists
  g_mkdir_with_parents(dirname.c_str(), 0600);

  // Start report generation
  grt->send_info("Generating schema report...");

  // --------------------------------------------------------------------------------------------

  // Build FK dictionary if required
  if (fks_show && fks_show_referred_fks)
  {
    // build schema_dict by loop over all schemata, add it to the main_dict
    for (size_t i= 0; i < catalog->schemata().count(); i++)
    {
      db_mysql_SchemaRef schema= catalog->schemata().get(i);

      // loop over all tables
      for (size_t j= 0; j < schema->tables().count(); j++)
      {
        db_mysql_TableRef table= schema->tables().get(j);

        // loop over all foreign keys
        for (size_t k= 0; k < table->foreignKeys().count(); k++)
        {
          db_mysql_ForeignKeyRef fk= table->foreignKeys().get(k);
          db_mysql_TableRef ref_tbl= fk->referencedTable();

          if (!ref_tbl.is_valid())
            continue;

          // look for table in tbl_fk_map
          map<std::string, vector<db_mysql_ForeignKeyRef> >::iterator tbl_fk_map_it=
            tbl_fk_map.find(ref_tbl.id());
          if (tbl_fk_map_it != tbl_fk_map.end())
          {
            // if found, add fk reference to table
            tbl_fk_map_it->second.push_back(fk);
          }
          else
          {
            // if table is not found, create entry
            vector<db_mysql_ForeignKeyRef> new_tbl_fk_map_v;

            // add fk reference to table
            new_tbl_fk_map_v.push_back(fk);

            tbl_fk_map.insert(make_pair(ref_tbl.id(), new_tbl_fk_map_v));
          }
        }
      }
    }
  }

  // --------------------------------------------------------------------------------------------
  // create main dictionary that will be used to expand the templates
  TemplateDictionary main_dict("basic index");

  // Set some global project info.
  main_dict.SetValue(ktd_TITLE, title);

  std::string time= bec::fmttime(0, DATETIME_FMT);
  main_dict.SetValue(ko_GENERATED, time);

  workbench_DocumentRef document= workbench_DocumentRef::cast_from(model->owner());
  main_dict.SetValue(ko_PROJECT_NAME, document->info()->project().c_str());
  main_dict.SetValue(ko_PROJECT_AUTHOR, document->info()->author().c_str());
  main_dict.SetValue(ko_PROJECT_TITLE, document->info()->caption().c_str());
  main_dict.SetValue(ko_PROJECT_CHANGED, document->info()->dateChanged().c_str());
  main_dict.SetValue(ko_PROJECT_CREATED, document->info()->dateCreated().c_str());
  main_dict.SetValue(ko_PROJECT_DESCRIPTION, document->info()->description().c_str());
  main_dict.SetValue(ko_PROJECT_VERSION, document->info()->version().c_str());

  workbench_model_reporting_TemplateStyleInfoRef styleInfo= get_template_style_from_name(grt, template_name, template_style_name);
  if (styleInfo.is_valid())
    main_dict.SetValue(ktd_STYLE_NAME, (std::string)styleInfo->styleTagValue());

  main_dict.SetIntValue(ktd_SCHEMA_COUNT, (int)catalog->schemata().count());

  int total_column_count= 0;
  int total_index_count= 0;
  int total_fk_count= 0;
  int total_table_count= 0;

  std::vector<SQLGeneratorInterfaceWrapper*> genmodules= grt->get_implementing_modules<SQLGeneratorInterfaceWrapper>();
  SQLGeneratorInterfaceWrapper *sqlgenModule= 0;

  for (std::vector<SQLGeneratorInterfaceWrapper*>::const_iterator iter= genmodules.begin();
       iter != genmodules.end(); ++iter)
  {
    if ((*iter)->getTargetDBMSName() == "mysql")
    {
      sqlgenModule= *iter;
      break;
    }
  }
  if (!sqlgenModule)
    throw std::logic_error("could not find SQL generation module for mysql");

  // Build schema_dict by looping over all schemata, add it to the main_dict.
  for (int i= 0; i < (int)catalog->schemata().count(); i++)
  {
    db_mysql_SchemaRef schema= catalog->schemata().get(i);

    TemplateDictionary *schema_dict= main_dict.AddSectionDictionary(ktd_SCHEMATA);
    schema_dict->SetIntValue(ktd_SCHEMA_ID, i + 1);
    schema_dict->SetValue(ktd_SCHEMA_NAME, *schema->name());

    set_ddl(schema_dict, sqlgenModule, schema, show_ddl);

    total_table_count += (int)schema->tables().count();
    schema_dict->SetIntValue(ktd_TABLE_COUNT, (int)schema->tables().count());

    // Loop over all tables. Build the nested tables sub groups and at the same time the
    // full collection of all columns, indices and foreign keys.
    int schema_column_count= 0;
    int schema_index_count= 0;
    int schema_fk_count= 0;
    for (int j= 0; j < (int)schema->tables().count(); j++)
    {
      db_mysql_TableRef table= schema->tables().get(j);

      TemplateDictionary *table_dict= schema_dict->AddSectionDictionary(ktd_TABLES);
      table_dict->SetIntValue(ktd_TABLE_ID, j + 1);      
      table_dict->SetValue(ktd_TABLE_NAME, *table->name());
      set_table_comment(table_dict, table->comment().c_str());

      set_ddl(table_dict, sqlgenModule, table, show_ddl);

      if (columns_show)
      {
        // loop over all columns
        TemplateDictionary *columns_list_dict;
        for (int k= 0; k < (int)table->columns().count(); k++)
        {
          // Create the dict for the outer section (inluding header)
          if (k == 0)
            columns_list_dict= table_dict->AddSectionDictionary(ktd_COLUMNS_LISTING);

          db_mysql_ColumnRef col= table->columns().get(k);

          // Fill data for table details.
          TemplateDictionary *col_dict= columns_list_dict->AddSectionDictionary(ktd_COLUMNS);
          fillColumnDict(col, table, col_dict, false);

          // Fill data for full details.
          col_dict= schema_dict->AddSectionDictionary(ktd_COLUMNS);
          fillColumnDict(col, table, col_dict, true);

          col_dict->SetIntValue(kdl_COLUMN_ID, schema_column_count++);
        }
      }

      if (indices_show)
      {
        // loop over all indices
        TemplateDictionary *idx_list_dict;
        for (int k= 0; k < (int)table->indices().count(); k++)
        {
          // Create the dict for the outer section (inluding header)
          if (k == 0)
            idx_list_dict= table_dict->AddSectionDictionary(ktd_INDICES_LISTING);

          db_mysql_IndexRef idx= table->indices().get(k);

          TemplateDictionary *idx_dict= idx_list_dict->AddSectionDictionary(ktd_INDICES);
          fillIndexDict(idx, table, idx_dict, false);

          idx_dict= schema_dict->AddSectionDictionary(ktd_INDICES);
          fillIndexDict(idx, table, idx_dict, true);

          idx_dict->SetIntValue(kdl_INDEX_ID, schema_index_count++);
        }
      }

      if (fks_show)
      {
        // loop over all foreign keys
        TemplateDictionary *fk_list_dict= NULL;
        for (int k= 0; k < (int)table->foreignKeys().count(); k++)
        {
          // Create the dict for the outer section (inluding header)
          if (k == 0)
            fk_list_dict= table_dict->AddSectionDictionary(ktd_REL_LISTING);

          db_mysql_ForeignKeyRef fk= table->foreignKeys().get(k);

          TemplateDictionary *fk_dict= fk_list_dict->AddSectionDictionary(ktd_REL);
          fillForeignKeyDict(fk, table, fk_dict, false);

          fk_dict= schema_dict->AddSectionDictionary(kdl_FOREIGN_KEYS);
          fillForeignKeyDict(fk, table, fk_dict, true);

          fk_dict->SetIntValue(kdl_FOREIGN_KEY_ID, schema_fk_count++);
        }

        if (fks_show_referred_fks)
        {
          // look for table in tbl_fk_map
          map<std::string, vector<db_mysql_ForeignKeyRef> >::iterator tbl_fk_map_it=
            tbl_fk_map.find(table->id());
          if (tbl_fk_map_it != tbl_fk_map.end())
          {
            vector<db_mysql_ForeignKeyRef>::iterator fk_it= tbl_fk_map_it->second.begin();
            for (; fk_it != tbl_fk_map_it->second.end(); fk_it++)
            {
              if (fk_list_dict == NULL)
                fk_list_dict= table_dict->AddSectionDictionary(ktd_REL_LISTING);

              db_mysql_ForeignKeyRef fk = *fk_it;

              TemplateDictionary *fk_dict= fk_list_dict->AddSectionDictionary(ktd_REL);
              fk_dict->SetValue(ktd_REL_NAME, fk->name().c_str());
              fk_dict->SetValue(ktd_REL_TYPE, bec::TableHelper::is_identifying_foreign_key(table, fk) ? "Identifying" : "Non-Identifying");
              fk_dict->SetValue(ktd_REL_PARENTTABLE, table->name().c_str());
              fk_dict->SetValue(ktd_REL_CHILDTABLE, fk->owner()->name().c_str());
              fk_dict->SetValue(ktd_REL_CARD, (fk->many() == 1) ? "1:n" : "1:1");
            }
          }
        }
      }
      schema_dict->SetIntValue(kdl_COLUMN_COUNT, schema_column_count);
      schema_dict->SetIntValue(kdl_INDICES_COUNT, schema_index_count);
      schema_dict->SetIntValue(kdl_FOREIGN_KEYS_COUNT, schema_fk_count);
    }
    total_column_count += schema_column_count;
    total_index_count += schema_index_count;
    total_fk_count += schema_fk_count;
  }

  main_dict.SetIntValue(ko_TOTAL_COLUMN_COUNT, total_column_count);
  main_dict.SetIntValue(ko_TOTAL_INDEX_COUNT, total_index_count);
  main_dict.SetIntValue(ko_TOTAL_FK_COUNT, total_fk_count);
  main_dict.SetIntValue(ko_TOTAL_TABLE_COUNT, total_table_count);

  // --------------------------------------------------------------------------------------------
  // Process template files

  string template_dir= getTemplateDirFromName(template_name);

  // loop over all files in the template dir
  const char *entry;
  GDir *dir= g_dir_open(template_dir.c_str(), 0, NULL);
  if (dir)
  {
    while ((entry= g_dir_read_name(dir)) != NULL)
    {
      char *path= g_build_filename(template_dir.c_str(), entry, NULL);

      // skip the info.xml file and preview pngs
      if (strcmp(entry, "info.xml") == 0 || 
        (g_str_has_prefix(entry, "preview_") && g_str_has_suffix(entry, ".png")))
        continue;

      if (g_file_test(path, (GFileTest)(G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR)))
      {
        if (g_str_has_suffix(entry, ".tpl"))
        {          
          // load template file
          Template *tpl_index= Template::GetTemplate(path, google::DO_NOT_STRIP);
          if (tpl_index == 0)
          {
            grt->send_error("Error while loading template files.");
            google::TemplateNamelist::SyntaxListType bad_list= google::TemplateNamelist::GetBadSyntaxList(true, 
              google::DO_NOT_STRIP);
            if (bad_list.size() > 0)
            {
              grt->send_error("Syntax errors found in file:");
              google::TemplateNamelist::SyntaxListType::iterator iterator= bad_list.begin();
              for (google::TemplateNamelist::SyntaxListType::iterator iterator= bad_list.begin(); 
                iterator != bad_list.end(); iterator++)
                  grt->send_error(*iterator);
            }
            grt->make_output_visible();
            return 0;
          }

          // expand the template based on the dictionary
          string* output= new string();
          tpl_index->Expand(output, &main_dict);

          // build output file name
          string template_filename(entry);      
          string output_filename= bec::make_path(output_dirname, template_filename.substr(0, template_filename.size() - 4));

          // write output to file
          ofstream fs;
          fs.open(output_filename.c_str(), ios::out);
          fs << output;
          fs.close();

          delete output;
        }
        else
        {
          // Copy files/folders.
          std::string target= bec::make_path(output_dirname, entry);
          if (g_file_test(path, G_FILE_TEST_IS_DIR))
            copy_folder(path, target);
          else
            copy_file(path, target.c_str());
        }
      }

      g_free(path);
    }

    g_dir_close(dir);
  }

  grt->send_info("Schema report generation completed.");

  return 1;
}

