#include "stdafx.h"

#include "grtpp_module_cpp.h"
#include "grtdb/db_object_helpers.h"
#include "grts/structs.db.h"

#include "grtpp.h"

#include "wb_module_validation.h"
#include <algorithm>
#include <boost/bind.hpp>

#include <iostream>

//-----------------------------------------------------------------------------
void GeneralValidator::walk_table(const db_TableRef& table) const
{
  dprint("walk_table\n");
  val::Chain<db_TableRef>::validate(table, val);
  columnNames_.clear();
  indexNames_.clear();
  triggerNames_.clear();
  
  //Order is important, check of columns should go before check of indices
  forEach(table->columns()    , this, &GeneralValidator::walk_column);
  forEach(table->indices()    , this, &GeneralValidator::walk_index);
  forEach(table->foreignKeys(), this, &GeneralValidator::walk_foreign_key);
  forEach(table->triggers()   , this, &GeneralValidator::walk_trigger);
}

//-----------------------------------------------------------------------------
void GeneralValidator::walk_view(const db_ViewRef& v) const
{
  dprint("walk_view\n");
  val::Chain<db_ViewRef>::validate(v, val);
}

//-----------------------------------------------------------------------------
void GeneralValidator::walk_routine(const db_RoutineRef& r) const
{
  dprint("walk_routine\n");
  val::Chain<db_RoutineRef>::validate(r, val);
}

//-----------------------------------------------------------------------------
void GeneralValidator::walk_user(const db_UserRef& user) const
{
  dprint("walk_user\n");
  val::Chain<db_UserRef>::validate(user, val);
}

//-----------------------------------------------------------------------------
void GeneralValidator::walk_column(const db_ColumnRef& column) const
{
  dprint("walk_column\n");
  val::Chain<db_ColumnRef>::validate(column, val);
}

//-----------------------------------------------------------------------------
void GeneralValidator::walk_index(const db_IndexRef& index) const
{
  dprint("walk_index\n");
  val::Chain<db_IndexRef>::validate(index, val);
}

//-----------------------------------------------------------------------------
void GeneralValidator::walk_foreign_key(const db_ForeignKeyRef& fk) const
{
  dprint("walk_foreign_key\n");
  val::Chain<db_ForeignKeyRef>::validate(fk, val);
}

//-----------------------------------------------------------------------------
void GeneralValidator::walk_trigger(const db_TriggerRef& trigger) const
{
  dprint("walk_trigger\n");
  val::Chain<db_TriggerRef>::validate(trigger, val);
}

//-----------------------------------------------------------------------------
void GeneralValidator::walk_role(const db_RoleRef& role) const
{
  dprint("walk_role\n");
  val::Chain<db_RoleRef>::validate(role, val);
}

//-----------------------------------------------------------------------------
void GeneralValidator::walk_routine_group(const db_RoutineGroupRef& rg) const
{
  dprint("walk_routine_group\n");
  val::Chain<db_RoutineGroupRef>::validate(rg, val);
}

//-----------------------------------------------------------------------------
void GeneralValidator::walk_schema(const db_SchemaRef& scm) const
{
  dprint("walk_schema\n");
  tableNames_.clear();
  viewNames_.clear();
  routineNames_.clear();
  routineGroupNames_.clear();
  
  forEach(scm->tables(), this, &GeneralValidator::walk_table);
  forEach(scm->views(), this, &GeneralValidator::walk_view);
  forEach(scm->routines(), this, &GeneralValidator::walk_routine);
  forEach(scm->routineGroups(), this, &GeneralValidator::walk_routine_group);
}

//-----------------------------------------------------------------------------
void GeneralValidator::validate(const db_CatalogRef& cat, val::ChainsSet *cs)
{
  totalItemsToCheck = 0.0;
  passedChecks      = 0;
  consistencyColumns.clear();
  cat_ = cat;
  
  val = cs;
  phys_model_ = workbench_physical_ModelRef::cast_from(cat->owner());

  roleNames_.clear();
  userNames_.clear();
  
  forEach(cat->schemata(), this, &GeneralValidator::walk_schema);
  forEach(cat->roles(), this, &GeneralValidator::walk_role);
  forEach(cat->users(), this, &GeneralValidator::walk_user);

  totalItemsToCheck = 0.0;
  passedChecks      = 0;
  consistencyColumns.clear();
  tableNames_.clear();
  viewNames_.clear();
  columnNames_.clear();
  indexNames_.clear();
  triggerNames_.clear();
  routineNames_.clear();
  routineGroupNames_.clear();
  roleNames_.clear();
  userNames_.clear();
}

//-----------------------------------------------------------------------------
void GeneralValidator::add_efficiency_validations(val::ChainsSet *cs)
{
  val::Chain<db_TableRef>::Ptr   ctable = val::Chain<db_TableRef>::chain(cs);
  val::Chain<db_ForeignKeyRef>::Ptr cfk = val::Chain<db_ForeignKeyRef>::chain(cs);
  
  ctable->add_check(val::atom(this, &GeneralValidator::efficiency_check_table));
  cfk->add_check(val::atom(this, &GeneralValidator::efficiency_check_foreign_key));
}

//-----------------------------------------------------------------------------
void GeneralValidator::add_consistency_validations(val::ChainsSet *cs)
{
  val::Chain<db_ColumnRef>::Ptr ccolumn = val::Chain<db_ColumnRef>::chain(cs);
  ccolumn->add_check(val::atom(this, &GeneralValidator::consistency_check_column));
}

//-----------------------------------------------------------------------------
void GeneralValidator::add_logic_validations(val::ChainsSet *cs)
{
  val::Chain<db_ForeignKeyRef>::Ptr    cfk = val::Chain<db_ForeignKeyRef>::chain(cs);
  val::Chain<db_TableRef>::Ptr      ctable = val::Chain<db_TableRef>::chain(cs);
  
     cfk->add_check(val::atom(this, &GeneralValidator::logic_check_fk_refers_primary_key));
  ctable->add_check(val::atom(this, &GeneralValidator::logic_check_table));
}

//-----------------------------------------------------------------------------
void GeneralValidator::add_empty_validations(val::ChainsSet *cs)
{
  val::Chain<db_TableRef>::Ptr     ctable = val::Chain<db_TableRef>::chain(cs);
  val::Chain<db_ViewRef>::Ptr       cview = val::Chain<db_ViewRef>::chain(cs);
  val::Chain<db_UserRef>::Ptr       cuser = val::Chain<db_UserRef>::chain(cs);
  val::Chain<db_RoleRef>::Ptr       crole = val::Chain<db_RoleRef>::chain(cs);
  val::Chain<db_RoutineGroupRef>::Ptr crg = val::Chain<db_RoutineGroupRef>::chain(cs);
  val::Chain<db_RoutineRef>::Ptr       cr = val::Chain<db_RoutineRef>::chain(cs);

  ctable->add_check(val::atom(this, &GeneralValidator::empty_check_table));
   cview->add_check(val::atom(this, &GeneralValidator::empty_check_view));
   cuser->add_check(val::atom(this, &GeneralValidator::empty_check_user));
   crole->add_check(val::atom(this, &GeneralValidator::empty_check_role));
     crg->add_check(val::atom(this, &GeneralValidator::empty_check_routine_group));
      cr->add_check(val::atom(this, &GeneralValidator::empty_check_routine));
}

//-----------------------------------------------------------------------------
void GeneralValidator::add_dup_validations(val::ChainsSet *cs)
{
  val::Chain<db_TableRef>::chain(cs)->add_check(val::atom(this, &GeneralValidator::dup_check_table_name));
  val::Chain<db_ViewRef>::chain(cs)->add_check(val::atom(this, &GeneralValidator::dup_check_view_name));
  val::Chain<db_ColumnRef>::chain(cs)->add_check(val::atom(this, &GeneralValidator::dup_check_column_name));
  val::Chain<db_IndexRef>::chain(cs)->add_check(val::atom(this, &GeneralValidator::dup_check_index_name));
  val::Chain<db_TriggerRef>::chain(cs)->add_check(val::atom(this, &GeneralValidator::dup_check_trigger_name));
  val::Chain<db_RoutineRef>::chain(cs)->add_check(val::atom(this, &GeneralValidator::dup_check_routine_name));
  val::Chain<db_RoutineGroupRef>::chain(cs)->add_check(val::atom(this, &GeneralValidator::dup_check_routine_group_name));
  val::Chain<db_RoleRef>::chain(cs)->add_check(val::atom(this, &GeneralValidator::dup_check_role_name));
  val::Chain<db_UserRef>::chain(cs)->add_check(val::atom(this, &GeneralValidator::dup_check_user_name));
}

static bool isDuplicated(Names* nm, const GrtNamedObjectRef& object)
{
  bool ret = false;
  const std::string name = object->name().c_str();
  Names::const_iterator item = nm->find(name);
  if ( item == nm->end() )
    nm->insert(std::pair<Names::key_type, Names::mapped_type>(name, object));
  else
    ret = true;
  return ret;
}

//-----------------------------------------------------------------------------
void GeneralValidator::dup_check_table_name(const db_TableRef& table) const
{
  if ( isDuplicated(&tableNames_, table) )
    resultsList->add_error("Duplicated table name '%s'", table->name().c_str());
}

//-----------------------------------------------------------------------------
void GeneralValidator::dup_check_view_name(const db_ViewRef& view) const
{
  if ( isDuplicated(&viewNames_, view) )
    resultsList->add_error("Duplicated view name '%s'", view->name().c_str());
}

//-----------------------------------------------------------------------------
void GeneralValidator::dup_check_role_name(const db_RoleRef& role) const
{
  if ( isDuplicated(&roleNames_, role) )
    resultsList->add_error("Duplicated role name '%s'", role->name().c_str());
}

//-----------------------------------------------------------------------------
void GeneralValidator::dup_check_user_name(const db_UserRef& user) const
{
  if ( isDuplicated(&userNames_, user) )
    resultsList->add_error("Duplicated user name '%s'", user->name().c_str());
}

//-----------------------------------------------------------------------------
void GeneralValidator::dup_check_routine_name(const db_RoutineRef& routine) const
{
  if ( isDuplicated(&routineNames_, routine) )
    resultsList->add_error("Duplicated routine name '%s'", routine->name().c_str());
}

//-----------------------------------------------------------------------------
void GeneralValidator::dup_check_routine_group_name(const db_RoutineGroupRef& rg) const
{
  if ( isDuplicated(&routineGroupNames_, rg) )
    resultsList->add_error("Duplicated routine group name '%s'", rg->name().c_str());
}

//-----------------------------------------------------------------------------
void GeneralValidator::dup_check_column_name(const db_ColumnRef& col) const
{
  if ( isDuplicated(&columnNames_, col) )
    resultsList->add_error("Duplicated column name '%s' in table '%s'", 
                          col->name().c_str(),
                          col->owner()->name().c_str()
                         );
}

//-----------------------------------------------------------------------------
void GeneralValidator::dup_check_index_name(const db_IndexRef& idx) const
{
  if ( isDuplicated(&indexNames_, idx) )
    resultsList->add_error("Duplicated index name '%s' in table '%s'", 
                          idx->name().c_str(),
                          idx->owner()->name().c_str()
                         );
}

//-----------------------------------------------------------------------------
void GeneralValidator::dup_check_trigger_name(const db_TriggerRef& trigger) const
{
  if ( isDuplicated(&triggerNames_, trigger) )
    resultsList->add_error("Duplicated trigger name '%s' in table '%s'", 
                          trigger->name().c_str(),
                          trigger->owner()->name().c_str()
                         );
}

//-----------------------------------------------------------------------------
void GeneralValidator::efficiency_check_table(const db_TableRef& table) const
{
  if (!table->primaryKey().is_valid())
  {
    resultsList->add_warning("Table '%s' has no primary key", 
                            table->name().c_str()
                           );
  }
  
  if (table->primaryKey().is_valid() && table->primaryKey()->columns().count() > 0 )
  {
    const grt::ListRef<db_IndexColumn> keys = table->primaryKey()->columns();
    grt::ListRef<db_IndexColumn>::const_iterator col = keys.begin();
    const grt::ListRef<db_IndexColumn>::const_iterator last = keys.end();

    for (; last != col; col++)
    {
      const db_ColumnRef         column = (*col)->referencedColumn();
      const db_SimpleDatatypeRef type   = column->simpleType();

      // TODO verify this is correct
      if (!(type.is_valid() 
        && type->numericPrecision() != EMPTY_TYPE_PRECISION
        && (0 == type->numericScale() || type->numericScale() == EMPTY_TYPE_SCALE))
         )
      {
        resultsList->add_warning("Primary Key for table '%s' is not integer based (field '%s')", 
                               table->name().c_str(),
                               column->name().c_str()
                             );
      }
    }
  }
  
  
  if ( table->columns().count() )
  {
    const grt::ListRef<db_Column> cols = table->columns();
    grt::ListRef<db_Column>::const_iterator col = cols.begin();
    const grt::ListRef<db_Column>::const_iterator last = cols.end();

    for ( ; last != col; ++col )
    {
      if ( !(*col)->simpleType().is_valid() )
      {
        resultsList->add_warning("Column '%s' in table '%s' has no valid type defined", 
                                (*col)->name().c_str(),
                                table->name().c_str()
                               );
      }
    }
  }
}

//-----------------------------------------------------------------------------
void GeneralValidator::efficiency_check_foreign_key(const db_ForeignKeyRef& fk) const
{
  const db_TableRef table = db_TableRef::cast_from(fk->owner());
  const grt::ListRef<db_Column> refCols = fk->referencedColumns();
  const grt::ListRef<db_Column> cols = fk->columns();

  grt::ListRef<db_Column>::const_iterator       rc_it  = refCols.begin();
  const grt::ListRef<db_Column>::const_iterator rlast  = refCols.end();

  grt::ListRef<db_Column>::const_iterator       c_it  = cols.begin();
  const grt::ListRef<db_Column>::const_iterator last  = cols.end();

  for (; rlast != rc_it || last != c_it; ++rc_it, ++c_it )
  {
    const bool rc_it_valid = (*rc_it).is_valid();
    const bool c_it_valid = (*c_it).is_valid();

    if ( c_it_valid && rc_it_valid )
    {
      if(bec::ColumnHelper::compare_column_types(*rc_it, *c_it) != COLUMNS_TYPES_EQUAL)
      {
        resultsList->add_error(
            "Foreign key '%s' in table '%s' has inconsistent type. Type of referring field '%s' differs from referred '%s'", 
            fk->name().c_str(),
            fk->owner()->name().c_str(),
            (*c_it)->name().c_str(),
            (*rc_it)->name().c_str()
            );
      }
    }
    else
    {
      if ( !c_it_valid && rc_it_valid )
      {
        resultsList->add_error("Foreign key '%s'.'%s' referencing column not defined.",
                              table->name().c_str(),
                              fk->name().c_str());
      }
      else if ( c_it_valid && !rc_it_valid )
      {
        resultsList->add_error("Foreign key '%s'.'%s' references nothing.",
                              table->name().c_str(),
                              fk->name().c_str());
      }
      else
      {}
    }
  }
}

static inline void concatFlags(std::string& to, const std::string& sep, const grt::StringRef& ap)
{
  to.append(ap);
  to.append(sep);
}

void GeneralValidator::consistency_check_column(const db_ColumnRef& column) const
{
  const Names::const_iterator col = consistencyColumns.find(column->name().c_str());

  if ( consistencyColumns.end() != col )
  {
    const db_ColumnRef col2 = db_ColumnRef::cast_from(col->second);
   
    if ( bec::ColumnHelper::compare_column_types(column, col2) != COLUMNS_TYPES_EQUAL )
    {
      const std::string colType  = bec::ColumnHelper::format_column_type(column, 0, true);
      const std::string col2Type = bec::ColumnHelper::format_column_type(col2, 0, true);
      static const std::string sep = "";      

      std::string attr_col;
      std::string attr_col2;
      forEach(column->flags(), boost::bind(&concatFlags, boost::ref(attr_col), sep, _1));
      forEach(col2->flags(),   boost::bind(&concatFlags, boost::ref(attr_col2), sep, _1));
      
      resultsList->add_warning("Column '%s'.'%s' <%s %s> "
                              "has different type than the column '%s'.'%s' <%s %s>", 
                              column->owner()->name().c_str(),
                              column->name().c_str(),
                              colType.c_str(), attr_col.c_str(),
                              col2->owner()->name().c_str(),
                              column->name().c_str(),
                              col2Type.c_str(), attr_col2.c_str()
                             );
      
    }
  }
  else
  {
    consistencyColumns.insert(std::pair<Names::key_type, 
                                        Names::mapped_type
                                       >
                                       (column->name().c_str()
                                       ,column
                                       )
                             );
  }
}

static grt::StringRef getNameFromIndexColumn(const db_IndexColumnRef& ic)
{
  return ic->referencedColumn()->name();
}

//-----------------------------------------------------------------------------
void GeneralValidator::logic_check_fk_refers_primary_key(const db_ForeignKeyRef& fk) const
{
  const grt::ListRef<db_Column> refCols = fk->referencedColumns();
  const db_TableRef refTable = fk->referencedTable();

  grt::ListRef<db_Column>::const_iterator       rcit   = refCols.begin();
  const grt::ListRef<db_Column>::const_iterator rclast = refCols.end();

  bool refColIsPK = false;
  if ( refTable->primaryKey().is_valid() )
  {
    for (; rclast != rcit; ++rcit )
    {
        if ( refColIsPK = is_name_in_list(refTable->primaryKey()->columns(), (*rcit)->name(), &getNameFromIndexColumn))
          break;
    }
  }
  
  if ( !refColIsPK )
  {
    resultsList->add_warning("Foreign key '%s'.'%s' references non-primary key column in table '%s'",
                            fk->owner()->name().c_str(),
                            fk->name().c_str(),
                            refTable->name().c_str()
                           );
  }
}



//-----------------------------------------------------------------------------
static void validateTableWritePrivilege(const db_TableRef         &table,
                                        bool                   *writeable,
                                        const db_RolePrivilegeRef  &privilege
                                       )
{
  static const grt::StringRef INSERT = "INSERT";
  static const grt::StringRef ALL    = "ALL";

   if (privilege->databaseObject().is_valid() 
        && privilege->databaseObject().id() == table.id())
  {      
    const size_t count= privilege->privileges().count();
    for(size_t i = 0; i < count; i++)
    {
      const grt::StringRef priv = privilege->privileges().get(i);
      if ( priv.is_valid() && ( priv == INSERT || priv == ALL ))
        *writeable = true;
    }
  }
}

//-----------------------------------------------------------------------------
static void validateTableWriteAccess(const db_TableRef &table, 
                                     bool           *writeable,
                                     const db_RoleRef  &role
                                     )
{
  forEach(role->privileges(), boost::bind(&validateTableWritePrivilege, table, writeable, _1));
}

//-----------------------------------------------------------------------------
void GeneralValidator::logic_check_table(const db_TableRef& table) const
{
  const grt::ListRef<db_Role> roles = cat_->roles();
  if ( roles.count() > 0 )
  {
    bool writeable = false;
    forEach(roles, boost::bind(&validateTableWriteAccess, table, &writeable, _1));

    if ( !writeable )
    {
      resultsList->add_warning("No role has INSERT privilege for table '%s'", 
                               table->name().c_str()
                             );
    }
  }

  if (table->isStub())
  {
    resultsList->add_warning("Table '%s' seems to be a Stub (e.g. created during Reverse Engineering)", 
                            table->name().c_str()
                           );
  }
}

//-----------------------------------------------------------------------------
void GeneralValidator::empty_check_table(const db_TableRef& table) const
{
  empty_check_object_is_referenced_by_role("Table", table, cat_);

  if ( !table->columns().is_valid() || table->columns().count() == 0 )
    resultsList->add_error("Table '%s' has no columns defined", table->name().c_str());

  empty_check_object_is_used_by_view(table, phys_model_);
}

//-----------------------------------------------------------------------------
void GeneralValidator::empty_check_view(const db_ViewRef& view) const
{
  empty_check_object_is_referenced_by_role("View", view, cat_);
  static const grt::StringRef empty = "";
  if ( !view->sqlDefinition().is_valid() || view->sqlDefinition() == empty )
    resultsList->add_error("View '%s' has no expression defined", view->name().c_str());

  empty_check_object_is_used_by_view(view, phys_model_);
}

//-----------------------------------------------------------------------------
void GeneralValidator::empty_check_user(const db_UserRef& user) const
{
  grt::ListRef<db_Role> roles = cat_->roles();  
  if ( roles.is_valid() && roles.count() )
  {
    roles = user->roles();
    if ( !roles.is_valid() || roles.count() == 0 )
    {
      resultsList->add_warning("User '%s' has no roles defined", 
                              user->name().c_str()
                             );
    }
  }
}

//-----------------------------------------------------------------------------
void GeneralValidator::empty_check_routine_group(const db_RoutineGroupRef& rg) const
{
  if ( !rg->routines().is_valid() || rg->routines().count() == 0 )
  {
    resultsList->add_warning("Routine group '%s' has no routines", 
                            rg->name().c_str()
                           );
  }

  empty_check_object_is_used_by_view(rg, phys_model_);
}

static void findRoutineInGroups(const db_RoutineRef& r, const db_RoutineGroupRef& rg, bool *found)
{
  if ( is_name_in_list(rg->routines(), r->name(), &defaultExtractName) )
    *found = true;
}

static bool checkRoutineIsInAGroup(const db_RoutineRef& r)
{
  const db_SchemaRef scm = db_SchemaRef::cast_from(r->owner());
  bool ret = false;
  
  forEach(scm->routineGroups(), boost::bind(&findRoutineInGroups, r, _1, &ret));
  
  return ret;
}

//-----------------------------------------------------------------------------
void GeneralValidator::empty_check_routine(const db_RoutineRef& routine) const
{
  empty_check_object_is_referenced_by_role("Routine", routine, cat_);

  if (!routine->sqlDefinition().is_valid() || routine->sqlDefinition().empty())
  {
    resultsList->add_warning("Routine '%s' has no SQL code", 
                            routine->name().c_str()
                           );
  }
  
  if (!checkRoutineIsInAGroup(routine))
  {
    resultsList->add_warning("Routine '%s' does not belong to any routine group", 
                            routine->name().c_str()
                           );
  }
}

//-----------------------------------------------------------------------------
void GeneralValidator::empty_check_role(const db_RoleRef& role) const
{
  const grt::ListRef<db_RolePrivilege> role_privileges = role->privileges();

  const size_t count = role_privileges.count();
  if (!role_privileges.is_valid() || count == 0 )
  {
    resultsList->add_warning("Role '%s' has no privileges defined", 
                             role->name().c_str()
                           );
  }
  else
  {  
    for(size_t i = 0; i < count; i++)
    {
      const db_RolePrivilegeRef privilege= role_privileges.get(i);
      
      if ( !privilege->privileges().is_valid() || privilege->privileges().count() == 0)
      {
        resultsList->add_error("Object '%s' has no privileges assigned in role '%s'", 
                              privilege->databaseObject()->name().c_str(), 
                              role->name().c_str()
                             );

      }
      
    }
  }
}

//-----------------------------------------------------------------------------
static void validateRolePrivelesHasNoObject(const grt::StringRef  &objectName, 
                                                  bool              *found,
                                            const db_RolePrivilegeRef  &privilege
                                           )
{
  if (privilege->databaseObjectName() == objectName 
      || (privilege->databaseObject().is_valid() && privilege->databaseObject()->name() == objectName ))
  {    
    *found = true;
  }
}

//-----------------------------------------------------------------------------
static void validateRoleHasNoObject(const grt::StringRef  &objectName, 
                                          bool              *found,
                                    const db_RoleRef           &role
                                   )
{
  forEach(role->privileges(), boost::bind(&validateRolePrivelesHasNoObject, objectName, found, _1));
}

//-----------------------------------------------------------------------------
void GeneralValidator::empty_check_object_is_referenced_by_role(const char* type, 
                                                          const GrtNamedObjectRef& obj, 
                                                          const db_CatalogRef& catalog
                                                         ) const
{
  const grt::ListRef<db_Role> roles = catalog->roles();
  if ( roles.count() > 0 )
  {
    bool found = false;
    forEach(roles, boost::bind(&validateRoleHasNoObject, obj->name(), &found, _1));

    if ( !found )
    {
      resultsList->add_warning(
            "%s '%s' is not referenced by any role", 
            type,
            obj->name().c_str());
    }
  }
}

//-----------------------------------------------------------------------------
template <typename T>
inline void figureWalk(const T& object, bool *found, const model_FigureRef& figure)
{
  if ( object->name() == figure->name() )
    *found = true;
}

//-----------------------------------------------------------------------------
template <>
inline void figureWalk<db_TableRef>(const db_TableRef& object, bool *found, const model_FigureRef& figure)
{
  if ( workbench_physical_TableFigureRef::can_wrap(figure) )
  {
    const db_TableRef table = (workbench_physical_TableFigureRef::cast_from(figure))->table();
    if ( table.is_valid() && table->name().is_valid() && ( table->name() == object->name() ) )
      *found = true;
  }
}

//-----------------------------------------------------------------------------
template <>
inline void figureWalk<db_ViewRef>(const db_ViewRef& object, bool *found, const model_FigureRef& figure)
{
  if ( workbench_physical_ViewFigureRef::can_wrap(figure) )
  {
    const db_ViewRef view = (workbench_physical_ViewFigureRef::cast_from(figure))->view();
    if ( view.is_valid() && view->name().is_valid() && ( view->name() == object->name() ) )
      *found = true;
  }
}

//-----------------------------------------------------------------------------
template <>
inline void figureWalk<db_RoutineGroupRef>(const db_RoutineGroupRef& object, bool *found, const model_FigureRef& figure)
{
  if ( workbench_physical_RoutineGroupFigureRef::can_wrap(figure) )
  {
    const db_RoutineGroupRef rg = (workbench_physical_RoutineGroupFigureRef::cast_from(figure))->routineGroup();
    if ( rg.is_valid() && rg->name().is_valid() && ( rg->name() == object->name() ) )
      *found = true;
  }
}

//-----------------------------------------------------------------------------
template <typename T>
inline void modelWalk(const T& object, bool *found, const workbench_physical_DiagramRef& view)
{
  forEach(view->figures(), boost::bind(&figureWalk<T>, object, found, _1));
}

//-----------------------------------------------------------------------------
template <typename T>
void GeneralValidator::empty_check_object_is_used_by_view(const T                        &object, 
                                                    const workbench_physical_ModelRef &model
                                                   ) const
{
  const grt::ListRef<workbench_physical_Diagram> views = model->diagrams();
  if ( views.count() > 0 )
  {
    bool found = false;
    forEach(views, boost::bind(&modelWalk<T>, object, &found, _1));
    if ( !found )
    {
      resultsList->add_warning("Object '%s' is not placed on any diagram", object->name().c_str());
    }
  }
}                                                   



std::string WbModuleValidationImpl::getValidationDescription(GrtObjectRef root)
{
  if (workbench_physical_ModelRef::can_wrap(root))
    root= workbench_physical_ModelRef::cast_from(root)->catalog();

  if (db_CatalogRef::can_wrap(root))
    return "General validations";

  return "";
}



#define INIT_SUMMARY(routine)\
  get_grt()->make_output_visible();

//// All validations
int WbModuleValidationImpl::validateAll(GrtObjectRef root)
{
  INIT_SUMMARY(validateAll);
  
  ResultsList rl(this);
  GeneralValidator gv(&rl, this);
  val::ChainsSet cs;
  gv.add_empty_validations(&cs);
  gv.add_efficiency_validations(&cs);
  gv.add_consistency_validations(&cs);
  gv.add_logic_validations(&cs);
  gv.add_dup_validations(&cs);
  
  if ( workbench_physical_ModelRef::can_wrap(root) )
  {
    db_CatalogRef cat(db_CatalogRef::cast_from(((workbench_physical_ModelRef::cast_from(root))->catalog())));
    gv.validate(cat, &cs);
  }
  else if ( db_mysql_CatalogRef::can_wrap(root) )
  {
    db_CatalogRef cat(db_CatalogRef::cast_from(root));
    gv.validate(cat, &cs);
  }

  #ifdef DEBUG
  rl.print(); 
  #endif 
  //const int ret = iter.iterate(*this, root, false, false);
  return rl.errors_count() + rl.warnings_count();
}

//// Empty Content Validation
int WbModuleValidationImpl::validateEmptyContent(GrtObjectRef root)
{
  INIT_SUMMARY(validateEmptyContent);

  ResultsList rl(this);
  GeneralValidator gv(&rl, this);
  val::ChainsSet cs;
  gv.add_empty_validations(&cs);
  
  if ( workbench_physical_ModelRef::can_wrap(root) )
  {
    db_CatalogRef cat(db_CatalogRef::cast_from(((workbench_physical_ModelRef::cast_from(root))->catalog())));
    dprint("validateEmptyContent: doing validation for workbench_physical_Model\n");
    gv.validate(cat, &cs);
  }
  else if ( db_mysql_CatalogRef::can_wrap(root) )
  {
    db_CatalogRef cat(db_CatalogRef::cast_from(root));
    dprint("validateEmptyContent: doing validation for db_Catalog\n");
    gv.validate(cat, &cs);
  }

  #ifdef DEBUG
  rl.print(); 
  #endif 
  //const int ret = iter.iterate(*this, root, false, false);
  return rl.errors_count() + rl.warnings_count();
}

// Table Efficiency Validation
int WbModuleValidationImpl::validateTableEfficiency(GrtObjectRef root)
{
  INIT_SUMMARY(validateTableEfficiency);

  ResultsList rl(this);
  GeneralValidator gv(&rl, this);
  val::ChainsSet cs;
  gv.add_efficiency_validations(&cs);
  
  if ( workbench_physical_ModelRef::can_wrap(root) )
  {
    db_CatalogRef cat(db_CatalogRef::cast_from(((workbench_physical_ModelRef::cast_from(root))->catalog())));
    gv.validate(cat, &cs);
  }
  else if ( db_mysql_CatalogRef::can_wrap(root) )
  {
    db_CatalogRef cat(db_CatalogRef::cast_from(root));
    gv.validate(cat, &cs);
  }

  #ifdef DEBUG
  rl.print();
  #endif
  //iter.iterate(*this, root, false, false);
  return rl.errors_count() + rl.warnings_count();
}

// Duplicated Identifiers Validation
int WbModuleValidationImpl::validateDuplicateIdentifiers(GrtObjectRef root)
{
  INIT_SUMMARY(validateDuplicateIdentifiers);

  ResultsList rl(this);
  GeneralValidator gv(&rl, this);
  val::ChainsSet cs;
  gv.add_dup_validations(&cs);
  
  if ( workbench_physical_ModelRef::can_wrap(root) )
  {
    db_CatalogRef cat(db_CatalogRef::cast_from(((workbench_physical_ModelRef::cast_from(root))->catalog())));
    gv.validate(cat, &cs);
  }
  else if ( db_mysql_CatalogRef::can_wrap(root) )
  {
    db_CatalogRef cat(db_CatalogRef::cast_from(root));
    gv.validate(cat, &cs);
  }

  //return iter.iterate(*this, root, false, false);
  return rl.errors_count() + rl.warnings_count();
}

// Consistency Validation
int WbModuleValidationImpl::validateConsistency(GrtObjectRef root)
{
  INIT_SUMMARY(validateConsistency);

  ResultsList rl(this);
  GeneralValidator gv(&rl, this);
  val::ChainsSet cs;
  gv.add_consistency_validations(&cs);
  
  if ( workbench_physical_ModelRef::can_wrap(root) )
  {
    db_CatalogRef cat(db_CatalogRef::cast_from(((workbench_physical_ModelRef::cast_from(root))->catalog())));
    gv.validate(cat, &cs);
  }
  else if ( db_mysql_CatalogRef::can_wrap(root) )
  {
    db_CatalogRef cat(db_CatalogRef::cast_from(root));
    gv.validate(cat, &cs);
  }

  //iter.iterate(*this, root, false, false);
  return rl.errors_count() + rl.warnings_count();
}

// Logic Validation
int WbModuleValidationImpl::validateLogic(GrtObjectRef root)
{
  INIT_SUMMARY(validateLogic);

  ResultsList rl(this);
  GeneralValidator gv(&rl, this);
  val::ChainsSet cs;
  gv.add_logic_validations(&cs);

  if ( workbench_physical_ModelRef::can_wrap(root) )
  {
    db_CatalogRef cat(db_CatalogRef::cast_from(((workbench_physical_ModelRef::cast_from(root))->catalog())));
    gv.validate(cat, &cs);
  }
  else if ( db_mysql_CatalogRef::can_wrap(root) )
  {
    db_CatalogRef cat(db_CatalogRef::cast_from(root));
    gv.validate(cat, &cs);
  }

  //iter.iterate(*this, root, false, false);
  return rl.errors_count() + rl.warnings_count();
}

GRT_MODULE_ENTRY_POINT(WbModuleValidationImpl);
