#include "stdafx.h"
#include "mysql_relationship_editor.h"


using namespace bec;


RelationshipEditorBE::RelationshipEditorBE(GRTManager *grtm, const workbench_physical_ConnectionRef &relationship)
: BaseEditor(grtm, relationship), _relationship(relationship)
{
}


void RelationshipEditorBE::set_caption(const std::string &caption)
{
  AutoUndoEdit undo(this, _relationship, "caption");
  _relationship->caption(caption);
  undo.end(_("Change Relationship Caption"));
}


std::string RelationshipEditorBE::get_caption()
{
  return _relationship->caption();
}


void RelationshipEditorBE::set_extra_caption(const std::string &caption)
{
  AutoUndoEdit undo(this, _relationship, "extraCaption");
  _relationship->extraCaption(caption);
  undo.end(_("Change Relationship 2nd Caption"));
}


std::string RelationshipEditorBE::get_extra_caption()
{
  return _relationship->extraCaption();
}


void RelationshipEditorBE::set_mandatory(bool flag)
{
  AutoUndoEdit undo(this);
  _relationship->foreignKey()->mandatory(flag?1:0);
  undo.end(_("Change Mandatory"));
}


bool RelationshipEditorBE::get_mandatory()
{
  return _relationship->foreignKey()->mandatory() != 0;
}


void RelationshipEditorBE::set_ref_mandatory(bool flag)
{
  AutoUndoEdit undo(this);
  _relationship->foreignKey()->referencedMandatory(flag?1:0);
  undo.end(_("Change Referred Mandatory"));
}


bool RelationshipEditorBE::get_ref_mandatory()
{
  return _relationship->foreignKey()->referencedMandatory() != 0;
}


void RelationshipEditorBE::set_to_many(bool flag)
{
  AutoUndoEdit undo(this);
  _relationship->foreignKey()->many(flag?1:0);
  undo.end(_("Change Relationship Cardinality"));
}


bool RelationshipEditorBE::get_to_many()
{
  return _relationship->foreignKey()->many() != 0;
}


void RelationshipEditorBE::set_comment(const std::string &comment)
{
  AutoUndoEdit undo(this, _relationship, "comment");
  _relationship->comment(comment);
  undo.end(_("Change Relationship Comment"));
}


std::string RelationshipEditorBE::get_comment()
{
  return _relationship->comment();
}


bool RelationshipEditorBE::get_visible()
{
  return _relationship->visible() != 0;
}

bool RelationshipEditorBE::get_splitted()
{
  return _relationship->drawSplit() != 0;
}

void RelationshipEditorBE::set_visible(bool flag)
{
  AutoUndoEdit undo(this);
  _relationship->visible(flag?1:0);
  undo.end(_("Change Relationship Visibility"));
}


void RelationshipEditorBE::set_splitted(bool flag)
{
  AutoUndoEdit undo(this);
  _relationship->drawSplit(flag?1:0);
  undo.end(_("Change Relationship Visibility"));
}
