#include "wb_config.h"
#include "linux_utilities/plugin_editor_base.h"
#include "../backend/mysql_relationship_editor.h"
#include "linux_utilities/MGGladeXML.h"
#include "grtdb/db_object_helpers.h"
#include "treemodel_wrapper.h"

class MGGladeXML;

//==============================================================================
//
//==============================================================================
class DbMySQLRelationshipEditor : public PluginEditorBase
{
  RelationshipEditorBE               _be;
  MGGladeXML                        *_xml;
  
  virtual bec::BaseEditor *get_be();
  
  std::vector<Gtk::Widget*>          _widgets;
  Gtk::RadioButton	            *_card_1_to_n;
  Gtk::RadioButton	            *_card_1_to_1;

  Gtk::CheckButton              *_integrity_referencing_mandatory;
  Gtk::CheckButton              *_integrity_referenced_mandatory;
  
  Gtk::RadioButton              *_visibility_fully;
  Gtk::RadioButton              *_visibility_split;
  Gtk::RadioButton              *_visibility_hidden;

  void toggled_1_to_1();
  void toggled_1_to_n();

  void toggled_fully();
  void toggled_split();
  void toggled_hidden();
  
  void toggled_ref_mandatory();
  void toggled_mandatory();
  
  bool                          _refreshing;
 public:
  DbMySQLRelationshipEditor(bec::GRTManager *grtm, const grt::BaseListRef &args);
  
  virtual ~DbMySQLRelationshipEditor();
  virtual void do_refresh_form_data();
  virtual std::string get_title();
};

DbMySQLRelationshipEditor::DbMySQLRelationshipEditor(bec::GRTManager *grtm, const grt::BaseListRef &args)
    : PluginEditorBase(grtm, args)
    , _be(grtm, workbench_physical_ConnectionRef::cast_from(args[0]))
    , _refreshing(false)
{
  _xml= new MGGladeXML(grtm->get_data_file_path("modules/data/editor_relationship.glade"));

  Gtk::Notebook *editor_window(0);
  _xml->get("mysql_relationship_editor", &editor_window);

  _be.set_refresh_ui_slot(sigc::mem_fun(this, &DbMySQLRelationshipEditor::refresh_form_data));

  editor_window->reparent(*this);                                                                                                       
  editor_window->show(); 
  
  Gtk::Entry* entry(0);
  _xml->get("conn_name", &entry);
  add_entry_change_timer(entry, sigc::mem_fun(&_be, &RelationshipEditorBE::set_caption));

  entry = 0;
  _xml->get("conn_name2", &entry);
  add_entry_change_timer(entry, sigc::mem_fun(&_be, &RelationshipEditorBE::set_extra_caption));
  
  Gtk::TextView* tv(0);
  _xml->get("conn_comments", &tv);
  add_text_change_timer(tv, sigc::mem_fun(&_be, &RelationshipEditorBE::set_comment));
  
  Gtk::Frame *frame(0);
  // Construct Cardinality
  _xml->get("frame1", &frame);
  Gtk::HBox *box = new Gtk::HBox();
  frame->add(*box);
  _widgets.push_back(box);
  
  _card_1_to_n = new Gtk::RadioButton("One-to-Many Relationship (1:n)");
  _widgets.push_back(_card_1_to_n);
  box->add(*_card_1_to_n);
  
  _card_1_to_1 = new Gtk::RadioButton("One-to-One Relationship (1:1)");
  Gtk::RadioButtonGroup grp = _card_1_to_n->get_group();
  _card_1_to_1->set_group(grp);
  _widgets.push_back(_card_1_to_1);
  box->add(*_card_1_to_1);

  
  //Construct Integrity
  _xml->get("frame2", &frame);
  box = new Gtk::HBox();
  _widgets.push_back(box);
  frame->add(*box);

  _integrity_referencing_mandatory = new Gtk::CheckButton("Referencing Table Mandatory");
  box->add(*_integrity_referencing_mandatory);
  _widgets.push_back(_integrity_referencing_mandatory);

  _integrity_referenced_mandatory = new Gtk::CheckButton("Referenced Table Mandatory");
  box->add(*_integrity_referenced_mandatory);
  _widgets.push_back(_integrity_referenced_mandatory);

  // Construct visibility
  _xml->get("frame3", &frame);
  box = new Gtk::HBox();
  frame->add(*box);
  _widgets.push_back(box);

  _visibility_fully = new Gtk::RadioButton("Fully Visible");
  grp = _visibility_fully->get_group();
  _widgets.push_back(_visibility_fully);
  box->add(*_visibility_fully);

  _visibility_split = new Gtk::RadioButton("Draw Split");
  _visibility_split->set_group(grp);
  _widgets.push_back(_visibility_split);
  box->add(*_visibility_split);

  _visibility_hidden  = new Gtk::RadioButton("Hide");
  _visibility_hidden->set_group(grp);
  _widgets.push_back(_visibility_hidden);
  box->add(*_visibility_hidden);

  refresh_form_data();
  show_all();

  _card_1_to_1->signal_toggled().connect(sigc::mem_fun(this, &DbMySQLRelationshipEditor::toggled_1_to_1));
  _card_1_to_n->signal_toggled().connect(sigc::mem_fun(this, &DbMySQLRelationshipEditor::toggled_1_to_n));

  _integrity_referencing_mandatory->signal_toggled().connect(sigc::mem_fun(this, &DbMySQLRelationshipEditor::toggled_mandatory));
  _integrity_referenced_mandatory->signal_toggled().connect(sigc::mem_fun(this, &DbMySQLRelationshipEditor::toggled_ref_mandatory));


  _visibility_fully->signal_toggled().connect(sigc::mem_fun(this, &DbMySQLRelationshipEditor::toggled_fully));
  _visibility_split->signal_toggled().connect(sigc::mem_fun(this, &DbMySQLRelationshipEditor::toggled_split));
  _visibility_hidden->signal_toggled().connect(sigc::mem_fun(this, &DbMySQLRelationshipEditor::toggled_hidden));
}

//------------------------------------------------------------------------------
DbMySQLRelationshipEditor::~DbMySQLRelationshipEditor()
{
  delete _xml;
  for ( int i = _widgets.size() - 1; i >= 0; --i )
    delete _widgets[i];
  _widgets.clear();
}

//------------------------------------------------------------------------------
void DbMySQLRelationshipEditor::toggled_1_to_1()
{
  if ( !_refreshing && _card_1_to_1->get_active() )
    _be.set_to_many(false);
  else
    _be.set_to_many(true);
}

//------------------------------------------------------------------------------
void DbMySQLRelationshipEditor::toggled_1_to_n()
{
  if ( !_refreshing && _card_1_to_n->get_active() )
    _be.set_to_many(true);
  else
    _be.set_to_many(false);
}

//------------------------------------------------------------------------------
void DbMySQLRelationshipEditor::toggled_mandatory()
{
  if ( !_refreshing )
    _be.set_mandatory(_integrity_referencing_mandatory->get_active());
}

//------------------------------------------------------------------------------
void DbMySQLRelationshipEditor::toggled_ref_mandatory()
{
  if ( !_refreshing )
    _be.set_ref_mandatory(_integrity_referenced_mandatory->get_active());
}

//------------------------------------------------------------------------------
void DbMySQLRelationshipEditor::toggled_fully()
{
  if ( !_refreshing && _visibility_fully->get_active() )                                                                                  
  {                                                    
    _be.set_visible(true);                                                                                       
    _be.set_splitted(false);                                                                                     
  }
}

//------------------------------------------------------------------------------
void DbMySQLRelationshipEditor::toggled_split()
{
  if ( !_refreshing && _visibility_split->get_active() )                                                                                  
  {                                                    
    _be.set_visible(true);                                                                                       
    _be.set_splitted(true);                                                                                     
  }
}

//------------------------------------------------------------------------------
void DbMySQLRelationshipEditor::toggled_hidden()
{
  if ( !_refreshing && _visibility_hidden->get_active() )                                                                                  
  {                                                    
    _be.set_visible(false);                                                                                       
    _be.set_splitted(false);                                                                                     
  }
}

//------------------------------------------------------------------------------
std::string DbMySQLRelationshipEditor::get_title()
{
  return _be.get_caption();
}

//------------------------------------------------------------------------------
bec::BaseEditor *DbMySQLRelationshipEditor::get_be()
{
  return &_be;
}

//------------------------------------------------------------------------------
void DbMySQLRelationshipEditor::do_refresh_form_data()
{
  _refreshing = true;
  
  Gtk::Entry* entry(0);
  _xml->get("conn_name", &entry);
  entry->set_text(_be.get_caption());

  _xml->get("conn_name2", &entry);
  entry->set_text(_be.get_extra_caption());
  
  Gtk::TextView* tv(0);
  _xml->get("conn_comments", &tv);
  tv->get_buffer()->set_text(_be.get_comment());
  
  if ( _be.get_to_many() )
    _card_1_to_n->set_active(true);
  else
    _card_1_to_1->set_active(true);

  _integrity_referencing_mandatory->set_active(_be.get_mandatory());
  _integrity_referenced_mandatory->set_active(_be.get_ref_mandatory());

  if ( _be.get_visible() )
  {
    if ( _be.get_splitted() )
      _visibility_split->set_active(true);                       
    else
      _visibility_fully->set_active(true);                                       
  }
  else
    _visibility_hidden->set_active(true);

  _refreshing = false;
}

//------------------------------------------------------------------------------
extern "C" 
{
  GUIPluginBase *createDbMysqlRelationshipEditor(bec::GRTManager *grtm, const grt::BaseListRef &args)
  {
    return Gtk::manage(new DbMySQLRelationshipEditor(grtm, args));
  }
};
