#include "wb_config.h"
#include "linux_utilities/plugin_editor_base.h"
#include "../backend/mysql_routine_editor.h"
#include "linux_utilities/MGGladeXML.h"
#include "grtdb/db_object_helpers.h"
#include "treemodel_wrapper.h"
#include "sql_editor_fe.h"

class MGGladeXML;

//==============================================================================
//
//==============================================================================
class DbMySQLRoutineEditor : public PluginEditorBase
{
  MySQLRoutineEditorBE               _be;
  MGGladeXML                        *_xml;
  
  virtual bec::BaseEditor *get_be();
  
 public:
  DbMySQLRoutineEditor(bec::GRTManager *grtm, const grt::BaseListRef &args);
  
  virtual ~DbMySQLRoutineEditor();
  virtual void do_refresh_form_data();
  virtual std::string get_title();
  
  SqlEditorFE           _sql_editor;
};

DbMySQLRoutineEditor::DbMySQLRoutineEditor(bec::GRTManager *grtm, const grt::BaseListRef &args)
    : PluginEditorBase(grtm, args), _be(grtm, db_mysql_RoutineRef::cast_from(args[0]), get_rdbms_for_db_object(args[0]))
{
  _xml= new MGGladeXML(grtm->get_data_file_path("modules/data/editor_routine.glade"));

  Gtk::Notebook *editor_window(0);
  _xml->get("mysql_routine_editor_notebook", &editor_window);

  Gtk::Image *image;
  _xml->get("routine_editor_image", &image);
  image->set(ImageCache::get_instance()->image_from_filename("db.Routine.editor.48x48.png", false));

  _be.set_refresh_ui_slot(sigc::mem_fun(this, &DbMySQLRoutineEditor::refresh_form_data));
  
  editor_window->reparent(*this);
  editor_window->show();

  Gtk::VBox *ddl_win;
  _xml->get("routine_ddl", &ddl_win);
  _sql_editor.widget().set_size_request(-1, 100);
  ddl_win->add(_sql_editor.widget());
  add_sqleditor_text_change_timer(&_sql_editor, sigc::mem_fun(_be, &MySQLRoutineEditorBE::set_sql));
  ddl_win->resize_children();

  
  refresh_form_data();
  
  std::string ddl        = _be.get_sql_definition_header();
  std::string routineSQL = _be.get_sql();
  
  if ( 0 < routineSQL.length() )
  {
    ddl += routineSQL + "//\n";
    if ( _be.has_syntax_error() )
    {
    }
    else
    {
    }
    
  }
  else
  {
    ddl += "CREATE PROCEDURE `" + _be.get_schema_name() + "`.`" + _be.get_name() + "` ()\nBEGIN\n  ";
    ddl += "\nEND//\n";
  }
  
  _sql_editor.set_text(ddl);  

  show_all();
}

//------------------------------------------------------------------------------
DbMySQLRoutineEditor::~DbMySQLRoutineEditor()
{
  delete _xml;
}

//------------------------------------------------------------------------------
std::string DbMySQLRoutineEditor::get_title()
{
  return strfmt(_("Routine: %s"), _be.get_name().c_str());
}

//------------------------------------------------------------------------------
bec::BaseEditor *DbMySQLRoutineEditor::get_be()
{
  return &_be;
}

//------------------------------------------------------------------------------
void DbMySQLRoutineEditor::do_refresh_form_data()
{
  Gtk::Entry* entry(0);
  _xml->get("routine_name", &entry);
  if (entry->get_text() != _be.get_name())
  {
    entry->set_text(_be.get_name());
    _signal_title_changed.emit(get_title());
  }
}

//------------------------------------------------------------------------------
extern "C" 
{
  GUIPluginBase *createDbMysqlRoutineEditor(bec::GRTManager *grtm, const grt::BaseListRef &args)
  {
    return Gtk::manage(new DbMySQLRoutineEditor(grtm, args));
  }
};
