#include "mysql_table_editor_column_flags.h"
#include "../backend/mysql_table_editor.h"

//------------------------------------------------------------------------------
ColumnFlags::ColumnFlags(Gtk::TreeView* tv)
                       : _columns()
                       , _model(Gtk::ListStore::create(_columns))
{
  tv->append_column_editable("S", _columns.toggle);
  tv->append_column("F", _columns.name);

  Gtk::CellRendererToggle *cell = (Gtk::CellRendererToggle*)(tv->get_column_cell_renderer(0));
  cell->property_activatable() = true;
  
  _conn = cell->signal_toggled().connect(sigc::mem_fun(*this
                                                       ,&ColumnFlags::after_cell_toggle
                                                      )
                                        );

}

//------------------------------------------------------------------------------
ColumnFlags::~ColumnFlags()
{
  _conn.disconnect();
}

//------------------------------------------------------------------------------
void ColumnFlags::after_cell_toggle(const Glib::ustring &path_string)
{
  Gtk::ListStore::Row row = *(_model->get_iter(path_string));

  // Invert stored flag
  const bool         flag_state  = !row[_columns.toggle];
  const std::string  name        = row[_columns.name];

  row[_columns.toggle]    = flag_state;

  _columns_be->set_column_flag(_node, name, flag_state);
}

//------------------------------------------------------------------------------
void ColumnFlags::fill(const ::bec::NodeId& node, MySQLTableColumnsListBE* columns_be, Gtk::TreeView* tv)
{
  _node       = node;
  _columns_be = columns_be;
  // Get the list of flags
  std::vector<std::string> flags(_columns_be->get_datatype_flags(node));
  
  std::vector<std::string>::const_iterator it   = flags.begin();
  std::vector<std::string>::const_iterator last = flags.end();

  _model->clear();
  
  for ( ; last != it; ++it )
  {
    bool is_checked = _columns_be->get_column_flag(node, *it);

    Gtk::TreeModel::Row row = *(_model->append());
    row[_columns.toggle]    = is_checked;
    row[_columns.name]      = *it;
  }
  
  tv->set_model(_model);
}
