#include "gtk/gtk.h"
#include "mysql_table_editor_fe.h"
//#include "../backend/mysql_table_editor.h"
#include "linux_utilities/MGGladeXML.h"
#include "grtdb/db_object_helpers.h"
#include "treemodel_wrapper.h"
#include "gtk_helpers.h"

#include "mysql_table_editor_column_page.h"
#include "mysql_table_editor_column_flags.h"

#include "menu_manager.h"

AutoCompletable DbMySQLTableEditorColumnPage::_completion;

//------------------------------------------------------------------------------
DbMySQLTableEditorColumnPage::DbMySQLTableEditorColumnPage(DbMySQLTableEditor *owner
                                                          ,MySQLTableEditorBE *be
                                                          ,MGGladeXML         *xml)
                             : _owner(owner)
                             , _be(be)
                             , _xml(xml)
{
  _tv = _xml->get("table_columns", &_tv);
  _tv->set_enable_tree_lines(true);
  _tv->get_selection()->set_mode(Gtk::SELECTION_MULTIPLE);
  
  _model = ListModelWrapper::create(_be->get_columns(), _tv, "DbMySQLTableEditorColumnPage");

  _model->model().append_string_column(MySQLTableColumnsListBE::Name, "Column Name", EDITABLE, WITH_ICON);
  _model->model().append_combo_column(MySQLTableColumnsListBE::Type, "Datatype", _model->model().create_model(get_types_for_table(_be->table())), EDITABLE);
  _model->model().append_check_column(MySQLTableColumnsListBE::IsNotNull, "NN", EDITABLE);
  _model->model().append_check_column(MySQLTableColumnsListBE::IsAutoIncrement, "AI", EDITABLE);
  _model->model().append_string_column(MySQLTableColumnsListBE::Default, "Default", EDITABLE);
  
  std::vector<Gtk::CellRenderer*> rends= _tv->get_column(0)->get_cell_renderers();
  rends[1]->signal_editing_started().
    connect(sigc::mem_fun(this, &DbMySQLTableEditorColumnPage::cell_editing_started));
  GtkCellRenderer* rend = _tv->get_column_cell_renderer(1)->gobj();
  g_signal_connect(rend, "editing-started", GCallback(&DbMySQLTableEditorColumnPage::type_cell_editing_started), this);
  
  _tv->set_model(_model);
  
  _tv->signal_row_activated().connect(sigc::mem_fun(*this, &DbMySQLTableEditorColumnPage::row_activated));
  _tv->signal_event().connect(sigc::mem_fun(*this, &DbMySQLTableEditorColumnPage::process_event));
  _tv->signal_cursor_changed().connect(sigc::mem_fun(*this, &DbMySQLTableEditorColumnPage::cursor_changed));

  Gtk::TreeView *flags_view;
  _xml->get("column_flags", &flags_view);
  
  _column_flags = new ColumnFlags(flags_view);

  Gtk::TextView *column_comment;
  _xml->get("column_comment", &column_comment);
  _owner->add_text_change_timer(column_comment, sigc::mem_fun(this, &DbMySQLTableEditorColumnPage::set_comment));

  Gtk::ComboBoxText *combo = 0;//new Gtk::ComboBoxText();
  _xml->get("column_collation_combo", &combo);
  combo->clear_items();
  fill_combo_from_string_list(combo, _be->get_charset_collation_list());

  _tv->signal_size_allocate().connect(sigc::mem_fun(this, &DbMySQLTableEditorColumnPage::check_resize));
  _tv->signal_visibility_notify_event().connect(sigc::mem_fun(this, &DbMySQLTableEditorColumnPage::do_on_visible));
}

//------------------------------------------------------------------------------
DbMySQLTableEditorColumnPage::~DbMySQLTableEditorColumnPage()
{
  delete _column_flags;
}

//------------------------------------------------------------------------------
void DbMySQLTableEditorColumnPage::check_resize(Gtk::Allocation& r)
{
  //   0       1        2  3         4
  // name    type      NN AI      Default
  const int step = r.get_width() / 10;
  _tv->get_column(0)->set_min_width(4 * step);
  _tv->get_column(1)->set_min_width(2 * step);
  _tv->get_column(4)->set_min_width(2 * step);
}

//------------------------------------------------------------------------------
bool DbMySQLTableEditorColumnPage::process_event(GdkEvent* event)
{
  if ( event->type == GDK_BUTTON_PRESS && event->button.button == 3 )
  {
    Gtk::TreeModel::Path   path;
    Gtk::TreeView::Column *column(0);
    int                    cell_x(-1);
    int                    cell_y(-1);
    
    if ( _tv->get_path_at_pos((int)event->button.x, (int)event->button.y, path, column, cell_x, cell_y) )
    {
      bec::NodeId node = _model->get_node_for_path(path);

      bec::MenuItemList menuitems;
      _be->get_columns()->get_node_popup_items(node, menuitems);
      MenuManager::run_popup(menuitems
                            ,event->button.x
                            ,event->button.y
                            ,event->button.time
                            ,sigc::mem_fun(this, &DbMySQLTableEditorColumnPage::menu_action_on_node)
                            );
    }
  }
  else if ( event->type == GDK_2BUTTON_PRESS && event->button.button == 1 )
  {
    Gtk::TreeModel::Path   path;
    Gtk::TreeView::Column *column(0);
    int                    cell_x(-1);
    int                    cell_y(-1);
    
    if ( _tv->get_path_at_pos((int)event->button.x, (int)event->button.y, path, column, cell_x, cell_y) )
    {
      bec::NodeId node = _model->get_node_for_path(path);
      if ( node.is_valid() )
      {
         bool primary_key = _be->get_columns()->get_column_flag(node, "PRIMARY KEY");
        _be->get_columns()->set_column_flag(node, "PRIMARY KEY", !primary_key);
      }
    }
  }
  else if ( event->type == GDK_KEY_RELEASE )
  {
    type_column_event(event);
  }
  return false;
}

//------------------------------------------------------------------------------
void DbMySQLTableEditorColumnPage::type_cell_editing_started(GtkCellRenderer* cr, GtkCellEditable* ce, gchar* path, gpointer udata)
{
  if ( GTK_IS_COMBO_BOX_ENTRY(ce) )
  {
    //g_signal_connect(ce, "event", GCallback(&DbMySQLTableEditorColumnPage::type_col_event), udata);
    GtkBin   *combo = GTK_BIN(ce);
    //GtkEntry *entry = (GtkEntry*)gtk_bin_get_child(combo);
    Gtk::Entry *entry = Glib::wrap((GtkEntry*)gtk_bin_get_child(combo));
    if ( entry )
      _completion.add_to_entry(entry);
    //g_signal_connect(entry, "event", GCallback(&DbMySQLTableEditorColumnPage::type_col_event), udata);
  }
}

//------------------------------------------------------------------------------
//gboolean DbMySQLTableEditorColumnPage::type_col_event(GtkWidget* w, GdkEvent* e, gpointer udata)
//{
//    g_message("ENTRY_EVENT");
//  DbMySQLTableEditorColumnPage* editor = (DbMySQLTableEditorColumnPage*)udata;
//  editor->type_column_event(e);
//  return false;
//}

//------------------------------------------------------------------------------
void DbMySQLTableEditorColumnPage::type_column_event(GdkEvent* event)
{
  if ( event->type == GDK_KEY_RELEASE )
  {
    const int key = event->key.keyval;
    if ( key == GDK_Tab/* || key == GDK_Return */)
    {
      // Advance to the next column
      Gtk::TreeModel::Path    path;
      Gtk::TreeView::Column  *column(0);
      _tv->get_cursor(path, column);
      
      if ( column )
      {
        //   0       1        2  3         4
        // name    type      NN AI      Default
        Glib::ListHandle<Gtk::TreeView::Column*> columns = _tv->get_columns();
  
        Glib::ListHandle<Gtk::TreeView::Column*>::const_iterator it   = columns.begin();
        Glib::ListHandle<Gtk::TreeView::Column*>::const_iterator last = columns.end();
        
        int i = 0;
        for ( ; last != it; ++it )
        {
          if ( (*it)->get_title() == column->get_title() )
            break;
          ++i;
        }
        
        ++it; ++i;
        if ( it != last && i <= 1 )
          _tv->set_cursor(node2path(_node_edited), **it, true);
        else // next row
        {
          Gtk::TreePath path(node2path(_node_edited));
          path.next();
          _tv->set_cursor(path, **columns.begin(), true);
        }
      }
    }
  }
}

//------------------------------------------------------------------------------
void DbMySQLTableEditorColumnPage::delete_column(const Gtk::TreeModel::Path& path)
{
  bec::NodeId node = _model->get_node_for_path(path);
  _be->remove_column(node);
}

//------------------------------------------------------------------------------
void DbMySQLTableEditorColumnPage::menu_action_on_node(Gtk::MenuItem* item)
{
  const std::string item_name = item->get_name();
  
  if ( item_name == "deleteSelectedColumnsToolStripMenuItem" )
  {
    Glib::RefPtr<Gtk::TreeSelection> selection = _tv->get_selection();
    selection->selected_foreach_path(sigc::mem_fun(this, &DbMySQLTableEditorColumnPage::delete_column));
  }
  else if ( item_name == "moveUpToolStripMenuItem" )
  {
    _be->get_columns()->reorder_up(_node);
  }
  else if ( item_name == "moveDownToolStripMenuItem" )
  {
    _be->get_columns()->reorder_down(_node);
  }
  else if ( item_name == "refreshGridToolStripMenuItem" )
    refresh();
}

//------------------------------------------------------------------------------
void DbMySQLTableEditorColumnPage::row_activated(const Gtk::TreeModel::Path& path
                                                     , Gtk::TreeView::Column* column
                                                )
{
}

//------------------------------------------------------------------------------
void DbMySQLTableEditorColumnPage::cursor_changed()
{
  Gtk::TreeModel::Path   path;
  Gtk::TreeView::Column *column(0);
  
  _tv->get_cursor(path, column);
  
  bec::NodeId node = _model->get_node_for_path(path);
  
  if ( node.is_valid() )
  {
    _node = node;
    update_column_details(_node);
  }
}

//------------------------------------------------------------------------------
void DbMySQLTableEditorColumnPage::update_column_details(const ::bec::NodeId& node)
{
  Gtk::TreeView *flags_view;
  _xml->get("column_flags", &flags_view);
  
  _column_flags->fill(node, _be->get_columns(), flags_view);

  Gtk::TextView *column_comment;
  _xml->get("column_comment", &column_comment);
  
  std::string comment;
  _be->get_columns()->get_field(node, MySQLTableColumnsListBE::Comment, comment);
  
  column_comment->get_buffer()->set_text(comment);
  
  update_collation();
}

//------------------------------------------------------------------------------
void DbMySQLTableEditorColumnPage::set_comment(const std::string& comment)
{
  ::bec::TableColumnsListBE* cols = _be->get_columns();
  cols->set_field(_node, (int)MySQLTableColumnsListBE::Comment, comment);
}

//------------------------------------------------------------------------------
void DbMySQLTableEditorColumnPage::update_collation()
{
  std::string has_charset;
  MySQLTableColumnsListBE* columns = _be->get_columns();

  Gtk::ComboBoxText *collation_combo;
  _xml->get("column_collation_combo", &collation_combo);
  
  columns->get_field(_node, MySQLTableColumnsListBE::HasCharset, has_charset);
  if ( "1" == has_charset )
  {
    std::string column_cscoll;
    columns->get_field(_node, MySQLTableColumnsListBE::CharsetCollation, column_cscoll);
    
    collation_combo->set_sensitive(true);
    collation_combo->set_active_text(column_cscoll);
  }
  else
  {
    collation_combo->set_sensitive(false);
  }
}

//------------------------------------------------------------------------------
grt::StringListRef DbMySQLTableEditorColumnPage::get_types_for_table(const db_TableRef table)
{
  // TODO: List should be obtained dynamically from the ???????
  // TODO: obtained list should be sorted by number of the types used
  grt::StringListRef list(table->get_grt());
  
  list.insert("INT");
  list.insert("VARCHAR()");
  list.insert("DECIMAL()");
  list.insert("DATETIME");       
  list.insert("BLOB");
  list.insert("----------");
  list.insert("BIT");
  list.insert("TINYINT");
  list.insert("SMALLINT");
  list.insert("MEDIUMINT");
  list.insert("INT");
  list.insert("BIGINT");
  list.insert("FLOAT");
  list.insert("DOUBLE");
  list.insert("----------");
  list.insert("DATE");
  list.insert("TIME");
  list.insert("TIMESTAMP");
  list.insert("YEAR");
  list.insert("----------");
  list.insert("CHAR()");
  list.insert("VARCHAR()");
  list.insert("BINARY()");
  list.insert("TINYBLOB");
  list.insert("BLOB");
  list.insert("MEDIUMBLOB");
  list.insert("LONGBLOB");
  list.insert("TINYTEXT");
  list.insert("TEXT");
  list.insert("MEDIUMTEXT");
  list.insert("LONGTEXT");
  list.insert("ENUM()");
  list.insert("SET()");

  _completion.clear();
  const int count = list.count();
  for ( int i = 0; i < count; ++i )
    _completion.add_completion_text(list[i]);

  return list;
}

//--------------------------------------------------------------------------------
bool DbMySQLTableEditorColumnPage::do_on_visible(GdkEventVisibility*)
{
  MySQLTableColumnsListBE *columns = _be->get_columns();
  if ( columns->count() == 1 )
  {
    ::bec::NodeId node(columns->get_node(0));
    // Set name. That will indicate to cell_editing_started that we are editing name cell
    columns->get_field(bec::NodeId(), MySQLTableColumnsListBE::Name, _1st_column_name);
    _tv->set_cursor(node2path(node), *(_tv->get_column(0)), true);
  }
  return false;
}

//--------------------------------------------------------------------------------
void DbMySQLTableEditorColumnPage::cell_editing_done(bool needs_refresh)
{
  _cell_edit_conn.disconnect();
  
  std::string type;
  MySQLTableColumnsListBE *columns = _be->get_columns();
  columns->get_field(_node, MySQLTableColumnsListBE::Type, type);
    
  if ( type == "" )
  {
    // type == "" means that we edit placement "fake" row
    if ( columns->count() > 2 )
    {
      // We are at 2+ row
      ((bec::TableColumnsListBE*)columns)->set_field(_node, MySQLTableColumnsListBE::Type, std::string("VARCHAR(45)"));
    }
    else
    {
      // We are at the first row, so it is adding a very first column to the table
      // Let's get default type from BE
      columns->get_field(bec::NodeId(), MySQLTableColumnsListBE::Type, type);
      if ( type != "" )
        ((bec::TableColumnsListBE*)columns)->set_field(_node, MySQLTableColumnsListBE::Type, type);
      else
        ((bec::TableColumnsListBE*)columns)->set_field(_node, MySQLTableColumnsListBE::Type, std::string("INT"));
    }
  }
  
  if ( columns->count() == 2 )
  {
    columns->set_field(_node, MySQLTableColumnsListBE::IsNotNull, 1);
    columns->set_field(_node, MySQLTableColumnsListBE::IsAutoIncrement, 1);
    columns->set_field(_node, MySQLTableColumnsListBE::IsPK, 1);
  }

  if (needs_refresh)
  {
    // refresh only the row that's been edited and then the new placeholder if needed
    Gtk::TreeModel::Path    path;
    Gtk::TreeView::Column  *column(0);
    _tv->get_cursor(path, column);
    columns->refresh();
    _model->row_changed(path, _model->get_iter(path));
    if (path.back() == columns->count()-2)
    {
      path.next();
      _model->row_inserted(path, _model->get_iter(path));
    }
  }
}


void DbMySQLTableEditorColumnPage::cell_editing_started(Gtk::CellEditable *cell, const Glib::ustring &path)
{
  bool needs_refresh= false;
  _node_edited = ::bec::NodeId(path);

  // if this is the last row, it is the placeholder. so we will need a refresh once its edited
  if (!_model->get_be_model()->has_next(_model->get_node_for_path(Gtk::TreePath(path))))
  { 
    // If we have _1st_column_name not empty that means that we started to edit the name cell
    if ( !_1st_column_name.empty() )
    {
      Gtk::Entry* entry = static_cast<Gtk::Entry*>(cell);
      entry->set_text(_1st_column_name);
      _1st_column_name.clear();
    }
    needs_refresh= true;
  }

  _cell_edit_conn.disconnect();
  _cell_edit_conn = cell->signal_editing_done().
    connect(sigc::bind(sigc::mem_fun(this, &DbMySQLTableEditorColumnPage::cell_editing_done), needs_refresh));
}


//--------------------------------------------------------------------------------

void DbMySQLTableEditorColumnPage::refresh()
{   
  bec::ListModel* m = _model->get_be_model();
  _model->set_be_model(0);
  _tv->unset_model();

  _tv->set_model(_model);

  _model->set_be_model(m);
  _tv->unset_model();
  _model->refresh();
  _tv->set_model(_model);
}
