#ifndef __WB_COLUMN_PAGE_HANDLING_H__
#define __WB_COLUMN_PAGE_HANDLING_H__

#include <gtkmm.h>
#include "mysql_table_editor_column_flags.h"
#include "grt/tree_model.h"

class MySQLTableEditorBE;
class MGGladeXML;
class ListModelWrapper;
class DbMySQLTableEditor;
class AutoCompletable;

//==============================================================================
//
//==============================================================================
class DbMySQLTableEditorColumnPage
{
  public:
    DbMySQLTableEditorColumnPage(DbMySQLTableEditor *owner, MySQLTableEditorBE* be, MGGladeXML* xml);
	~DbMySQLTableEditorColumnPage();
	
    void refresh();
  
  private:
    grt::StringListRef get_types_for_table(const db_TableRef table);

    void row_activated(const Gtk::TreeModel::Path& path, Gtk::TreeView::Column* column);
    bool process_event(GdkEvent* event);
    void cursor_changed();

    void cell_editing_started(Gtk::CellEditable *cell, const Glib::ustring &path);
    static void type_cell_editing_started(GtkCellRenderer* cr, GtkCellEditable* ce, gchar* path, gpointer udata);
    //static gboolean type_col_event(GtkWidget* w, GdkEvent* e, gpointer udata);
    void type_column_event(GdkEvent* e);
    void cell_editing_done(bool refresh);

    void update_column_details(const ::bec::NodeId &node);
    
    void set_comment(const std::string& comment);
    void update_collation();
    
    void menu_action_on_node(Gtk::MenuItem* item);
    
    void delete_column(const Gtk::TreeModel::Path& path);
    
    void check_resize(Gtk::Allocation& r);
    bool do_on_visible(GdkEventVisibility*);
  private:
    DbMySQLTableEditor                       *_owner;
    MySQLTableEditorBE                       *_be;
    MGGladeXML                               *_xml;
    Gtk::TreeView                            *_tv;
    ColumnFlags                              *_column_flags;
    ::bec::NodeId                             _node;
    ::bec::NodeId                             _node_edited;

    Glib::RefPtr<ListModelWrapper>            _model;
    std::string                               _1st_column_name;
    sigc::connection                          _cell_edit_conn;
    static AutoCompletable                    _completion;
};

#endif
