

#include "linux_utilities/plugin_editor_base.h"
#include "../backend/mysql_schema_editor.h"
#include "linux_utilities/MGGladeXML.h"
#include "grtdb/db_object_helpers.h"

class SchemaEditor : public PluginEditorBase
{
  MySQLSchemaEditorBE _be;
  MGGladeXML *_xml;

  virtual bec::BaseEditor *get_be()
  {
    return &_be;
  }

public:
  SchemaEditor(bec::GRTManager *grtm, const grt::BaseListRef &args)
    : PluginEditorBase(grtm, args), _be(grtm, db_SchemaRef::cast_from(args[0]), get_rdbms_for_db_object(args[0]))
  {
    set_border_width(8);

    _xml= new MGGladeXML(grtm->get_data_file_path("modules/data/editor_schema.glade"));

    Gtk::Widget *widget;
    _xml->get("base_table", &widget);

    Gtk::Image *image;
    _xml->get("image", &image);
    
    //image->set(grtm->get_data_file_path(""));
    
    Gtk::Entry *entry;
    _xml->get("name_entry", &entry);
    add_entry_change_timer(entry, sigc::mem_fun(_be, &MySQLSchemaEditorBE::set_name));

    Gtk::ComboBoxText *combo;
    _xml->get("collation_combo", &combo);
    fill_combo_from_string_list(combo, _be.get_charset_collation_list());
    add_option_combo_change_handler(combo, "CHARACTER SET - COLLATION", sigc::mem_fun(_be, &MySQLSchemaEditorBE::set_schema_option_by_name));

    Gtk::TextView *tview;
    _xml->get("text_view", &tview);
    add_text_change_timer(tview, sigc::mem_fun(_be, &MySQLSchemaEditorBE::set_comment));

    
    
    widget->reparent(*this);

    show_all();
    
    refresh_form_data();
  }
  
  virtual ~SchemaEditor()
  {
    delete _xml;
  }

  virtual void do_refresh_form_data()
  {    
    Gtk::Entry *entry;
    _xml->get("name_entry", &entry);

    Gtk::TextView *tview;
    _xml->get("text_view", &tview);

    Gtk::ComboBoxText *combo;
    _xml->get("collation_combo", &combo);

    combo->set_active_text(_be.get_schema_option_by_name("CHARACTER SET - COLLATION"));

    entry->set_text(_be.get_name());

    tview->get_buffer()->set_text(_be.get_comment());
  }
  
  virtual std::string get_title()
  {
    return "Schema";
  }
};



extern "C" 
{
  GUIPluginBase *createDbMysqlSchemaEditor(bec::GRTManager *grtm, const grt::BaseListRef &args)
  {
    return Gtk::manage(new SchemaEditor(grtm, args));
  }
};
