#include "stdafx.h"



#include "grtpp.h"
#include "interfaces/plugin.h"

#include "grts/structs.db.mgmt.h"


#define MODULE_VERSION "1.0.0"


#ifdef _WIN32
# define FRONTEND_LIBNAME(obj, windows_dll, linux_so, osx_dylib)\
  obj->moduleName(windows_dll)
#elif defined(__APPLE__)
# define FRONTEND_LIBNAME(obj, windows_dll, linux_so, osx_dylib)\
  obj->moduleName(osx_dylib)
#else
# define FRONTEND_LIBNAME(obj, windows_dll, linux_so, osx_dylib)\
  obj->moduleName(linux_so)
#endif



static grt::ListRef<app_Plugin> get_mysql_plugins_info(grt::GRT *grt);


class MySQLDbModuleImpl : public grt::ModuleImplBase, public PluginInterfaceImpl
{
public:
  MySQLDbModuleImpl(grt::CPPModuleLoader *ldr)
  : grt::ModuleImplBase(ldr)
  {
  }

  DEFINE_INIT_MODULE(MODULE_VERSION, "MySQL AB", grt::ModuleImplBase,
    DECLARE_MODULE_FUNCTION(MySQLDbModuleImpl::getPluginInfo), NULL);

  virtual grt::ListRef<app_Plugin> getPluginInfo()
  {
    return get_mysql_plugins_info(get_grt());
  }
};



static void set_object_argument(app_PluginRef &plugin, const std::string &struct_name)
{
  app_PluginObjectInputRef pdef(plugin.get_grt());

  pdef->objectStructName(struct_name);
  pdef->owner(plugin);

  plugin->inputValues().insert(pdef);
}



static grt::ListRef<app_Plugin> get_mysql_plugins_info(grt::GRT *grt)
{
  grt::ListRef<app_Plugin> plugins(grt);
  app_PluginRef diff_sql_generator(grt);

  {
    app_PluginRef plugin(grt);
    
    FRONTEND_LIBNAME(plugin,
                     ".\\db.mysql.wbp.fe.dll",
                     "db.mysql.wbp.so",
                     "");
    plugin->pluginType("gui");
    plugin->moduleFunctionName("WBPluginSQLExport");
    plugin->name("db.mysql.plugin.export.sql");
    plugin->caption("Export MySQL SQL Script");
    plugin->groups().insert("database/Database");

    grt::StringListRef document_types(grt);
    document_types.insert("workbench.Document");
    //plugin->documentStructNames(document_types);

    app_PluginObjectInputRef pdef(grt);
    pdef->objectStructName("db.Catalog");
    plugin->inputValues().insert(pdef);

    plugins.insert(plugin);
  }

  {
    app_PluginRef plugin(grt);
    
    FRONTEND_LIBNAME(plugin,
                     ".\\db.mysql.wbp.fe.dll",
                     "db.mysql.wbp.so",
                     "");
    plugin->pluginType("gui");
    plugin->moduleFunctionName("WBPluginSQLExportALTER");
    plugin->name("db.mysql.plugin.export.alter");
    plugin->caption("Export MySQL ALTER Script");
    plugin->groups().insert("database/Database");

    grt::StringListRef document_types(grt);
    document_types.insert("workbench.Document");
    //plugin->documentStructNames(document_types);

    app_PluginObjectInputRef pdef(grt);
    pdef->objectStructName("db.Catalog");
    plugin->inputValues().insert(pdef);

    plugins.insert(plugin);
  }

  {
    app_PluginRef plugin(grt);
    
    FRONTEND_LIBNAME(plugin,
                     ".\\db.mysql.wbp.fe.dll",
                     "db.mysql.wbp.so",
                     "");
    plugin->pluginType("gui");
    plugin->moduleFunctionName("WBPluginSQLSynchronize");
    plugin->name("db.mysql.plugin.sync.catalog");
    plugin->caption("MySQL Script Synchronization");
    plugin->groups().insert("database/Database");

    grt::StringListRef document_types(grt);
    document_types.insert("workbench.Document");
    //plugin->documentStructNames(document_types);

    app_PluginObjectInputRef pdef(grt);
    pdef->objectStructName("db.Catalog");
    plugin->inputValues().insert(pdef);

    plugins.insert(plugin);
  }

  {
    app_PluginRef plugin(grt);
    
    FRONTEND_LIBNAME(plugin,
                     ".\\db.mysql.sync.wbp.fe.dll",
                     "db.mysql.wbp.so",
                     "");
    plugin->pluginType("gui");
    plugin->moduleFunctionName("WBPluginDbSynchronize");
    plugin->name("db.mysql.plugin.sync.db");
    plugin->caption("Synchronize with Database");
    plugin->groups().insert("database/Database");

    grt::StringListRef document_types(grt);
    document_types.insert("workbench.Document");
    //plugin->documentStructNames(document_types);

    app_PluginObjectInputRef pdef(grt);
    pdef->objectStructName("db.Catalog");
    plugin->inputValues().insert(pdef);

    plugins.insert(plugin);
  }

  return plugins;
}


GRT_MODULE_ENTRY_POINT(MySQLDbModuleImpl);
