
#include "grtui/grt_wizard_plugin.h"
#include "grtui/wizard_finished_page.h"
#include "grtui/wizard_view_text_page.h"

#include "grtui/grtdb_object_filter.h"

#include "db_mysql_sql_script_sync.h"


using namespace grtui;
using namespace mforms;


#include "db/frontend/catalog_validation_page.h"



namespace ScriptSynchronize {

  
class WbPluginSQLSynchronize : public WizardPlugin
{
  DbMySQLScriptSync _be;
  
public:
  WbPluginSQLSynchronize(bec::GRTManager *grtm, const grt::BaseListRef &args);

  DbMySQLScriptSync *get_be() { return &_be; }
};

  

class ExportInputPage : public WizardPage
{
public:
  ExportInputPage(WizardForm *form)
    : WizardPage(form, "options"), _options(true)
  {
    set_title(_("Synchronize Options"));
    set_subtitle(_("Choose paths and other options for model synchronization and generation of ALTER Script."));

    _options.set_title(_("Options"));

    _options.add(&_options_box);
    _options_box.set_padding(12);
    _options_box.set_row_spacing(8);
    
    _options_box.set_column_count(3);
    _options_box.set_row_count(4);

    _options_box.add(&_in_descr, 0, 3, 0, 1);
    _options_box.add(&_in_label, 0, 1, 1, 2, (TableItemFlags)0);
    _options_box.add(&_in_entry, 1, 2, 1, 2, (TableItemFlags)(HExpandFlag|HFillFlag));
    _options_box.add(&_in_button, 2, 3, 1, 2, (TableItemFlags)0);
    _in_descr.set_text(_("Pick the SQL script file to be compared with the current model."));
    _in_label.set_text(_("Input File:"));
    _in_button.set_text(_("Browse..."));
    enable_file_browsing(&_in_entry, &_in_button, mforms::OpenFile, "SQL Files (*.sql)|*.sql");
    _in_entry.signal_changed().connect(sigc::mem_fun(this, &ExportInputPage::validate));

    _options_box.add(&_out_descr, 0, 3, 2, 3);
    _options_box.add(&_out_label, 0, 1, 3, 4, (TableItemFlags)0);
    _options_box.add(&_out_entry, 1, 2, 3, 4, (TableItemFlags)(HExpandFlag|HFillFlag));
    _options_box.add(&_out_button, 2, 3, 3, 4, (TableItemFlags)0);
    _out_descr.set_text(_("Enter the path for the ALTER script to be created."));
    _out_label.set_text(_("Output File:"));
    _out_button.set_text(_("Browse..."));
    enable_file_browsing(&_out_entry, &_out_button, mforms::SaveFile, "SQL Files (*.sql)|*.sql");
    _out_entry.signal_changed().connect(sigc::mem_fun(this, &ExportInputPage::validate));

    add(&_options, false, false);

    
    _preview_check.set_text(_("Preview Generated Script"));
    add(&_preview_check, false, false);
    _preview_check.signal_toggled().connect(sigc::mem_fun(this, &WizardPage::validate));
    
    signal_leave().connect(sigc::mem_fun(this, &ExportInputPage::gather_options));
  }

  void gather_options(grt::DictRef values)
  {
    ((WbPluginSQLSynchronize*)_form)->get_be()->set_option("InputFileName1", _in_entry.get_string_value());
    
    values.gset("InputPath", _in_entry.get_string_value());
    values.gset("OutputPath", _out_entry.get_string_value());
    values.gset("Preview", _preview_check.get_active() ? 1 : 0);
  }

  virtual bool allow_next()
  {
    return !_in_entry.get_string_value().empty();
  }

protected:
  Panel _options;
  Table _options_box;

  Label _in_label;
  Label _in_descr;
  TextEntry _in_entry;
  Button _in_button;

  Label _out_label;
  Label _out_descr;
  TextEntry _out_entry;
  Button _out_button;
  
  CheckBox _preview_check;
};



class PreviewScriptPage : public ViewTextPage
{
public:
  PreviewScriptPage(WizardForm *form)
    : ViewTextPage(form)
  {
    set_title(_("SQL Script"));
    set_subtitle(_("Review and edit the generated SQL script."));
    
    set_editable(true);

    signal_leave().connect(sigc::mem_fun(this, &PreviewScriptPage::save_script));
  }


  virtual void execute()
  {
    ((WbPluginSQLSynchronize*)_form)->get_be()->init_diff_tree(std::vector<std::string>(), db_CatalogRef(), db_CatalogRef());

    set_text(((WbPluginSQLSynchronize*)_form)->get_be()->generate_diff_tree_script());

    if (!values().get_int("Preview"))
      _form->go_to_next();
  }


  void save_script(grt::DictRef values)
  {
    _form->grtm()->push_status_text(_("Updating model catalog..."));
    _form->grtm()->get_grt()->send_info(_("Updating model catalog..."));
    ((WbPluginSQLSynchronize*)_form)->get_be()->apply_changes_to_model();
    _form->grtm()->replace_status_text(_("Model updated."));

    std::string path= values.get_string("OutputPath");
    if (!path.empty())
    {
      _form->grtm()->replace_status_text(_("Saving script..."));
      save_text_to(path);

      _form->grtm()->replace_status_text(strfmt(_("Wrote ALTER Script to '%s'"), path.c_str()));
      _form->grtm()->get_grt()->send_info(strfmt(_("Wrote ALTER Script to '%s'"), path.c_str()));
    }
  }

  virtual bool next_closes_wizard() { return true; }
};



#include "synchronize_differences_page.h"


//--------------------------------------------------------------------------------
//--------------------------------------------------------------------------------


WbPluginSQLSynchronize::WbPluginSQLSynchronize(bec::GRTManager *grtm, const grt::BaseListRef &args)
    : WizardPlugin(grtm, args), _be(grtm)
{
  set_title(_("Script Synchronization - Generate an ALTER Script"));

  add_page(new CatalogValidationPage(this));
  add_page(new ExportInputPage(this));

  SynchronizeDifferencesPage *page= new SynchronizeDifferencesPage(this, &_be);
  page->set_title(_("Model and SQL Script Differences"));
  add_page(page);

  add_page(new PreviewScriptPage(this));
}


}



extern "C" 
{
  GUIPluginBase *createWBPluginSQLSynchronize(bec::GRTManager *grtm, const grt::BaseListRef &args)
  {
    return new ScriptSynchronize::WbPluginSQLSynchronize(grtm, args);
  }
};

