#include "stdafx.h"

#include "wb_config.h"

#include <grtpp.h>
#include <grtpp_module_cpp.h>

#include "interfaces/plugin.h"

#include "grts/structs.db.mgmt.h"

#define MODULE_VERSION "1.0.0"

static grt::ListRef<app_Plugin> get_database_plugins_info(grt::GRT *grt);


class DatabaseModuleImpl : public grt::ModuleImplBase, public PluginInterfaceImpl
{
public:
  DatabaseModuleImpl(grt::CPPModuleLoader *ldr) : grt::ModuleImplBase(ldr) {}

  DEFINE_INIT_MODULE(MODULE_VERSION, "MySQL AB", grt::ModuleImplBase,
    DECLARE_MODULE_FUNCTION(DatabaseModuleImpl::getPluginInfo), NULL);

  virtual grt::ListRef<app_Plugin> getPluginInfo()
  {
    return get_database_plugins_info(get_grt());
  }
};


#ifdef _WIN32
# define FRONTEND_LIBNAME(obj, windows_dll, linux_so, osx_dylib)\
  obj->moduleName(windows_dll)
#elif defined(__APPLE__)
# define FRONTEND_LIBNAME(obj, windows_dll, linux_so, osx_dylib)\
  obj->moduleName(osx_dylib)
#else
# define FRONTEND_LIBNAME(obj, windows_dll, linux_so, osx_dylib)\
  obj->moduleName(linux_so)
#endif


static grt::ListRef<app_Plugin> get_database_plugins_info(grt::GRT *grt)
{
  grt::ListRef<app_Plugin> plugins(grt);
  {
    app_PluginRef plugin(grt);
    
    FRONTEND_LIBNAME(plugin,
                     ".\\db.wbp.fe.dll",
                     "db.wbp.so",
                     "");
    plugin->pluginType("gui");
    plugin->moduleFunctionName("WBPluginSQLImport");
    plugin->name("db.mysql.plugin.import.sql");
    plugin->caption("Import from SQL Script");
    plugin->groups().insert("database/Database");

    grt::StringListRef document_types(grt);
    document_types.insert("workbench.Document");
    //plugin->documentStructNames(document_types);

    app_PluginObjectInputRef pdef(grt);
    pdef->objectStructName("db.Catalog");
    plugin->inputValues().insert(pdef);

    plugins.insert(plugin);
  }
  return plugins;
}

GRT_MODULE_ENTRY_POINT(DatabaseModuleImpl);
